package ec3.client;

import java.util.Vector;

import ec3.shared.*;

/**
 * Verwaltung der Away-Nachrichten sowie Userliste, Admin-Status und Farben. Muss mur dem
 * Standard-Plugin eingebunden werden, damit Away-Nachrichten nach Fehlern
 * wiederhergestellt werden knnen.
 * 
 * @since 0.4
 */
public class AwaySys implements ProtocolExtension {
	
	public boolean sockCommand (String command, String data, Properties properties, ImplCallBack callBack) {
		
		String[] dataSplit = Tools.split (data, Tools.CHR0);
		try {
			
			if (command.equals("acceptUser") && dataSplit[0].startsWith("ok")) {
				// test, ob Awaystatus wiederhergestellt werden muss
				if (properties.getProperty("client.AwaySys.error.away", "0").equals("1")) {
					properties.setProperty("client.AwaySys.away", "1");
					properties.setProperty("client.AwaySys.awayMessage", properties.getProperty("client.AwaySys.error.awayMessage"));
					// Fehlerstatus rcksetzen
					properties.setProperty("client.AwaySys.error.away", "0");
					properties.setProperty("client.AwaySys.error.awayMessage", "");
				}
				
				return false; // Standard behandelt weiter
			} else if (command.equals ("getAwayMessage")) {
				
				//Away-Modus aktiv?
				if (properties.getProperty ("client.AwaySys.away", "0")
				 .equals ("1")) {
					
					//senden
					callBack.sockCommand (
						"awaymessage",
						dataSplit[0] + Tools.CHR0
						 + properties.getProperty (
						 	"client.AwaySys.awayMessage",
						 	""
						 )
					);
					
				}//if
				
				return true;
				
			//Awaynachricht empfangen
			} else if (command.equals ("awaymessage")) {
				
				if (dataSplit.length < 2) {
					
					throw (new ArrayIndexOutOfBoundsException ());
					
				}
				callBack.frontEndCommand ("awayMessage", dataSplit);
				
				return true;
				
			//Chat-Teilnehmer hat seinen Status gendert
			} else if (command.equals ("newClientState")) {
				//User geht away
				if (dataSplit[1].equals ("away")) {
					
					//keine Nachricht mitgeschickt
					if (dataSplit.length == 2) {
						
						//senden
						callBack.frontEndCommand (
							"away",
							new String[] {dataSplit[0], ""}
						);
						
					} else {
						
						//mit Nachricht senden
						callBack.frontEndCommand (
							"away",
							new String[] {dataSplit[0], dataSplit[2]}
						);
						
					}
					
				//User ist nicht mehr away
				} else if (dataSplit[1].equals ("normal")) {
					
					//senden
					callBack.frontEndCommand (
						"notAway",
						new String[] {dataSplit[0]}
					);
					
				//User hat Admin-Status erhalten
				} else if (dataSplit[1].equals ("admin")) {
					
					//senden
					callBack.frontEndCommand (
						"admin",
						new String[] {dataSplit[0]}
					);
					
				//User hat Admin-Status verloren
				} else if (dataSplit[1].equals ("noadmin")) {
					
					//senden
					callBack.frontEndCommand (
						"notAdmin",
						new String[] {dataSplit[0]}
					);
					
				}//if
				
				return true;
				
			//Chat-Teilnehmer hat neue Farbe
			} else if (command.equals ("newColor")) {
				
				if (dataSplit.length < 2) {
					
					throw (new ArrayIndexOutOfBoundsException ());
					
				}
				
				//Event senden
				callBack.frontEndCommand ("newColor", dataSplit);
				
				return true;
				
			//neuer User im Chatraum
			} else if (command.equals ("userJoined")) {
				
				if (dataSplit[2].equals (
					properties.getProperty ("client.chatRoom", "")
				) == false) {
					
					//ignorieren, da falscher Chatraum
					
				} else if (dataSplit[0].equals (
					properties.getProperty ("client.chatName", "")
				)) {
					
					//ignorieren, da eigener Name
					
				} else {
					
					//Protokollversion unterscheiden
					if (properties.getProperty ("server.protoVersion", "0").equals ("0")) {
						
						//senden
						callBack.frontEndCommand (
							"userJoined",
							new String[] {dataSplit[0], dataSplit[1], "0", "0"}
						);
						
					} else {
						
						//senden
						callBack.frontEndCommand (
							"userJoined",
							new String[] {
								dataSplit[0],
								dataSplit[1],
								dataSplit[3],
								dataSplit[4]
							}
						);
						
					}//if
					
				}//if
				
				return true;
				
			//User hat Chatraum verlassen
			} else if (command.equals ("userLeft")) {
				
				if (dataSplit[1].equals (
					properties.getProperty ("client.chatRoom", "")
				) == false) {
					
					//ignorieren, da falscher Chatraum
					
				} else if (dataSplit[0].equals (
					properties.getProperty ("client.chatName", "")
				)) {
					
					//ignorieren, da eigener Name
					
				} else {
					
					//Protokollversion unterscheiden
					if (properties.getProperty ("server.protoVersion", "0").equals ("0") || (dataSplit.length < 4)) {
						
						//senden
						callBack.frontEndCommand (
							"userLeft",
							new String [] {dataSplit[0], dataSplit[2], ""}
						);
						
					} else {
						
						//senden
						callBack.frontEndCommand (
							"userLeft",
							new String [] {dataSplit[0], dataSplit[2], dataSplit[3]}
						);
						
					}//if
					
				}//if
				
				return true;
				
			//Userliste empfangen
			} else if (command.equals ("userlist")) {
				
				//berprfen, ob richtiger Chatraum
				if (dataSplit[0].equals (
					properties.getProperty ("client.chatRoom", "")
				)) {
					
					dataSplit = Tools.split (dataSplit[1], Tools.CRLF);
					//Dynamiche Liste erstellen
					Vector roomList = new Vector (dataSplit.length * 4);
					//Daten parsen
					for (int i = 0; i < dataSplit.length; i++) {
						
						String[] dataSplit2 = Tools.split (
							dataSplit[i],
							Tools.CHR1
						);
						
						//in Liste eintragen
						roomList.addElement (dataSplit2[0]);
						roomList.addElement (dataSplit2[1]);
						roomList.addElement (dataSplit2[2]);
						if (dataSplit2.length > 3) {
							
							roomList.addElement (dataSplit2[3]);
							
						} else {
							
							roomList.addElement ("0");
							
						}
						
					}//for
					
					//array basteln
					String[] temp = new String[roomList.size ()];
					roomList.copyInto (temp);
					//Liste absenden
					callBack.frontEndCommand ("userList", temp);
					
				}//if
				
				return true;
				
			}//if
			
		} catch (ArrayIndexOutOfBoundsException e) {
			
			callBack.frontEndCommand (
				"error",
				new String[] {"AwaySys/" + command + "/ArrayOutOfBounds"}
			);
			return true;
			
		}
		
		return false;
		
	}
	
	public boolean frontEndCommand (String command, String[] data, Properties properties, ImplCallBack callBack) {
		
		try {
			
			if (command.equals ("message")) {
				
				//Leerzeichen etc. entfernen
				String message = data[0].trim ();
				
				//wenn noch etwas zu senden da ist
				if (message.equals ("") == false) {
					
					//Befehl
					if (message.startsWith ("/")) {
						
						//splitten in Befehl und angehngte Daten
						String[] messageSplit = null;
						//Daten gefunden
						if (message.indexOf (" ") > -1) {
							
							messageSplit = new String[2];
							messageSplit[0] =
							 message.substring (0, message.indexOf (" ")).toLowerCase ();
							messageSplit[1] =
							 message.substring (message.indexOf (" ") + 1);
							
						//keine Daten gefunden
						} else {
							
							messageSplit = new String[2];
							messageSplit[0] = message;
							messageSplit[1] = "";
							
						}//if
						
						//Befehl auswerten
						//Away-Modus
						if (messageSplit[0].equals ("/away") || messageSplit[0].equals ("/a")) {
							
							//nur, wenn eingeloggt
							if (properties.getProperty ("client.connected", "0")
							 .equals ("1")
							 && properties.getProperty ("client.loggedIn", "0")
							 .equals ("1")) {
								
								//Daten vorhanden?
								if (!messageSplit[1].equals("")) {
									
									//ja, Away-Modus aktivieren
									//senden
									callBack.sockCommand (
										"setClientState",
										"away" + Tools.CHR0 + messageSplit[1]
									);
									
									//Event senden
									callBack.frontEndCommand (
										"setAway",
										new String[] {"1"}
									);
									
									//Flags setzen
									properties.setProperty (
										"client.AwaySys.away",
										"1"
									);
									
									//Nachricht merken
									properties.setProperty (
										"client.AwaySys.awayMessage",
										messageSplit[1]
									);
									
								} else {
									
									//nein, Away-Modus umschalten
									
									if (
										properties.getProperty (
											"client.AwaySys.away", "0"
										).equals ("1")
									) {
										
										//senden
										callBack.sockCommand (
											"setClientState",
											"normal" + Tools.CHR0
										);
										
										//Event senden
										callBack.frontEndCommand (
											"setAway",
											new String[] {"0"}
										);
										
										//Flags setzen
										properties.setProperty (
											"client.AwaySys.away",
											"0"
										);
										
									} else {
										
										//senden
										callBack.sockCommand (
											"setClientState",
											"away" + Tools.CHR0
										);
										
										//Event senden
										callBack.frontEndCommand (
											"setAway",
											new String[] {"1"}
										);
										
										//Flags setzen
										properties.setProperty (
											"client.AwaySys.away",
											"1"
										);
											
										//Nachricht merken
										properties.setProperty (
											"client.AwaySys.awayMessage",
											""
										);
										
									}
										
								}//if
								
							//nicht eingeloggt
							} else {
								
								//Hinweis senden
								callBack.frontEndCommand (
									"notice",
									new String[] {"notLoggedIn"}
								);
								
							}//if
							
							return true;
							
						} else if (messageSplit[0].equals ("/back") || messageSplit[0].equals ("/b")) {
							
							if (
								properties.getProperty (
									"client.AwaySys.away"
								).equals ("1")
							) {
								
								//senden
								callBack.sockCommand (
									"setClientState",
									"normal" + Tools.CHR0
								);
								
								//Event senden
								callBack.frontEndCommand (
									"setAway",
									new String[] {"0"}
								);
								
								//Flags setzen
								properties.setProperty (
									"client.AwaySys.away",
									"0"
								);
								
							}
							
							return true;
							
						} else if (messageSplit[0].equals ("/getawaymsg") || messageSplit[0].equals ("/gam")) {
							
							//nur, wenn eingeloggt
							if (properties.getProperty ("client.connected", "0")
							 .equals ("1")
							 && properties.getProperty ("client.loggedIn", "0")
							 .equals ("1")) {
								
								if (messageSplit[1].trim ().equals ("")) {
									
									//Hinweis senden
									callBack.frontEndCommand (
										"notice",
										new String[] {"syntaxGam"}
									);
									
								} else {
									
									callBack.sockCommand ("getAwayMessage", messageSplit[1].trim ());
									
								}
								
							//nicht eingeloggt
							} else {
								
								//Hinweis senden
								callBack.frontEndCommand (
									"notice",
									new String[] {"notLoggedIn"}
								);
								
							}//if
							
							return true;
							
						}
						
					}//if
					
				}//if
				
			} else if (command.equals ("getUserList")) {
				
				//nur, wenn in Chatraum
				if (properties.getProperty ("client.connected", "0").equals ("1")
				 && properties.getProperty ("client.loggedIn", "0").equals ("1")
				 && properties.getProperty ("client.inChatRoom", "0").equals ("1")){
					
					callBack.sockCommand (
						"getUserlist",
						properties.getProperty ("client.chatRoom", "")
					);
					
				}//if
				
				return true;
				
			} else if (command.equals ("setAway")) {
				
				//nur, wenn eingeloggt
				if (properties.getProperty ("client.connected", "0")
				 .equals ("1")
				 && properties.getProperty ("client.loggedIn", "0")
				 .equals ("1")) {
				 	
					if (data[0].equals("0")) { //Awaymodus beenden
						
						//senden
						callBack.sockCommand (
							"setClientState",
							"normal" + Tools.CHR0
						);
						
						//Event senden
						callBack.frontEndCommand ("setAway", new String[] {"0"});
						
						//Flags setzen
						properties.setProperty ("client.AwaySys.away", "0");
						
					} else { //Awaymodus setzen
						
						//senden
						callBack.sockCommand (
							"setClientState",
							"away" + Tools.CHR0 + data[1]
						);
						
						//Event senden
						callBack.frontEndCommand ("setAway", new String[] {"1"});
						
						//Flags setzen
						properties.setProperty ("client.AwaySys.away", "1");
						
						//Nachricht merken
						properties.setProperty (
							"client.AwaySys.awayMessage",
							data[1]
						);
						
					}
					
				} else {
					
					//Hinweis senden
					callBack.frontEndCommand (
						"notice",
						new String[] {"notLoggedIn"}
					);
					
				}
				return true;
				
			} else if (command.equals ("getAwayMessage")) {
				
				callBack.sockCommand ("getAwayMessage", data[0]);
				
				return true;
				
			} else if (command.equals ("setChatColor")) {
				
				//nur, wenn eingeloggt
				if (properties.getProperty ("client.connected", "0").equals ("1")
				 && properties.getProperty ("client.loggedIn", "0").equals ("1")) {
					
					//senden
					callBack.sockCommand ("setColor", data[0]);
					
				}//if
				
				return true;
				
			}
			
		} catch (ArrayIndexOutOfBoundsException e) {
			
			callBack.frontEndCommand (
				"error",
				new String[] {"AwaySys (FEC)/" + command + "/ArrayOutOfBounds"}
			);
			return true;
			
		}
		
		return false;
		
	}
	
	/* (non-Javadoc)
	 * @see ec3.client.ProtocolExtension#pluginCommand(java.lang.String, java.lang.String[], ec3.shared.Properties, ec3.client.ImplCallBack)
	 */
	public boolean pluginCommand (String command, String[] data, Properties properties, ImplCallBack callBack) {
		// leer
		return false;
	}
	
	/* (non-Javadoc)
	 * @see ec3.client.ProtocolExtension#connect(ec3.shared.Properties, ec3.client.ImplCallBack)
	 */
	public void connect (Properties properties, ImplCallBack callBack) {
		
		properties.setProperty ("client.AwaySys.away", "0");
		
	}
	
	/* (non-Javadoc)
	 * @see ec3.client.ProtocolExtension#disconnect(ec3.shared.Properties, ec3.client.ImplCallBack)
	 */
	public void disconnect (Properties properties, ImplCallBack callBack) {
		
		//properties.setProperty ("client.AwaySys.away", "0");
		
	}
	
	/* (non-Javadoc)
	 * @see ec3.client.ProtocolExtension#sockError(java.lang.String, ec3.shared.Properties, ec3.client.ImplCallBack)
	 */
	public void sockError(String data, Properties properties, ImplCallBack callBack) {
		
		if (properties.getProperty("client.AwaySys.away", "0").equals("1")) {
			properties.setProperty("client.AwaySys.error.away", "1");
			properties.setProperty("client.AwaySys.error.awayMessage", properties.getProperty("client.AwaySys.awayMessage"));
		}
		
	}
	
}