package ec3.client;

import ec3.shared.*;

/**
 * Ablehnung nicht in {@link Standard} implementierter Funktionen. Sollte nach allen
 * anderen Plugins geladen werden.
 *
 * - Dateitransfer
 * - Plugins
 * - Awaymitteilungen
 * 
 * @since 0.4
 */
public class Deny implements ProtocolExtension {
	
	public boolean sockCommand (
		String command,
		String data,
		Properties properties,
		ImplCallBack callBack
	) {
		
		String[] dataSplit = Tools.split (data, Tools.CHR0);
		try {
			
			// Login erfolgreich; kein Verschlsselungsplugin ist aktiv
			if (command.equals("acceptUser")) {
				
				if (dataSplit[0].startsWith ("ok")) {
					properties.remove("client.encryption");
					//Event senden
					callBack.frontEndCommand ("encReady", new String[] {"no"});
					callBack.frontEndCommand ("acceptUser", new String[] {});
				}
				
				return true;
				
			//Pluginbefehl von anderem Client erhalten
			} else if (command.equals ("clientPluginData")) {
				
				//dataSplit: geteilt in data und data2 => Plugindaten
				dataSplit = Tools.split (dataSplit[0], Tools.CRLF);
				//^ 0 -> Plugin-ID  1 -> Befehl   2 -> Absender
				
				//Antwort: Plugin nicht gefunden
				callBack.sockCommand (
					"pluginResponse",
					"pluginnotfound"
					 + Tools.CHR0
					 + dataSplit[0]
					 + Tools.CHR0
					 + dataSplit[2]
				);
				
				return true;
				
			// Verschlsselungsliste vom Server empfangen
			} else if (command.equals("encryptionlist")) {
				
				// hier hat kein Verschlsselungsplugin reagiert
				// -> weiter ohne Verschlsselung
				properties.remove("client.encryption");
				callBack.frontEndCommand ("encReady", new String[] {"notFound"});
				callBack.frontEndCommand ("acceptUser", new String[] {});
				
				return true;
				
			//Dateitransferaufforderng erhalten
			} else if (command.equals ("fileRequest")) {
				
				//Ablehnung senden
				callBack.sockCommand (
					"acceptFiles",
					dataSplit[0] + Tools.CHR0 + "deny"
				);
				
				return true;
				
			//Pluginbefehl vom Server erhalten
			} else if (command.equals ("serverPluginData")) {
				
				//dataSplit: geteilt in data und data2 => Plugindaten
				dataSplit = Tools.split (dataSplit[0], Tools.CRLF);
				//^ 0 -> Plugin-ID  1 -> Befehl
				
				//Antwort: Plugin nicht gefunden
				callBack.sockCommand (
					"pluginResponse",
					"pluginnotfound" + Tools.CHR0 + dataSplit[0] + Tools.CHR0
				);
				
				return true;
			
			//diverse Kommandos abfangen, die nicht in {@link Standard}
			// definiert sind, aber trotzdem auftreten knnen
			} else if (command.equals ("getAwayMessage")
				|| command.equals ("newClientState")
				|| command.equals ("newColor")
				|| command.equals ("userJoined")
				|| command.equals ("userLeft")
			) {
				
				//nichts tun
				
				return true;
				
			}
			
		} catch (ArrayIndexOutOfBoundsException e) {
			
			callBack.frontEndCommand (
				"error",
				new String[] {"Deny/" + command + "/ArrayOutOfBounds"}
			);
			return true;
			
		}
		
		return false;
		
	}
	
	public boolean frontEndCommand (String command, String[] data, Properties properties, ImplCallBack callBack) {
		
		// keine Verschlsselung fr Chatraumpasswort bei Raumerstellung
		if (command.equals("encryptionSupported")) {
			
			data[0] = "0";
			return true;
			
		//unbekannter Befehl
		} else if (command.equals ("message")) {
			
			//Leerzeichen etc. entfernen
			String message = data[0].trim ();
			
			if (message.startsWith ("/")) {
				
				//Hinweis senden
				callBack.frontEndCommand (
					"notice",
					new String[] {"unknownCommand", message}
				);
				
				return true;
				
			}
			
		}
		
		return false;
		
	}
	
	/* (non-Javadoc)
	 * @see ec3.client.ProtocolExtension#pluginCommand(java.lang.String, java.lang.String[], ec3.shared.Properties, ec3.client.ImplCallBack)
	 */
	public boolean pluginCommand (String command, String[] data, Properties properties, ImplCallBack callBack) {
		// leer
		return false;
	}
	
	/* (non-Javadoc)
	 * @see ec3.client.ProtocolExtension#connect(ec3.shared.Properties, ec3.client.ImplCallBack)
	 */
	public void connect (Properties properties, ImplCallBack callBack) {
		// nix
	}
	
	/* (non-Javadoc)
	 * @see ec3.client.ProtocolExtension#disconnect(ec3.shared.Properties, ec3.client.ImplCallBack)
	 */
	public void disconnect (Properties properties, ImplCallBack callBack) {
		// nix
	}

    /* (non-Javadoc)
     * @see ec3.client.ProtocolExtension#sockError(java.lang.String, ec3.shared.Properties, ec3.client.ImplCallBack)
     */
    public void sockError(String data, Properties properties, ImplCallBack callBack) {
        
        // leer
        
    }
	
}