package ec3.client;

import ec3.shared.CryptoHook;

/**
 * interface, das einer {@link ProtocolExtension} erlaubt, das Socket zu benutzen
 *
 * @since 0.4
 */
public interface ImplCallBack {
	
	/**
	 * ermglicht den {@link ProtocolExtension}'s den Zugriff auf das Socket-Objekt
	 *
	 * @param command zu sendender Befehl
	 * @param data zu sendende Daten
	 */
	public void sockCommand (String command, String data);
	
	/**
	 * ermglicht den {@link ProtocolExtension}'s den Zugriff auf das Socket-Objekt
	 *
	 * @param command zu sendender Befehl
	 * @param data zu sendende Daten
	 */
	public void sockCommand (String command, byte[] data);
	
	/**
	 * ermglicht den {@link ProtocolExtension}'s den Zugriff auf das frontEnd-Objekt
	 *
	 * @param command zu sendender Befehl
	 * @param data zu sendende Daten
	 */
	public void frontEndCommand (String command, String[] data);
	
	/**
	 * ermglicht den {@link ProtocolExtension}'s den Zugriff auf andere
	 * {@link ProtocolExtension}'s
	 * 
	 * @param command zu sendender Befehl
	 * @param data zu sendende Daten
	 * @return <code>true</code> wenn der Befehl bearbeitet wurde. 
	 */
	public boolean pluginCommand(String command, String[] data);
	
	/**
	 * erstellt eine Verbindung zum angegebenen Server
	 * 
	 * @param server der Server, zu dem verbunden werden soll
	 * @param port der Port, auf dem der Server luft
	 */
	public void connect (String server, int port);
	
	/**
	 * Erstellt eine Verbindung zum letzten bekannten Server
	 */
	public void connect ();
	
	/**
	 * trennt eine Verbindung
	 */
	public void disconnect ();
	
	/**
	 * Aktiviert die Verschlsselung mittels des angegebenen CryptoHook's.
	 * 
	 * @param hook die Verschlsselungsimplementation.
	 */
	public void enableEncryption(CryptoHook hook);
	
}