package ec3.client;

import ec3.shared.*;

/**
 * interface zum erweitern des Befehlssatzes des Protokollobjekts
 *
 * @since 0.4
 * @author Andreas Mann
 */
public interface ProtocolExtension {
	
	/**
	 * verarbeitet Nachrichtenpakete, die vom Socket empfangen wurden
	 *
	 * @param command der Befehlsteil der empfangenen Nachricht
	 * @param data der Datenteil der empfangenen Nachricht
	 * @param properties das Eigenschafts-Objekt der ClientImpl
	 * @param callBack Referenz auf das ClientImpl-Objekt selbst, um Nachrichten an das Socket zu senden
	 * @return true, wenn die Nachricht verarbeitet wurde, false sonst
	 */
	public boolean sockCommand (String command, String data, Properties properties, ImplCallBack callBack);
	
	/**
	 * Verarbeitet Fehlermeldungen des Sockets, nachdem diese an das Frontend
	 * gesendet wurden.
	 * 
	 * @param data die Fehlermeldung des Sockets
	 * @param properties das Eigenschafts-Objekt der ClientImpl
	 * @param callBack Referenz auf das ClientImpl-Objekt selbst, um Nachrichten an das Socket zu senden
	 */
	public void sockError(String data, Properties properties, ImplCallBack callBack);
	
	/**
	 * verarbeitet Nachrichten,	die	vom	User-Interface stammen
	 *
	 * @param command der Befehlsteil der Nachricht
	 * @param data der Datenteil der Nachricht
	 * @param properties das Eigenschafts-Objekt der ClientImpl
	 * @param callBack Referenz auf das ClientImpl-Objekt selbst, um Nachrichten an das Socket zu senden
	 * @return true, wenn die NAchricht verarbeitet wurde, false sonst
	 */
	public boolean frontEndCommand (String command, String[] data, Properties properties, ImplCallBack callBack);
	
	/**
	 * verarbeitet Nachrichten,	die	von anderen Plugins stammen
	 *
	 * @param command der Befehlsteil der Nachricht
	 * @param data der Datenteil der Nachricht
	 * @param properties das Eigenschafts-Objekt der ClientImpl
	 * @param callBack Referenz auf das ClientImpl-Objekt selbst, um Nachrichten an das Socket zu senden
	 * @return true, wenn die NAchricht verarbeitet wurde, false sonst
	 */
	public boolean pluginCommand (String command, String[] data, Properties properties, ImplCallBack callBack);
	
	/**
	 * aufgerufen, wenn eine Verbindung aufgebaut wird
	 *
	 * @param properties das Eigenschafts-Objekt der ClientImpl
	 * @param callBack Referenz auf das ClientImpl-Objekt selbst, um Nachrichten an das Socket zu senden
	 */
	public void connect (Properties properties, ImplCallBack callBack);
	
	/**
	 * aufgerufen, wenn eine Verbindung getrennt wurde
	 *
	 * @param properties das Eigenschafts-Objekt der ClientImpl
	 * @param callBack Referenz auf das ClientImpl-Objekt selbst, um Nachrichten an das Socket zu senden
	 */
	public void disconnect (Properties properties, ImplCallBack callBack);
	
}