package ec3.client;

import ec3.shared.Properties;

/**
 * ProtocolExtension zum Automatischen Wiederverbinden nach Fehlern.
 * 
 * @author am
 */
public class Reconnect implements ProtocolExtension, Runnable {
	
	/**
	 * true, wenn ein Fehler aufgetreten ist und Reconnect versucht werden soll.
	 */
	private boolean error;
	
	/**
	 * der Ping-Thread
	 */
	private Thread pinger;
	
	/**
	 * true, wenn eine Antwort auf Ping empfangen wurde
	 */
	private boolean response;
	
	/**
	 * Das benutzte Callback-Objekt. Wird beim Verbinden gesetzt.
	 */
	private ImplCallBack callback;
	
    /* (non-Javadoc)
     * @see ec3.client.ProtocolExtension#sockCommand(java.lang.String, java.lang.String, ec3.shared.Properties, ec3.client.ImplCallBack)
     */
    public boolean sockCommand(
        String command,
        String data,
        Properties properties,
        ImplCallBack callBack
    ) {
    	if (command.equals("pong")) {
    		response = true;
    		return true;
    	}
        return false;
    }
    
    /* (non-Javadoc)
     * @see ec3.client.ProtocolExtension#frontEndCommand(java.lang.String, java.lang.String[], ec3.shared.Properties, ec3.client.ImplCallBack)
     */
    public boolean frontEndCommand(String command, String[] data,
		Properties properties, ImplCallBack callBack
	) {
    	return false;
    }
    
	/* (non-Javadoc)
	 * @see ec3.client.ProtocolExtension#pluginCommand(java.lang.String, java.lang.String[], ec3.shared.Properties, ec3.client.ImplCallBack)
	 */
	public boolean pluginCommand (String command, String[] data, Properties properties, ImplCallBack callBack) {
		// leer
		return false;
	}
	
    /* (non-Javadoc)
     * @see ec3.client.ProtocolExtension#connect(ec3.shared.Properties, ec3.client.ImplCallBack)
     */
    public void connect(Properties properties, ImplCallBack callBack) {
    	this.callback = callBack;
    	error = false;
    	pinger = new Thread(this);
    	pinger.start();
    }
    
    /* (non-Javadoc)
     * @see ec3.client.ProtocolExtension#disconnect(ec3.shared.Properties, ec3.client.ImplCallBack)
     */
    public void disconnect(Properties properties, ImplCallBack callBack) {
    	pinger.interrupt();
    }
    
    /* (non-Javadoc)
     * @see ec3.client.ProtocolExtension#sockError(java.lang.String, ec3.shared.Properties, ec3.client.ImplCallBack)
     */
    public void sockError(String data, Properties properties, ImplCallBack callBack) {
    	callback.frontEndCommand("notice", new String[] {"waitReconnect"});
    	error = true;
    	new Thread(this).start();
    }

	/* (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 */
	public void run() {
		if (error) {
			try {
				Thread.sleep(30000);
			} catch (InterruptedException e) {
				// ignoriert
			}
			// wenn der error noch ansteht
			if (error) {
				callback.connect();
			}
	    	return;
		}
		while (!Thread.interrupted()) {
			try {
				Thread.sleep(60000);
				if (!response) {
					// hier is was faul mitm Server
					//callback.frontEndCommand("notice", new String[] {"noPingReply"});
				}
				response = false;
				callback.sockCommand("ping", "");
			} catch (InterruptedException e) {
				Thread.currentThread().interrupt();
			}
		}
	}
    
}
