package ec3.client;

import java.util.Vector;

import ec3.shared.Properties;
import ec3.shared.Tools;
import ec3.shared.crypto.B64Hex;
import ec3.shared.crypto.SHA256;

/**
 * Grundstzliche Befehle des Chatprotokolls
 *
 * - Loginbefehle
 * - Chatnachrichten
 * - Chatraumliste
 * - Userliste (incl. Anzeige: away/Admin)
 * - Chatfarben
 * 
 * @since 0.4
 */
public class Standard implements ProtocolExtension {
	
	public boolean sockCommand (
		String command,
		String data,
		Properties properties,
		ImplCallBack callBack
	) {
		String[] dataSplit = Tools.split (data, Tools.CHR0);
		try {
		
			//Chatraum betreten
			if (command.equals ("acceptChatroom")) {
				
				//OK - Chatraum betreten
				if (dataSplit[1].equals ("ok")) {
					
					//Eigenschaften setzen
					properties.setProperty ("client.inChatRoom", "1");
					properties.setProperty ("client.chatRoom", dataSplit[0]);
					
					//Event senden
					callBack.frontEndCommand (
						"acceptChatRoom",
						new String[] {dataSplit[0]}
					);
					
				//bereits drin => ignorieren
				} else if (dataSplit[1].equals ("alreadyjoined")) {
					
					properties.setProperty ("client.inChatRoom", "1");
					
				//Fehler bei Verschlsselung
				// => geht trotzdem, aber ohne Verschlsselung
				} else if (dataSplit[1].equals ("badcrc")) {
					
					//Eigenschaften setzen
					properties.setProperty ("client.inChatRoom", "1");
					properties.setProperty ("lient.chatRoom", dataSplit[0]);
					
					//Event senden
					callBack.frontEndCommand (
						"acceptChatRoom",
						new String[] {dataSplit[0]}
					);
					
				//Chatraum nicht gefunden
				} else if (dataSplit[1].equals ("notfound")) {
					
					//Event senden
					callBack.frontEndCommand (
						"chatRoomNotFound",
						Tools.emptyStringArray
					);
					
				//Chatraum konnte nicht erstellt werden (existiert bereits)
				} else if (dataSplit[1].equals ("notcreated")) {
					
					//Event senden
					callBack.frontEndCommand (
						"chatRoomNotCreated",
						Tools.emptyStringArray
					);
					
				//Chatraum darf nicht erstellt werden
				} else if (dataSplit[1].equals ("notallowed")) {
					
					//Event senden
					callBack.frontEndCommand (
						"chatRoomCreateNotAllowed",
						Tools.emptyStringArray
					);
					
				//Chatraum darf nicht erstellt werden
				} else if (dataSplit[1].equals ("failed")) {
					
					//Event senden
					callBack.frontEndCommand (
						"badChatRoomPwd",
						Tools.emptyStringArray
					);
					
				//unbekannter Status
				} else {
					
					//Fehler senden
					callBack.frontEndCommand (
						"error",
						new String[] {"Standard/acceptChatroom/" + dataSplit[1]}
					);
					
				}//if
				
				return true;
				
			//Login erfolgreich
			} else if (command.equals ("acceptUser")) {
				
				//OK - Login erfolgreich
				if (dataSplit[0].equals ("ok")) {
					
					//Chatname speichern
					properties.setProperty ("client.chatName", dataSplit[1]);
					//Event senden
					//callBack.frontEndCommand ("acceptUser", new String[] {});
					//Verarbeitung durch Verschlsselung ermglichen
					return false;
					
				//Login erfolgreich, aber keine Benutzung des Passwortes
				// fr Verschlsselung
				} else if (dataSplit[0].equals ("ok_nopwd")) {
					
					//Chatname speichern, Passwort lschen
					properties.setProperty ("client.chatName", dataSplit[1]);
					properties.setProperty ("client.password", "");
					//Event senden
					//callBack.frontEndCommand ("acceptUser", new String[] {});
					//Verarbeitung durch Verschlsselung ermglichen
					return false;
					
				//Chatname bereits vergeben
				} else if (dataSplit[0].equals ("used")) {
					
					//Event senden
					callBack.frontEndCommand (
						"chatNameUsed",
						Tools.emptyStringArray
					);
					
				//Anmeldung fehlgeschlagen (falsches Passwort)
				} else if (dataSplit[0].equals ("denied")) {
					
					//Event senden
					callBack.frontEndCommand (
						"loginDenied",
						Tools.emptyStringArray
					);
					
				//Anmeldung fehlgeschlagen (keine Gste erlaubt)
				} else if (dataSplit[0].equals ("noguests")) {
					
					//Event senden
					callBack.frontEndCommand (
						"noGuestsAllowed",
						Tools.emptyStringArray
					);
					
				//Falsche Protokollversion
				} else if (dataSplit[0].equals ("unsupported")) {
					
					//Event senden
					callBack.frontEndCommand (
						"serverProtoVersion",
						new String[] {properties.getProperty("server.protoVersion")}
					);
					
				//unbekannter Status
				} else {
					
					//Fehler senden
					callBack.frontEndCommand (
						"error",
						new String[] {"Standard/acceptUser/" + dataSplit[0]}
					);
					
				}//if
				
				return true;
				
			//Chatraumliste empfangen
			} else if (command.equals ("chatroomlist")) {
				
				//testen, ob Chatrume erstellt werden knnen
				if (dataSplit[1].equals ("0")) {
					
					properties.setProperty ("server.canCreateChatRoom", "0");
					
				} else {
					
					properties.setProperty ("server.canCreateChatRoom", "1");
					
				}//if
				
				dataSplit = Tools.split (dataSplit[0], Tools.CRLF);
				//dynamische Liste erzeugen
				Vector roomList = new Vector (dataSplit.length * 4);
				//Flag fr Fehlerfall
				boolean error = false;
				for (int i = 0; (i < dataSplit.length - 1) & !error; i++) {
					
					//Daten parsen
					String[] dataSplit2 = Tools.split (dataSplit[i], Tools.CHR1);
					//zu kurzer Raumeintrag
					if (dataSplit2.length < 4) {
						
						//Fehler senden
						callBack.frontEndCommand (
							"error",
							new String[] {"Standard/chatroomList/dataSplit2"}
						);
						error = true;
						continue;
						
					//Raumeintrag in Liste bernehmen
					}//if
					
					try {
						
						//vorher testen, ob die Userzahl eine Zahl ist
						Integer.parseInt (dataSplit2[1]);
						roomList.addElement (dataSplit2[0]);
						roomList.addElement (dataSplit2[1]);
						roomList.addElement (dataSplit2[2]);
						roomList.addElement (dataSplit2[3]);
						
					} catch (NumberFormatException e__) {
						
						//Fehler senden
						callBack.frontEndCommand (
							"error",
							new String[] {
								"Standard/chatroomList/NumberFormatException"
							}
						);
						error = true;
						continue;
						
					}//try
					
				}//for
				
				if (!error) {
					
					//Array basteln
					String[] temp = new String[roomList.size ()];
					roomList.copyInto (temp);
					//komplette Liste versenden
					callBack.frontEndCommand ("chatRoomList", temp);
					
				}//if
				
				return true;
				
			//Chatraum serverseitig geschlossen
			} else if (command.equals ("closeChatroom")) {
				
				//berprfen, ob berhaupt richtiger Raum
				if (dataSplit[0].equals (
					properties.getProperty ("client.chatRoom", "")
				)) {
					
					properties.setProperty ("client.chatRoom", "");
					properties.setProperty ("client.inChatRoom", "0");
					//Ereignis senden
					callBack.frontEndCommand (
						"closeChatRoom",
						Tools.emptyStringArray
					);
					
				}//if
				
				return true;
				
			} else if (command.equals ("message")) {
				
//				try {
//					
//					dataSplit[dataSplit.length - 1] = new String (dataSplit[dataSplit.length - 1].getBytes (), "UTF-8");
//					
//				} catch (java.io.UnsupportedEncodingException e) {}
				
				//Nachrichtenart untersuchen
				//normal
				if (dataSplit[0].equals ("normal")) {
					
					//Event senden
					callBack.frontEndCommand (
						"normalMessage",
						new String[] {
							dataSplit[1], //Name
							dataSplit[3], //Nachricht
							dataSplit[2]  //Farbe
						}
					);
					
				//"me"-Nachricht
				} else if (dataSplit[0].equals ("me")) {
					
					//Event senden
					callBack.frontEndCommand (
						"meMessage",
						new String[] {
							dataSplit[1], //Name
							dataSplit[3], //Nachricht
							dataSplit[2]  //Farbe
						}
					);
					
				//schreien-Nachricht
				} else if (dataSplit[0].equals ("shout")) {
					
					//Event senden
					callBack.frontEndCommand (
						"shoutMessage",
						new String[] {
							dataSplit[1], //Name
							dataSplit[3], //Nachricht
							dataSplit[2]  //Farbe
						}
					);
					
				//Private Nachricht
				} else if (dataSplit[0].equals ("private")) {
					
					//Event senden
					callBack.frontEndCommand (
						"privateMessage",
						new String[] {
							dataSplit[1], //Sender
							dataSplit[2], //Empfnger
							dataSplit[4], //Nachricht
							dataSplit[3]  //Farbe
						}
					);
					
				//Server-Nachricht
				} else if (dataSplit[0].equals ("server")) {
					
					//Event senden
					callBack.frontEndCommand (
						"serverMessage",
						new String[] {dataSplit[1]}
					);
					
				}//if
				
				return true;
				
			//Ping zur Sicherstellung der Verbindung
			} else if (command.equals ("ping")) {
				
				//antworten
				callBack.sockCommand ("pong", "");
				
				return true;
				
			} else if (command.equals ("welcome")) {
				String[] dataSplit2 = Tools.split (
					dataSplit[0],
					Tools.CRLF
				);
				//Informationen des Servers speichern
				properties.setProperty (
					"server.version",
					dataSplit2[0]
				);
				properties.setProperty (
					"server.state",
					dataSplit2[1]
				);
				properties.setProperty (
					"server.lang",
					dataSplit2[2]
				);
				
				int serverMaxTextLen;
				try {
					
					serverMaxTextLen = Integer.parseInt (
						dataSplit2[3]
					);
					
				} catch (NumberFormatException e_) {
					
					serverMaxTextLen = -1;
					
				}//try
				properties.setProperty (
					"server.maxTextLen",
					new Integer (serverMaxTextLen).toString ()
				);
				
				properties.setProperty (
					"server.admin",
					dataSplit2[4]
				);
				properties.setProperty (
					"server.adminMail",
					dataSplit2[5]
				);
				properties.setProperty (
					"server.stdChatRoom",
					dataSplit2[6]
				);
				properties.setProperty (
					"server.onlyChatRoom",
					dataSplit2[7]
				);
				properties.setProperty (
					"server.authHash",
					dataSplit2[8]
				);
				
				//nach gemeinsamer Protokollversion suchen
				String[] protoVersion = Tools.split (
					properties.getProperty ("client.supportedProtoVersion", ""),
					","
				);
				String s_vers = "," + dataSplit2[9] + ",";
				String serverProtoVersion = "";
				int i = 0;
				while (serverProtoVersion == "") {
					
					if (s_vers.indexOf (protoVersion[i]) != -1) {
						
						serverProtoVersion = protoVersion[i];
						
					}//if
					i++;
					if (i > protoVersion.length) {
						
						serverProtoVersion = "#";
						
					}//if
					
				}//while
				properties.setProperty (
					"server.protoVersion",
					serverProtoVersion
				);
				
				//annehmen, das Chatrume erstellt werden knnen
				properties.setProperty (
					"server.canCreateChatRoom",
					"1"
				);
				
				//falsche maximale Textlnge
				if (serverMaxTextLen == -1) {
					
					//Fehler senden
					callBack.frontEndCommand (
						"error",
						new String[] {"welcome/serverMaxTextLen"}
					);
					
				//keine gemeinsame Protokollversion vorhanden
				} else if (serverProtoVersion == "#") {
					
					//Fehler senden
					callBack.frontEndCommand (
						"error",
						new String[] {
							"welcome/serverProtoVersion",
							properties.getProperty ("client.supportedProtoVersion", ""),
							dataSplit2[9]
						}
					);
					
				} else {
					
					String welcomeMsg = dataSplit[1].trim ();
					
					if (!welcomeMsg.equals("")) {
						//Event senden
						callBack.frontEndCommand (
							"welcome",
							new String[] {welcomeMsg}
						);
					}
					//Login anfordern
					callBack.frontEndCommand (
						"login",
						Tools.emptyStringArray
					);
					
				}//if
				
				return true;
				
			} else if (command.equals ("__lookup")) {
				//Frontend benachrichtigen
				callBack.frontEndCommand (
					"lookup",
					new String[] {data}
				);
				return true;
			}
			
		} catch (ArrayIndexOutOfBoundsException e) {
			
			callBack.frontEndCommand (
				"error",
				new String[] {"Standard/" + command + "/ArrayOutOfBounds"}
			);
			e.printStackTrace();
			return true;
			
		}
		
		return false;
		
	}
	
	public boolean frontEndCommand (
		String command,
		String[] data,
		Properties properties,
		ImplCallBack callBack
	) {
		
		if (command.equals ("createChatRoom")) {
			
			//nur, wenn eingeloggt
			if (properties.getProperty ("client.connected", "0").equals ("1")
			 && properties.getProperty ("client.loggedIn", "0").equals ("1")) {
				
				//Chatraum merken
				properties.setProperty ("client.chatRoom", data[0]);
				properties.setProperty ("client.inChatRoom", "0");
				
				//wenn kein Passwort gegeben...
				if ((data.length == 2) || (data[2] == null) || (data[2].equals (""))) {
					
					properties.setProperty ("client.chatRoomPassword", "");
					//Chatraum erstellen
					callBack.sockCommand (
						"createChatroom",
						data[0] + Tools.CHR0 + data[1] + Tools.CHR0
						 + Tools.CHR0
					);
					
				// Chatraum mit Passwort erstellen
				} else {
					
					String[] res = new String[] {data[2]};
					boolean test = callBack.pluginCommand("encryptPwd", res);
					
					if (test) {
						properties.setProperty ("client.chatRoomPassword", data[2]);
						//Chatraum erstellen
						callBack.sockCommand (
							"createChatroom",
							data[0] + Tools.CHR0 + data[1] + Tools.CHR0
							 + res[0]
						);
					} else {
						callBack.frontEndCommand (
							"notImplemented",
							new String[] {"Standard/createChatRoom/password"}
						);
					}
					
				}//if
				
			}//if
			
			return true;
			
		} else if (command.equals ("getChatRoomList")) {
			
			//nur, wenn eingeloggt
			if (properties.getProperty ("client.connected", "0").equals ("1")
			 && properties.getProperty ("client.loggedIn", "0").equals ("1")) {
				
				callBack.sockCommand ("getChatroomlist", "");
				
			} else {
				
				//Hinweis senden
				callBack.frontEndCommand (
					"notice",
					new String[] {"notLoggedIn"}
				);
				
			}//if
			
			return true;
			
		} else if (command.equals ("joinChatRoom")) {
			
			//nur, wenn eingeloggt
			if (properties.getProperty ("client.connected", "0").equals ("1")
			 && properties.getProperty ("client.loggedIn", "0").equals ("1")) {
				
				//Chatraum merken
				properties.setProperty ("client.chatRoom", data[0]);
				properties.setProperty ("client.inChatRoom", "0");
				
				//wenn kein Passwort gegeben...
				if ((data.length == 1) || (data[1].equals (""))) {
					
					//Chatraum betreten
					callBack.sockCommand ("joinChatroom", data[0] + Tools.CHR0);
					properties.setProperty ("client.chatRoomPassword", "");
					
				//Chatraum mit Passwort betreten
				} else {
					String pwdString = data[1]
						 + properties.getProperty ("client.chatName", "")
						 + properties.getProperty ("server.authHash", "");
					//Chatraum betreten
					callBack.sockCommand (
						"joinChatroom",
						data[0] + Tools.CHR0 + B64Hex.hexEncode(SHA256.digest(pwdString.getBytes()))
					);
					
					//Passwort speichern
					properties.setProperty ("client.chatRoomPassword", data[1]);
					
				}//if
				
			}//if
			
			return true;
			
		} else if (command.equals ("login")) {
			
			//nur, wenn noch nicht eingeloggt, aber verbunden
			if (properties.getProperty ("client.connected", "0").equals ("1")
			 && properties.getProperty ("client.loggedIn", "0").equals ("0")) {
				
				//Name und Farbe merken
				properties.setProperty ("client.chatName", data[0]);
				properties.setProperty ("client.chatPassword", data[1]);
				properties.setProperty ("client.chatColor", data[2]);
				properties.setProperty ("client.hasChatName", "1");
				
				//Passwort?
				String pw = "";
				
				//wenn Passwort bergeben...
				if ((data[1] != null)
				 && (data[1].equals ("") == false)) {
					pw = data[1] + properties.getProperty ("server.authHash", "");
					//dann Hashen
					pw = B64Hex.hexEncode(SHA256.digest(pw.getBytes()));
					
				}//if
				
				//Login senden
				callBack.sockCommand (
					"login",
					properties.getProperty ("client.chatName", "")
					 + Tools.CHR0 + pw + Tools.CHR0
					 + properties.getProperty ("client.chatColor", "")
					 + Tools.CHR0
					 + properties.getProperty ("client.version", "")
					 + Tools.CHR0
					 + properties.getProperty ("server.protoVersion", "")
				);
				
				//Eigenschaften setzen
				properties.setProperty ("client.loggedIn", "1");
				properties.setProperty ("client.inChatRoom", "0");
				
			}//if
			
			return true;
			
		} else if (command.equals ("message")) {
			
			//data[0] - Nachricht
			//data[1]... - Zielpersonen (keine angegeben => an alle)
			
			//Leerzeichen etc. entfernen
			String message = data[0].trim ();
			
			//wenn noch etwas zu senden da ist
			if (message.equals ("") == false) {
				
				//Befehl
				if (message.startsWith ("/")) {
					
					//splitten in Befehl und angehngte Daten
					String[] messageSplit = new String[2];
					//Daten gefunden
					if (message.indexOf (" ") > -1) {
						
						messageSplit[0] =
						 message.substring (0, message.indexOf (" ")).toLowerCase ();
						messageSplit[1] =
						 message.substring (message.indexOf (" ") + 1);
						
					//keine Daten gefunden
					} else {
						
						messageSplit[0] = message.toLowerCase ();
						messageSplit[1] = "";
						
					}//if
					
					//Private Nahcricht an letzten Empfnger
					if (messageSplit[0].equals ("//")) {
						
						//nur, wenn eingeloggt
						if (properties.getProperty ("client.connected", "0")
						 .equals ("1")
						 && properties.getProperty ("client.loggedIn", "0")
						 .equals ("1")) {
							
							if (!messageSplit[1].equals("")
							 && !properties.getProperty ("client.lastMsgReceiver", "").equals ("")) {
								
								//Senden
								callBack.sockCommand (
									"message",
									"private" + Tools.CHR0 + properties.getProperty ("client.lastMsgReceiver", "")
									 + Tools.CHR0 + messageSplit[1]
								);
								
							} else {
								
								//Hinweis senden
								callBack.frontEndCommand (
									"notice",
									new String[] { "syntax//" }
								);
								
							}//if
							
						//nicht eingeloggt
						} else {
							
							//Hinweis senden
							callBack.frontEndCommand (
								"notice",
								new String[] {"notLoggedIn"}
							);
							
						}//if
						
						return true;
						
					//(neu) verbinden
					} else if (messageSplit[0].equals ("/connect")) {
						
						messageSplit = Tools.splitArguments (messageSplit[1], 3);
						String name = messageSplit[1];
						String pwd = messageSplit[2];
						if (!name.equals ("") || properties.getProperty ("client.hasChatName", "0").equals ("1")) {
							
							messageSplit = Tools.split (messageSplit[0], ":");
							if (messageSplit[0].equals ("")) {
								
								if (properties.getProperty ("server.adress").equals ("")) {
									
									callBack.frontEndCommand (
										"notice",
										new String[] {"syntaxConnect"}
									);
									
								} else {
									
									try {
										
										callBack.frontEndCommand (
											"connectData",
											new String[] {properties.getProperty ("server.adress"), properties.getProperty ("server.port"), name, pwd}
										);
										callBack.connect (properties.getProperty ("server.adress"), new Integer (properties.getProperty ("server.port")).intValue ()); //0 - Defaultport
										
									} catch (NumberFormatException e) {
										
										callBack.frontEndCommand (
											"notice",
											new String[] {"syntaxConnect"}
										);
										
									} catch (IllegalArgumentException e) {
										
										callBack.frontEndCommand (
											"notice",
											new String[] {"syntaxConnect"}
										);
										
									}
									
								}
								
							} else if (messageSplit.length == 2) {
								
								try {
									
									callBack.frontEndCommand (
										"connectData",
										new String[] {messageSplit[0], messageSplit[1], name, pwd}
									);
									callBack.connect (messageSplit[0], new Integer (messageSplit[1]).intValue ());
									
								} catch (NumberFormatException e) {
									
									callBack.frontEndCommand (
										"notice",
										new String[] {"syntaxConnect"}
									);
									
								} catch (IllegalArgumentException e) {
									
									callBack.frontEndCommand (
										"notice",
										new String[] {"syntaxConnect"}
									);
									
								}
								
							} else {
								
								callBack.frontEndCommand (
									"connectData",
									new String[] {messageSplit[0], "0", name, pwd}
								);
								callBack.connect (messageSplit[0], 0); //0 - Defaultport
								
							}
							
						} else {
							
							callBack.frontEndCommand (
								"notice",
								new String[] {"noChatName"}
							);
							
						}
						
						return true;
						
					//Chatraum erstellen
					} else if (messageSplit[0].equals ("/create") || messageSplit[0].equals ("/cr")) {
						
						//nur, wenn eingeloggt
						if (properties.getProperty ("client.connected", "0").equals ("1")
						 && properties.getProperty ("client.loggedIn", "0").equals ("1")) {
							
							// Name Beschreibung (Passwort)
							messageSplit = Tools.splitArguments (messageSplit[1], 3);
							
							if (messageSplit[0].equals("")) {
								
								callBack.frontEndCommand (
									"notice",
									new String[] {"syntaxCreate"}
								);
								
							} else {
								
								//Chatraum merken
								properties.setProperty ("client.chatRoom", messageSplit[0]);
								properties.setProperty ("client.inChatRoom", "0");
								
								//wenn kein Passwort gegeben...
								if (messageSplit[2].equals ("")) {
									
									properties.setProperty ("client.chatRoomPassword", "");
									//Chatraum betreten
									callBack.sockCommand (
										"createChatroom",
										messageSplit[0] + Tools.CHR0 + messageSplit[1]
										 + Tools.CHR0 + Tools.CHR0
									);
									
								// Chatraum mit Passwort erstellen
								} else {
									
									String[] res = new String[] {messageSplit[2]};
									boolean test = callBack.pluginCommand("encryptPwd", res);
									
									if (test) {
										properties.setProperty ("client.chatRoomPassword", messageSplit[2]);
										//Chatraum erstellen
										callBack.sockCommand (
											"createChatroom",
											messageSplit[0] + Tools.CHR0 + messageSplit[1]
											 + Tools.CHR0 + res[0]
										);
									} else {
										callBack.frontEndCommand (
											"notImplemented",
											new String[] {"Standard/createChatRoom/password"}
										);
									}
									
								}//if
								
							}
							
						//nicht eingeloggt
						} else {
							
							//Hinweis senden
							callBack.frontEndCommand (
								"notice",
								new String[] {"notLoggedIn"}
							);
							
						}//if
						
						return true;
						
					//Verbindung trennen
					} else if (messageSplit[0].equals ("/disconnect") || messageSplit[0].equals ("/d")) {
						
						if (properties.getProperty ("client.connected", "0").equals ("1")) {
							
							if (messageSplit[1].equals ("")) {
								
								callBack.disconnect ();
								
							} else {
								
								properties.setProperty ("client.ignoreDisconnect", "1");
								callBack.sockCommand ("quit", messageSplit[1]);
								callBack.disconnect();
								
							}
							
						}
						
						return true;
						
					//Verbindung trennen und Fenster schlieen
					} else if (messageSplit[0].equals("/exit") || messageSplit[0].equals ("/quit") || messageSplit[0].equals ("/q")) {
						
						if (properties.getProperty ("client.connected", "0").equals ("1")) {
							
							if (messageSplit[1].equals ("")) {
								
								callBack.disconnect ();
								callBack.frontEndCommand("closeWindow", Tools.emptyStringArray);
								
							} else {
								
								properties.setProperty ("client.ignoreDisconnect", "1");
								callBack.sockCommand ("quit", messageSplit[1]);
								callBack.disconnect();
								callBack.frontEndCommand("closeWindow", Tools.emptyStringArray);
								
							}
							
						}
						
						return true;
						
					// Userliste neu abfragen
					} else if (messageSplit[0].equals("/getul")) {
						
						//nur, wenn in Chatraum
						if (properties.getProperty ("client.connected", "0").equals ("1")
						 && properties.getProperty ("client.loggedIn", "0").equals ("1")
						 && properties.getProperty ("client.inChatRoom", "0").equals ("1")){
							
							callBack.sockCommand (
								"getUserlist",
								properties.getProperty ("client.chatRoom", "")
							);
							
						//nicht im Chatraum
						} else {
							
							//Hinweis senden
							callBack.frontEndCommand (
								"notice",
								new String[] {"notInChatRoom"}
							);
							
						}//if
						
						return true;
						
					//Chatraum betreten
					} else if (messageSplit[0].equals ("/join") || messageSplit[0].equals ("/j")) {
						
						//nur, wenn eingeloggt
						if (properties.getProperty ("client.connected", "0")
						 .equals ("1")
						 && properties.getProperty ("client.loggedIn", "0")
						 .equals ("1")) {
							
							messageSplit = Tools.splitArguments (messageSplit[1], 2);
							
							//Chatraum merken
							properties.setProperty ("client.chatRoom", messageSplit[0]);
							properties.setProperty ("client.inChatRoom", "0");
							
							//wenn kein Passwort gegeben...
							if (messageSplit[1].equals ("")) {
								
								//Chatraum betreten
								callBack.sockCommand ("joinChatroom", messageSplit[0] + Tools.CHR0);
								properties.setProperty ("client.chatRoomPassword", "");
								
							//Chatraum mit Passwort betreten
							} else {
								String pwdString = messageSplit[1]
								 + properties.getProperty ("client.chatName", "")
								 + properties.getProperty ("server.authHash", "");
								//Chatraum betreten
								callBack.sockCommand (
									"joinChatroom",
									messageSplit[0] + Tools.CHR0 + B64Hex.hexEncode(SHA256.digest(pwdString.getBytes()))
								);
								
								//Passwort speichern
								properties.setProperty ("client.chatRoomPassword", messageSplit[1]);
								
							}//if
							
						//nicht eingeloggt
						} else {
							
							//Hinweis senden
							callBack.frontEndCommand (
								"notice",
								new String[] {"notLoggedIn"}
							);
							
						}//if
						
						return true;
						
					//"Me"-Nachricht
					} else if (messageSplit[0].equals ("/me")) {
						
						//nur, wenn eingeloggt
						if (properties.getProperty ("client.connected", "0")
						 .equals ("1")
						 && properties.getProperty ("client.loggedIn", "0")
						 .equals ("1")
						 && properties.getProperty ("client.inChatRoom", "0")
						 .equals ("1")) {
							
							//wenn Daten vorhanden
							if (!messageSplit[1].equals ("")) {
								
								//absenden
								callBack.sockCommand (
									"message",
									"me" + Tools.CHR0 + Tools.CHR0
									 + messageSplit[1]
								);
								
							//keine Daten vorhanden
							} else {
								
								//Hinweis senden
								callBack.frontEndCommand (
									"notice",
									new String[] {"syntaxMe"}
								);
									
							}//if
							
						//nicht eingeloggt
						} else {
							
							//Hinweis senden
							callBack.frontEndCommand (
								"notice",
								new String[] {"notInChatRoom"}
							);
							
						}//if
						
						return true;
						
					//private Nachricht
					} else if (messageSplit[0].equals ("/msg")) {
						
						//nur, wenn eingeloggt
						if (properties.getProperty ("client.connected", "0")
						 .equals ("1")
						 && properties.getProperty ("client.loggedIn", "0")
						 .equals ("1")) {
							
							messageSplit = Tools.splitArguments (messageSplit[1], 2);
							if (!messageSplit[0].equals ("") && !messageSplit[1].equals ("")) {
								
								//Senden
								callBack.sockCommand (
									"message",
									"private" + Tools.CHR0 + messageSplit[0]
									 + Tools.CHR0 + messageSplit[1]
								);
								properties.setProperty ("client.lastMsgReceiver", messageSplit[0]);
								
							} else {
								
								//Hinweis senden
								callBack.frontEndCommand (
									"notice",
									new String[] { "syntaxMsg" }
								);
								
							}
							
						//nicht eingeloggt
						} else {
							
							//Hinweis senden
							callBack.frontEndCommand (
								"notice",
								new String[] {"notLoggedIn"}
							);
							
						}//if
						
						return true;
						
					//schreien-Nachricht
					} else if (messageSplit[0].equals ("/s")) {
						
						//nur, wenn eingeloggt
						if (properties.getProperty ("client.connected", "0")
						 .equals ("1")
						 && properties.getProperty ("client.loggedIn", "0")
						 .equals ("1")
						 && properties.getProperty ("client.inChatRoom", "0")
						 .equals ("1")) {
							
							//wenn Daten da sind
							if (!messageSplit[1].equals ("")) {
								
								//Senden
								callBack.sockCommand (
									"message",
									"shout" + Tools.CHR0 + Tools.CHR0
									 + messageSplit[1]
								);
								
							//keine Daten
							} else {
								
								//Hinweis senden
								callBack.frontEndCommand (
									"notice",
									new String[] {"syntaxS"}
								);
								
							}//if
							
						//nicht eingeloggt
						} else {
							
							//Hinweis senden
							callBack.frontEndCommand (
								"notice",
								new String[] {"notInChatRoom"}
							);
							
						}//if
						
						return true;
						
					}//if
					
					return false;
					
				//Serverbefehl
				} else if (message.startsWith ("\\")) {
					
					//splitten in Befehl und angehngte Daten
					String[] messageSplit = new String[2];
					//Daten gefunden
					if (message.indexOf (" ") > -1) {
						
						messageSplit[0] =
						 message.substring (1, message.indexOf (" "));
						messageSplit[1] =
						 message.substring (message.indexOf (" ") + 1);
						
					//keine Daten gefunden
					} else {
						
						messageSplit[0] = message.substring(1);
						messageSplit[1] = "";
						
					}//if
					//nur, wenn verbunden
					if (properties.getProperty ("client.connected", "0")
					 .equals ("1")
					 && properties.getProperty ("client.loggedIn", "0")
					 .equals ("1")) {
						
						//senden
						callBack.sockCommand ("command", Tools.join(messageSplit, Tools.CHR0));
						
					//nicht eingeloggt
					} else {
						
						//Hinweis senden
						callBack.frontEndCommand (
							"notice",
							new String[] {"notLoggedIn"}
						);
						
					}
					return true;
					
				//kein Befehl
				} else {
					
					//nur, wenn eingeloggt
					if (properties.getProperty ("client.connected", "0")
					 .equals ("1")
					 && properties.getProperty ("client.loggedIn", "0")
					 .equals ("1")
					 && properties.getProperty ("client.inChatRoom", "0")
					 .equals ("1")) {
						
						String[] temp = new String[data.length - 1];
						System.arraycopy (data, 1, temp, 0, temp.length);
						String to = Tools.join (temp, Tools.CRLF);
						//senden
						callBack.sockCommand ("message", "normal" + Tools.CHR0 + to + Tools.CHR0 + message);
						
					//nicht eingeloggt
					} else {
						
						//Hinweis senden
						callBack.frontEndCommand (
							"notice",
							new String[] {"notInChatRoom"}
						);
						
					}//if
					
					return true;
					
				}//if
				
			}//if
			
			//nur leere Nachricht => nix tun, keine weiteren Aktionen
			return true;
			
		}
		
		return false;
		
	}
	
	/* (non-Javadoc)
	 * @see ec3.client.ProtocolExtension#pluginCommand(java.lang.String, java.lang.String[], ec3.shared.Properties, ec3.client.ImplCallBack)
	 */
	public boolean pluginCommand (String command, String[] data, Properties properties, ImplCallBack callBack) {
		// leer
		return false;
	}
	
	/* (non-Javadoc)
	 * @see ec3.client.ProtocolExtension#connect(ec3.shared.Properties, ec3.client.ImplCallBack)
	 */
	public void connect (Properties properties, ImplCallBack callBack) {
		// nix
	}
	
	/* (non-Javadoc)
	 * @see ec3.client.ProtocolExtension#disconnect(ec3.shared.Properties, ec3.client.ImplCallBack)
	 */
	public void disconnect (Properties properties, ImplCallBack callBack) {
		// nix
	}

    /* (non-Javadoc)
     * @see ec3.client.ProtocolExtension#sockError(java.lang.String, ec3.shared.Properties, ec3.client.ImplCallBack)
     */
    public void sockError(String data, Properties properties, ImplCallBack callBack) {
        
        // leer
        
    }
	
}