package ec3.client.applet;

import java.awt.Button;
import java.awt.Checkbox;
import java.awt.Cursor;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.List;
import java.awt.Panel;
import java.awt.TextArea;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.Hashtable;

import ec3.shared.ChatRoomListItem;

/**
 * Ein Fenster zum Anzeigen der Chatraumliste
 *
 * @since 0.4
 */
public class ChatRoomList extends Frame implements InputFrameCallBack,
WindowListener, ItemListener, ActionListener, KeyListener {
	
	/**
	 * erstellt das Chatraumlistenfenster mit Callback-Objekt
	 *
	 * @param callBack das aufrufende Applet
	 * @param enableCreatePwd <code>true</code>, wenn das Erstellen von Rumen mit Passwort
	 *        mglich ist.
	 */
	public ChatRoomList (ChatRoomListCallBack callBack, boolean enableCreatePwd) {
		super ("Chatraumliste");
		
		addWindowListener(this);
		
		//Raumliste
		list = new Hashtable ();
		
		//Fensterelemente
		roomList = new List ();
		roomList.addItemListener(this);
		roomList.addKeyListener(this);
		desc = new TextArea ();
		desc.setEditable (false);
		desc.addKeyListener(this);
		pwd = new Checkbox ("Passwort?");
		pwd.setEnabled (false);
		countDescLabel = new Label ("Anzahl Personen im Raum:");
		countLabel = new Label ();
		enterButton = new Button ("Betreten");
		enterButton.addActionListener(this);
		enterButton.addKeyListener(this);
		cancelButton = new Button ("Abbrechen");
		cancelButton.addActionListener(this);
		cancelButton.addKeyListener(this);
		createButton = new Button ("Erstellen");
		createButton.addActionListener(this);
		createButton.addKeyListener(this);
		
		Panel panel1 = new Panel ();
		panel1.setLayout (new GridLayout (0, 1));
		Panel panel2 = new Panel ();
		panel2.setLayout (new GridLayout (0, 1));
		panel2.setCursor (new Cursor (Cursor.DEFAULT_CURSOR));
		Panel panel3 = new Panel ();
		panel3.setLayout (new FlowLayout (FlowLayout.LEFT));
		Panel panel4 = new Panel ();
		panel4.setLayout (new GridLayout (1, 0));
		
		setLayout (new GridLayout (1, 0));
		setSize (600, 200);
		
		add (roomList);
		add (panel1);
		panel1.add (desc);
		panel1.add (panel2);
		panel2.add (pwd);
		panel2.add (panel3);
		panel2.add (panel4);
		panel3.add (countDescLabel);
		panel3.add (countLabel);
		panel4.add (enterButton);
		panel4.add (cancelButton);
		panel4.add (createButton);
		
		this.callBack = callBack;
		
		//Eingabefelder
		pwdInputFrame = new InputFrame ("Passwort eingeben", this, true);
		chatRoomCreate = new ChatRoomCreate (this, enableCreatePwd);
		
	}//ChatRoomList(...)
	
	/**
	 * neue Liste eintragen
	 *
	 * @param data die neue Chatraumliste
	 */
	public void setList (String[] data) {
		
		//alte Liste entfernen
		roomList.removeAll ();
		list.clear ();
		
		//neue Liste aufbauen
		for (int i = 0; i < data.length; i += 4) {
			
			ec3.shared.ChatRoomListItem item = new ChatRoomListItem (data[i], Integer.parseInt (data[i + 1]), !data[i + 2].equals (""), data[i + 3]);
			list.put (data[i], item);
			roomList.add (data[i]);
			
		}//for
		
		//1. Chatraum selektieren...
		roomList.select (0);
		//...und Infos auslesen
		if (data.length > 0) {
			
			desc.setText (data[3]);
			countLabel.setText (data[1]);
			pwd.setState (!data[2].equals (""));
			
		}//if
		
	}//setList(...)
	
	/* 
	 * (non-Javadoc)
	 * @see java.awt.Component#show()
	 */
	public void show () {
		super.show ();
		roomList.requestFocus ();
	}//show()
	
	/* 
	 * (non-Javadoc)
	 * @see ec3.client.applet.InputFrameCallBack#inputFrameCallBack(ec3.client.applet.InputFrame, java.lang.String)
	 */
	public void inputFrameCallBack (InputFrame source, String text) {
		
		if (source == pwdInputFrame) {
			
			setVisible(false);
			callBack.chatRoomListCallBack (tempName, text, null, ChatRoomListCallBack.JOIN);
			
		}//if
		
	}//inputFrameCallBack(...)
	
	/* 
	 * (non-Javadoc)
	 * @see java.awt.Component#hide()
	 */
	public void hide () {
		super.hide ();
		pwdInputFrame.setVisible(false);
	}//hide()
	
	/**
	 * sendet eine Benachrichtigung, das ein Chatraum erstellt werden soll
	 * 
	 * @param name der Name des Raumes
	 * @param description die Beschreibung des Raumes
	 * @param password das Passwort des Raumes
	 */
	public void createCallBack (String name, String description, String password) {
		
		setVisible(false);
		callBack.chatRoomListCallBack (name, password, description, ChatRoomListCallBack.CREATE);
		
	}//createCallBack(...)
	
	/**
	 * die Liste aller Chatrume
	 */
	private List roomList;
	
	/**
	 * die TextArea fr die Beschreibung
	 */
	private TextArea desc;
	
	/**
	 * die Checkbox fr "Passwort gesetzt"
	 */
	private Checkbox pwd;
	
	/**
	 * das Label fr die Beschreibung "Personenanzahl"
	 */
	private Label countDescLabel;
	
	/**
	 * das Laben fr die Personenanzahl
	 */
	private Label countLabel;
	
	/**
	 * der "Betreten"-Button
	 */
	private Button enterButton;
	
	/**
	 * der "Abbrechen"-Button
	 */
	private Button cancelButton;
	
	/**
	 * der "Erstellen"-Button
	 */
	private Button createButton;
	
	/**
	 * beinhaltet die Daten zu allen Chatrumen
	 */
	private Hashtable list;
	
	/**
	 * das Callback-Objekt, das benachrichtugt werden soll
	 */
	private ChatRoomListCallBack callBack;
	
	/**
	 * das Eingabefenster fr das Paasswort
	 */
	private InputFrame pwdInputFrame;
	
	/**
	 * das Eingabefenster zum Erstellen eines neuen Chatraumes
	 */
	private ChatRoomCreate chatRoomCreate;
	
	/**
	 * temporre Speicherung eines Chatraumnamens (z.b. fr Passworteingabe)
	 */
	private String tempName;

	/* 
	 * (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowActivated(java.awt.event.WindowEvent)
	 */
	public void windowActivated(WindowEvent e) {
		// nix
	}

	/* 
	 * (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowClosed(java.awt.event.WindowEvent)
	 */
	public void windowClosed(WindowEvent e) {
		// nix
	}

	/* 
	 * (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowClosing(java.awt.event.WindowEvent)
	 */
	public void windowClosing(WindowEvent e) {
		if (e.getSource() != this) return;
		//Fenster nicht schlieen, nur verstecken
		setVisible(false);
	}

	/* 
	 * (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowDeactivated(java.awt.event.WindowEvent)
	 */
	public void windowDeactivated(WindowEvent e) {
		// nix
	}

	/* 
	 * (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowDeiconified(java.awt.event.WindowEvent)
	 */
	public void windowDeiconified(WindowEvent e) {
		// nix
	}

	/* 
	 * (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowIconified(java.awt.event.WindowEvent)
	 */
	public void windowIconified(WindowEvent e) {
		// nix
	}

	/* 
	 * (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowOpened(java.awt.event.WindowEvent)
	 */
	public void windowOpened(WindowEvent e) {
		// nix
	}

	/* 
	 * (non-Javadoc)
	 * @see java.awt.event.ItemListener#itemStateChanged(java.awt.event.ItemEvent)
	 */
	public void itemStateChanged(ItemEvent e) {
		if (e.getSource() != roomList) return;
		ChatRoomListItem item
		 = (ChatRoomListItem)list.get (roomList.getSelectedItem ());
		desc.setText (item.desc);
		countLabel.setText (new Integer (item.users).toString ());
		pwd.setState (item.pwd);
	}

	/* 
	 * (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		//Chatraum betreten
		if (e.getSource() == enterButton) {
			ChatRoomListItem item = (ChatRoomListItem)list.get (roomList.getSelectedItem ());
			if (item.pwd) {
				tempName = item.name;
				pwdInputFrame.setVisible(true);
			} else {
				pwdInputFrame.setVisible(false);
				setVisible(false);
				callBack.chatRoomListCallBack (item.name, "", null, ChatRoomListCallBack.JOIN);
			}//if
		//Fenster verstecken
		} else if (e.getSource() == cancelButton) {
			setVisible(false);
		//Chatraum erstellen
		} else if (e.getSource() == createButton) {
			chatRoomCreate.setVisible(true);
			pwdInputFrame.setVisible(false);
		}//if
		
	}
	
	/* 
	 * (non-Javadoc)
	 * @see java.awt.event.KeyListener#keyPressed(java.awt.event.KeyEvent)
	 */
	public void keyPressed(KeyEvent e) {
		//Escape -> Fenster vestecken
		if (/*(e.getSource() == this) && */(e.getModifiers() == 0)
		 && (e.getKeyCode() == KeyEvent.VK_ESCAPE)) {
			setVisible(false);
			e.consume();
		//Tab und Shift+Tab im Beschreibungsfeld -> Focus bewegen
		} else if ((e.getSource() == desc)
		 && (e.getKeyCode() == KeyEvent.VK_TAB)) {
			if (e.getModifiers() == InputEvent.SHIFT_MASK) {
				roomList.requestFocus();
			} else if (e.getModifiers() == 0) {
				desc.transferFocus();
			}//if
			e.consume();
		// Enter auf createButton
		} else if ((e.getSource() == createButton)
		 && (e.getKeyCode() == KeyEvent.VK_ENTER) && (e.getModifiers() == 0)) {
			chatRoomCreate.setVisible(true);
			pwdInputFrame.setVisible(false);
			e.consume();
		// Enter auf cancelButton
		} else if ((e.getSource() == cancelButton)
		 && (e.getKeyCode() == KeyEvent.VK_ENTER) && (e.getModifiers() == 0)) {
			setVisible(false);
			e.consume();
		// Enter auf allen anderen
		} else if ((e.getKeyCode() == KeyEvent.VK_ENTER)
		 && (e.getModifiers() == 0)) {
			ChatRoomListItem item = (ChatRoomListItem)list.get (
				roomList.getSelectedItem ()
			);
			if (item.pwd) {
				tempName = item.name;
				pwdInputFrame.setVisible(true);
			} else {
				pwdInputFrame.setVisible(false);
				setVisible(false);
				callBack.chatRoomListCallBack (item.name, "", null,
					ChatRoomListCallBack.JOIN
				);
			}//if
			e.consume();
		}
	}

	/* 
	 * (non-Javadoc)
	 * @see java.awt.event.KeyListener#keyReleased(java.awt.event.KeyEvent)
	 */
	public void keyReleased(KeyEvent e) {
		// nix
	}

	/* 
	 * (non-Javadoc)
	 * @see java.awt.event.KeyListener#keyTyped(java.awt.event.KeyEvent)
	 */
	public void keyTyped(KeyEvent e) {
		// nix
	}
}