package ec3.client.applet;

import java.applet.AudioClip;
import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Checkbox;
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Panel;
import java.awt.ScrollPane;
import java.awt.TextArea;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.StringSelection;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.Calendar;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import ec3.client.ClientImpl;
import ec3.shared.EC3Event;
import ec3.shared.EC3Listener;
import ec3.shared.StringFIFO;
import ec3.shared.Tools;
import ec4.tools.ui.ColorTextArea;
import ec4.tools.ui.SelectionEvent;
import ec4.tools.ui.SelectionListener;
import ec4.tools.ui.SimpleScrollPane;
import ec4.tools.ui.Splitter;
import ec4.tools.ui.URLEvent;
import ec4.tools.ui.URLListener;

/**
 * Das Applet fr EasyChat mit farbiger Textanzeige
 *
 * @since 0.4
 * @version 0.6
 */
public class ColorApplet extends java.applet.Applet
implements EC3Listener, InputFrameCallBack, ChatRoomListCallBack, URLListener,
 KeyListener, ActionListener, ItemListener, SelectionListener {
	
	/**
	 * das Eingabefenster fr die Awaynachricht
	 */
	private InputFrame awayInput;
	
	/**
	 * der AudioClip fr "Neue Nachricht"
	 */
	private AudioClip beepAudioClip;
	
	/**
	 * die aktuelle Chatfarbe
	 */
	private String chatColor;
	
	/**
	 * der aktuelle Chatname
	 */
	private String chatName;
	
	/**
	 * das aktuelle Passwort
	 */
	private String chatPassword;
	
	/**
	 * der aktuelle Chatraum
	 */
	private String chatRoom;
	
	/**
	 * das Passwort fr den Chatraum
	 */
	private String chatRoomPassword;
	
	/**
	 * das Protokollobjekt
	 */
	private ClientImpl impl;
	
	/**
	 * das Panel fr das Eingabefeld unten
	 */
	private Panel inputPanel;
	
	/**
	 * das Panel fr die Buttons oben
	 */
	private Panel buttonPanel;
	
	/**
	 * das Panel fr die Userliste links
	 */
	private Panel userListPanel;
	
	/**
	 * der Splitter zwischen inputPanel und Chattext
	 */
	private Splitter mainSplitter;
	
	/**
	 * der Splitter zwischen Userliste und Chattext
	 */
	private Splitter splitter2;
	
	/**
	 * der Port
	 */
	private int port;
	
	/**
	 * das Fenster fr die Chatraumliste
	 */
	private ChatRoomList roomList;
	
	/**
	 * der "Senden"-Button
	 */
	private Button sendButton;
	
	/**
	 * der "Verbinden"-Button
	 */
	private Button connectButton;
	
	/**
	 * der "Away"-Button
	 */
	private Button awayButton;
	
	/**
	 * der "Chatraum"-Button
	 */
	private Button chatroomButton;
	
	/**
	 * der Server
	 */
	private String server;
	
	/**
	 * das Anzeigefeld dpr den Chattext
	 */
	/*private*/ ColorTextArea chatTextArea;
	
	/**
	 * die ScrollPane fr den Chattext
	 */
	private SimpleScrollPane textAreaScrollPane;
	
	/**
	 * das Eingabefeld
	 */
	private TextArea inputTextArea;
	
	/**
	 * die Daten der Userliste
	 */
	private Hashtable userList;
	
	/**
	 * die ScrollPane fr die Userlsite
	 */
	private ScrollPane userListScrollPane;
	
	/**
	 * der SPeicher fr die Befehlhistory
	 */
	private StringFIFO history;
	
	/**
	 * die aktuelle Position in der Befehlhistory
	 */
	private int historyPos;
	
	/**
	 * eine Liste aller ignorierter Usernamen
	 */
	private Hashtable ignoreList;
	
	/**
	 * eine Liste aller Usernamen, an die <u>nicht</u> gesendet werden soll.
	 */
	private Hashtable notSendToList;
	
	/**
	 * true, wenn Sound abgespielt werden soll
	 */
	private boolean sound = true;
	
	/**
	 * Gibt an, ob das Eingabefeld mehrzeilig benutzt werden soll.
	 * <br>
	 * Werte:<br>
	 * 0 - kein Multiline<br>
	 * 1 - Multiline<br>
	 * 2 - Multiline; 2x Enter sendet
	 */
	private int multiline = 0;
	
	/**
	 * <code>true</code>, wenn als letztes Enter in der Einganezeile gedrckt wurde.
	 */
	private boolean lastEnter = false;
	
	/* 
	 * (non-Javadoc)
	 * @see ec3.client.applet.ChatRoomListCallBack#chatRoomListCallBack(java.lang.String, java.lang.String, java.lang.String, java.lang.String)
	 */
	public void chatRoomListCallBack (String room, String pwd, String desc, int action) {
		
		//Chatraum betreten
		if (action == JOIN) {
			
			implCallBack ("joinChatRoom", new String[] {room, pwd});
			
		//Chatraum erstellen
		} else if (action == CREATE) {
			
			implCallBack ("createChatRoom", new String[] {room, desc, pwd});
			
		}//if
		
	}//chatRoomListCallBack(...)
	
	/* 
	 * (non-Javadoc)
	 * @see java.applet.Applet#destroy()
	 */
	public void destroy () {
		
		impl.disconnect ();
		
	}//destroy()
	
	public void EC3CallBack (EC3Event e) {
		
		//Sender und Daten ermitteln
		Object source = e.getSource ();
		String event = e.getEvent ();
		String[] data = e.getData ();
		//Protokollobjekt
		if (source == impl) {
			
			//Fehler des Socket.Objekts
			if (event.equals ("aborted")) {
				// Userliste leeren
				userListPanel.removeAll ();
				userList.clear ();
				
				if (data[0].equals ("badDataLength")) {
					
					addError ("Socketfehler", "die Datenlnge enthielt ungltige oder zu viele Zeichen.");
					
				} else if (data[0].equals ("connectionRefused")) {
					
					addError ("Socketfehler", "kann keine Verbindung zum Server herstellen. (" + data[1] + ")");
					
				} else if (data[0].equals ("maxCommandLength")) {
					
					addError ("Socketfehler", "Die maximale Befehlslnge wurde berschritten.");
					
				} else if (data[0].equals ("maxDataLength")) {
					
					addError ("Socketfehler", "Die maximale Datenlnge wurde berschritten.");
					
				} else if (data[0].equals ("networkUnreachable")) {
					
					addError ("Socketfehler", "Netzwerk nicht erreichbar. (" + data[1] + ")");
					
				} else if (data[0].equals ("noHeaderEndMark")) {
					
					addError ("Socketfehler", "kein Headerendezeichen gefunden.");
					
				} else  if (data[0].equals ("security")) {
					
					addError ("Socketfehler", "Security-Manager verweigert Verbindungsaufbau. (" + data[1] + ")");
					
				} else  if (data[0].equals ("sendError")) {
					
					addError ("Socketfehler", "Konnte nicht senden.");
					
				} else  if (data[0].equals ("serverClosedConnection")) {
					
					addError ("Socketfehler", "Server hat unerwartet die Verbindung beendet.");
					
				} else  if (data[0].equals ("SocketException")) {
					
					addError ("Socketfehler", "Es ist ein Fehler im TCP-Protokoll aufgetreten. (" + data[1] + ")");
					
				} else if (data[0].equals ("unknownHost")) {
					
					addError ("Socketfehler", "unbekannter Host. (" + data[1] + ")");
					
				} else {
					
					addError (
						"Socketfehler",
						data[0] + "\n" +
						"Bitte benachrichtigen Sie den Autor dieses Programmes (Andreas Mann)"
					);
					
				}//if
				
			//Chatraum betreten
			} else if (event.equals ("acceptChatRoom")) {
				
				addSysMsg ("System", "Chatraum \"" + data[0] + "\" betreten.");
				chatRoom = data[0];
				implCallBack ("getUserList", Tools.emptyStringArray);
				
			//Login erfolgreich
			} else if (event.equals ("acceptUser")) {
				
				awayButton.setEnabled (true);
				chatroomButton.setEnabled (true);
				
				//nach einem Chatraum zum betreten suchen
				//Reihenfolge: Bookmark => Serverstandard => einzigster des Servers => Liste
				if (chatRoom.equals ("")) {
					
					chatRoom = impl.getProperty ("server.stdChatRoom", "");
					if (chatRoom.equals ("")) {
						
						chatRoom = impl.getProperty ("server.onlyChatRoom", "");
						if (chatRoom.equals ("")) {
							
							implCallBack ("getChatRoomList", Tools.emptyStringArray);
							
						} else {
							
							implCallBack ("joinChatRoom", new String[] {chatRoom});
							
						}//if
						
					} else {
						
						implCallBack ("joinChatRoom", new String[] {chatRoom});
						
					}//if
					
				} else {
					
					implCallBack ("joinChatRoom", new String[] {chatRoom, chatRoomPassword});
					
				}//if
				
			//User wird Admin
			} else if (event.equals ("admin")) {
				
				//data[0] - Name
				UserListItem item = (UserListItem)userList.get (data[0].toLowerCase ());
				if (item != null) {
					
					item.setAdmin (true);
					buildLabel (item);
					
				}//if
				userListPanel.validate ();
				
				boolean ignored = false;
				if (item != null) {
					ignored = item.isIgnored();
				}
				if (!ignored) {
					addSysMsg ("System", data[0] + " wurde zum Admin ernannt.");
				}
				
			//User ist away
			} else if (event.equals ("away")) {
				
				//data[0] - User
				//data[1] - Nachricht
				UserListItem item = (UserListItem)userList.get (data[0].toLowerCase ());
				if (item != null) {
					
					item.setAway (true);
					buildLabel (item);
					
				}//if
				userListPanel.validate ();
				
				boolean ignored = false;
				if (item != null) {
					ignored = item.isIgnored();
				}
				if (!ignored) {
					if (data[1].equals ("")) {
						
						addSysMsg ("System", data[0] + " ist jetzt away.");
						
					} else {
						
						addSysMsg ("System", data[0] + " ist jetzt away: \"" + data[1] + "\".");
						
					}//if
				}
				
			//awayMessage Empfangeg
			} else if (event.equals ("awayMessage")) {
				
				//data[0] - Name
				//data[1] - Nachricht
				
				addSysMsg ("System", "Die Away-Nachricht von " + data[0] + " lautet: \"" + data[1] + "\".");
				
			//awayMessage Empfangeg
			} else if (event.equals ("badChatRoomPwd")) {
				
				addSysMsg ("System", "Falsches Passwort fr den Chatraum.");
				
			//Name beim Login bereits vergeben
			} else if (event.equals ("chatNameUsed")) {
				
				addSysMsg ("System", "Der Chatname ist bereits vergeben.");
				impl.disconnect ();
				
			//Chatraumliste empfangen
			} else if (event.equals ("chatRoomList")) {
				
				roomList.setList (data);
				roomList.setVisible(true);
				
			//Chatraumerstellung verboten
			} else if (event.equals ("chatRoomCreateNotAllowed")) {
				
				addSysMsg ("System", "Der Chatraum konnte nicht erstellt werden.\nGrund: Der Server verbietet das Erstellen von Chatrumen.");
				implCallBack ("getChatRoomList", Tools.emptyStringArray);
				
			//Chatraumerstellung nicht mglich; existiert bereits
			} else if (event.equals ("chatRoomNotCreated")) {
				
				addSysMsg ("System", "Der Chatraum konnte nicht erstellt werden.\nGrund: Es existiert bereits ein Chatraum mit dem angegeben Namen.");
				implCallBack ("getChatRoomList", Tools.emptyStringArray);
				
			//Chatraum nicht gefunden
			} else if (event.equals ("chatRoomNotFound")) {
				
				if (impl.getProperty ("server.canCreateChatRoom").equals ("0")) {
					
					addSysMsg ("System", "Der Chatraum konnte nicht betreten werden.\nGrund: Der Chatraum wurde nicht gefunden.");
					implCallBack ("getChatRoomList", Tools.emptyStringArray);
					
				} else {
					
					implCallBack ("createChatRoom", new String[] {chatRoom, chatRoomPassword});
					
				}
			
			//closeChatRoom
			} else if (event.equals ("closeChatRoom")) {
				
				addSysMsg ("System", "Der Chatraum wurde serverseitig geschlossen.");
				userListPanel.removeAll ();
				userList.clear ();
				userListPanel.validate ();
				implCallBack ("getChatRoomList", Tools.emptyStringArray);
				
			//Aufforderung, nach /quit das Fenster zu schlieen
			} else if (event.equals ("closeWindow")) {
				try {
					String closeDocument = getParameter("CloseDocument");
					if (closeDocument == null) {
						closeDocument = "close.html";
					}
					getAppletContext().showDocument(new URL(getCodeBase(), closeDocument));
				} catch (MalformedURLException e1) {
					// sollt net passieren
				}
			//Verbindung erfolgreich
			} else if (event.equals ("connect")) {
				
				addSysMsg ("System", "Verbindung wird aufgebaut...");
				
			//daten vom /connect-Befehl
			} else if (event.equals ("connectData")) {
				
				server = data[0];
				try {
					
					port = new Integer (data[1]).intValue ();
					
				} catch (NumberFormatException e_) {
					// kann net vorkommen
				}
				chatName = data[2].equals ("") ? chatName : data[2];
				chatPassword = data[3].equals ("") ? chatPassword : data[3];
				chatRoom = "";
				chatRoomPassword = "";
				
			//Verbindung getrennt
			} else if (event.equals ("disconnect")) {
				// Userliste leeren
				userListPanel.removeAll ();
				userList.clear ();
				
				addSysMsg ("System", "Verbindung getrennt");
				awayButton.setLabel ("Awaymodus aktivieren");
				awayButton.setEnabled (false);
				chatroomButton.setEnabled (false);
				
			// Nachrichten ber Verschlsselung
			} else if (event.equals("encReady")) {
				
				if (data[0].equals("ok")) {
					addSysMsg ("System", "Verschlsselung " + data[1] + " aktiv");
				} else if (data[0].equals("no")) {
//					addSysMsg ("System", "kein Verschlsselungsplugin aktiv");//TODO
				} else if (data[0].equals("notFound")) {
					addSysMsg ("System", "kein Verschlsselungsplugin kompatibel zum Server");
				} else if (data[0].equals("badEncryption")) {
					addSysMsg ("System", "der Server akzeptiert die Verschlsselung nicht");
				} else if (data[0].equals("badCRC")) {
					addSysMsg ("System", "der Server schickte ungltige Verschlsselungsdaten");
				} else if (data[0].equals("serverError")) {
					addSysMsg ("System", "der Server akzeptierte die Verschlsselungsdaten nicht");
				} else {
					addSysMsg ("System", "Meldung der Verschlsselung: " + data[0]);
				}
				
			//error
			} else if (event.equals ("error")) {
				
				if (data[0].equals("welcome/serverProtoVersion")) {
					addError("Protokollfehler", "Client und Server sind nicht " +
						"kompatibel:\nClient untersttzt: " + data[1] + 
						"\nServer untersttzt: " + data[2]
					);
				} else {
					addError ("Protokollfehler", data[0] +
						"\nBitte benachrichtigen Sie den Programmierer."
					);
				}
				
			//Login angefordert
			} else if (event.equals ("login")) {
				
				implCallBack ("login", new String[] {chatName, chatPassword, chatColor});
				
			//Falsches Login-Passwort
			} else if (event.equals ("loginDenied")) {
				
				addSysMsg ("System", "Das Login-Passwort war falsch.");
				impl.disconnect ();
				
			//Zielhost aufgelst
			} else if (event.equals ("lookup")) {
				
				addSysMsg ("System", "Zielhost: " + data[0]);
				
			//"me"-Nachricht
			} else if (event.equals ("meMessage")) {
				
				boolean ignore = false;
				UserListItem item = (UserListItem)userList.get (data[0].toLowerCase ());
				if (item != null) {
					
					ignore = item.isIgnored ();
					
				}//if
				
				if (data[0].equals (chatName) || !ignore) {
					
					addMessage ("System", data[0] + " " + data[1], Tools.long2Color(data[2]));
					if (!data[0].equals (chatName)) {
						
						beep();
						
					}
					
				}//if
				
			//newColor
			} else if (event.equals ("newColor")) {
				
				//data[0] - Name
				//data[1] - Farbe
				
				Color c = Tools.long2Color(data[1]);
				
				UserListItem item = (UserListItem)userList.get(data[0].toLowerCase());
				if (item != null) {
					item.setColor(c);
					buildLabel(item);
				}
				
				boolean ignored = false;
				if (item != null) {
					ignored = item.isIgnored();
				}
				if (!ignored) {
					addSysMsg("System", data[0] + " hat "
					 + ColorTextArea.makeFgColor("die Farbe gewechselt.", c));
				}
				
			//Kein Gastzugang erlaubt
			} else if (event.equals ("noGuestsAllowed")) {
				
				addSysMsg ("System", "Kein Gastzugang erlaubt.");
				impl.disconnect ();
				
			//normale Nachricht
			} else if (event.equals ("normalMessage")) {
				
				boolean ignore = false;
				UserListItem item = (UserListItem)userList.get (data[0].toLowerCase ());
				if (item != null) {
					
					ignore = item.isIgnored ();
					
				}//if
				
				if (data[0].equals (chatName) || !ignore) {
					
					addMessage (data[0], data[1], Tools.long2Color(data[2]));
					if (!data[0].equals (chatName)) {
						
						beep();
						
					}
					
				}//if
				
			//notAdmin
			} else if (event.equals ("notAdmin")) {
				
				//data[0] - Name
				UserListItem item = (UserListItem)userList.get (data[0].toLowerCase ());
				if (item != null) {
					
					item.setAdmin (false);
					buildLabel (item);
					
				}//if
				userListPanel.validate ();
				boolean ignored = false;
				if (item != null) {
					ignored = item.isIgnored();
				}
				if (!ignored) {
					addSysMsg ("System", data[0] + " wurde der Admin-Rang entzogen.");
				}
			//notAway
			} else if (event.equals ("notAway")) {
				
				//data[0] - Name
				UserListItem item = (UserListItem)userList.get (
					data[0].toLowerCase ()
				);
				if (item != null) {
					
					item.setAway (false);
					buildLabel (item);
					
				}//if
				userListPanel.validate ();
				boolean ignored = false;
				if (item != null) {
					ignored = item.isIgnored();
				}
				if (!ignored) {
					addSysMsg ("System", data[0] + " ist jetzt wieder da.");
				}
				
			//Hinweise (Falscheingaben des Users etc.)
			} else if (event.equals ("notice")) {
				
				if (data[0].equals ("syntax//")) {
					
					addSysMsg ("Hinweis", "Syntax des Befehls:\n" +
					 "// Nachricht\n" +
					 "Es wird eine private Nachricht an den" +
					 " letzten Empfnger einer privaten Nachricht gesendet.\n" +
					 "Es mu also vorher mindestens 1x der Befehl" +
					 " /msg ausgefhrt worden sein.");
					
				} else if (data[0].equals ("syntaxMe")) {
					
					addSysMsg ("Hinweis", "Syntax des Befehls:\n" +
					 "/me Nachricht\n" +
					 "Die Nachricht beschreibt eine Ttigkeit.\n" +
					 "Ihr Name wird der Nachricht automatisch vorangestellt.");
					
				} else if (data[0].equals ("syntaxMsg")) {
					
					addSysMsg ("Hinweis", "Syntax des Befehls:\n" +
					 "/msg Empfnger Nachricht\n" +
					 "Die private Nachricht erhlt nur der Empfnger.");
					
				} else if (data[0].equals ("syntaxS")) {
					
					addSysMsg ("Hinweis", "Syntax des Befehls:\n" +
					 "/s Nachricht\n" +
					 "Die Schrei-Nachricht wird besonders hervorgehoben.");
					
				} else if (data[0].equals ("syntaxConnect")) {
					
					addSysMsg ("Hinweis", "Syntax des Befehls:\n" +
					 "/connect [server:[port] [chatname [passwort]]]\n" +
					 "Diese Meldung erscheint immer dann," +
					 " wenn nicht genug Informationen angegeben sind."
					);
					
				} else if (data[0].equals ("syntaxCreate")) {
					
					String[] res = new String[] {null};
					implCallBack ("encryptionSupported", res);
					
					if (res[0].equals("1")) {
						addSysMsg ("Hinweis", "Syntax des Befehls:\n" +
						 "/create Raumname [Beschreibung] [Passwort]\n" +
						 "Erstellt einen neuen Chatraum mit den angegebenen Daten." +
						 " Passwrter funktionieren nur, wenn Verschlsselung aktiv ist."
						);
					} else {
						addSysMsg ("Hinweis", "Syntax des Befehls:\n" +
						 "/create Raumname [Beschreibung]\n" +
						 "Erstellt einen neuen Chatraum mit den angegebenen Daten."
						);
					}
					
				} else if (data[0].equals ("syntaxGam")) {
					
					addSysMsg ("Hinweis", "Syntax des Befehls:\n" +
					 "/getawaymsg Username\n" +
					 "Fragt die Awaynachricht des Users ab.");
					
				} else if (data[0].equals ("unknownCommand")) {
					
					// schauen, ob es nicht ein Befehl ist, mit dem eh nur das
					// FrontEnd was anfangen kann:
					int pos = data[1].indexOf(" ");
					String command;
					String params;
					if (pos < 0) {
						command = data[1];
						params = "";
					} else {
						command = data[1].substring(0, pos);
						params = data[1].substring(pos + 1);
					}
					// Chat lschen
					if (command.equals("/clear")
					 || command.equals("/cls")) {
						chatTextArea.clear();
					// Farbe ndern
					} else if (command.equals("/color")
					 || command.equals("/c")) {
						if (params.equals("")) {
							//TODO Farbdialog anzeigen
							addSysMsg ("Hinweis",
								"Syntax: /color neue_Farbe (als" +
								" Konstante oder als HTML-Farbwert)");
						} else {
							String col = Tools.parseColor (params);
							Color c = Tools.long2Color(col);
							if (Tools.testColor(c)) {
								implCallBack ("setChatColor",
									new String[] {col}
								);
							} else {
								addSysMsg ("Hinweis", "Die Farbe ist zu hell");
							}
						}//if
					// Schrift ndern
					} else if (command.equals("/font")) {
						if (params.trim().equals("")) {
							addSysMsg ("Hinweis", "Syntax des Befehls:\n" +
								"/font Schrift [Gre]\n" +
								"wenn nur die Gre gendert werden soll, dann bei Schrift \"\" angeben."
							);
						} else {
							try {
								String[] paramSplit = Tools.splitArguments(params, 2);
								Font f = chatTextArea.getFont();
								if (!paramSplit[0].equals("")) {
									f = new Font(paramSplit[0], Font.PLAIN, f.getSize());
								}
								if (!paramSplit[1].equals("")) {
									f = new Font(f.getName(), Font.PLAIN, Integer.parseInt(paramSplit[1]));
								}
								chatTextArea.setFont(f);
							} catch (NumberFormatException _e) {
								addSysMsg ("Hinweis", "Syntax des Befehls:\n" +
									"/font Schrift [Gre]\n" +
									"wenn nur die Gre gendert werden soll, dann bei Schrift \"\" angeben."
								);
							}
						}
					// ignorieren/nicht ignorieren
					} else if (command.equals("/ignore")
					 || command.equals("/i")) {
						// kein Name => Liste anzeigen
						if (params.equals("")) {
							Vector liste = new Vector ();
							for (Enumeration e_ = ignoreList.elements(); e_.hasMoreElements ();) {
								liste.addElement (e_.nextElement());
							}
							String[] temp = new String[liste.size ()];
							liste.copyInto (temp);
							addSysMsg ("System", "ignorierte User:\n" + Tools.join (temp, "\n"));
						// ignore-Flag umschalten
						} else {
							UserListItem item = (UserListItem)userList.get (params.toLowerCase ());
							if (item != null) {
								if (item.isIgnored()) {
									item.setIgnored (false);
									ignoreList.remove(params.toLowerCase ());
									addSysMsg ("System", params + " wird ab sofort nicht mehr ignoriert.");
								} else {
									item.setIgnored (true);
									item.setSendTo(false);
									ignoreList.put(params.toLowerCase (), params);
									notSendToList.put(params.toLowerCase (), params);
									addSysMsg ("System", params + " wird ab sofort ignoriert.");
								}
								buildLabel (item);
								userListPanel.validate ();
							} else {
								if (ignoreList.containsKey(params.toLowerCase ())) {
									ignoreList.remove(params.toLowerCase ());
									addSysMsg ("System", params + " wird ab sofort nicht mehr ignoriert.");
								} else {
									ignoreList.put(params.toLowerCase (), params);
									notSendToList.put(params.toLowerCase (), params);
									addSysMsg ("System", params + " wird ab sofort ignoriert.");
								}
							}//if
						}//if
					//Multiline umschalten
					} else if (command.equals("/ml") || command.equals("/multiline")) {
						if (params.equals("")) {
							multiline = (multiline > 0 ? 0 : 1);
							if (multiline > 0) {
								addSysMsg ("System", "Multiline aktiviert");
							} else {
								addSysMsg ("System", "Multiline deaktiviert");
							}
						} else {
							params = params.trim();
							if ("0".equals(params)) {
								multiline = 0;
								addSysMsg ("System", "Multiline deaktiviert");
							} else if ("1".equals(params)) {
								multiline = 1;
								addSysMsg ("System", "Multiline aktiviert");
							} else if ("2".equals(params)) {
								multiline = 2;
								addSysMsg ("System", "Multiline aktiviert");
								addSysMsg ("System", "Senden mit 2x Enter aktiviert");
							} else {
								addError("System", params + " ist kein gltiger Zustand");
								addSysMsg("System", "mglich sind:");
								addSysMsg("System", "0 - kein Multiline");
								addSysMsg("System", "1 - Multiline aktiv");
								addSysMsg("System", "2 - Multiline aktiv: senden mit 2x Enter mglich");
							}
						}
					//Serverinfos anzeigen
					} else if (command.equals("/nfo")) {
						addSysMsg ("System", "Serverinformationen:\n" +
							"Host: " + server + "\n" +
							"Version: " + impl.getProperty("server.version") + "\n" +
							"Sprache: " + impl.getProperty("server.lang") + "\n" +
							"max. Nachrichtenlnge: " + impl.getProperty("server.maxTextLen") + "\n" +
							"Admin: " + impl.getProperty("server.admin") + " <" + impl.getProperty("server.adminMail") + ">\n"
						);
					//senden an User ein/ausstellen
					} else if (command.equals("/sendto")) {
						// kein Name => Liste anzeigen
						if (params.equals("")) {
							Vector liste = new Vector ();
							for (Enumeration e_ = notSendToList.elements(); e_.hasMoreElements ();) {
								liste.addElement (e_.nextElement());
							}
							String[] temp = new String[liste.size ()];
							liste.copyInto (temp);
							addSysMsg ("System", "User, an die nicht gesendet wird:\n" + Tools.join (temp, "\n"));
						// sendTo-Flag umschalten
						} else {
							UserListItem item = (UserListItem)userList.get (params.toLowerCase ());
							if (item != null) {
								if (item.sendTo()) {
									item.setSendTo(false);
									notSendToList.put(params.toLowerCase (), params);
									addSysMsg ("System", "an " + params + " wird ab sofort nicht mehr gesendet.");
								} else {
									item.setSendTo(true);
									notSendToList.remove(params.toLowerCase ());
									addSysMsg ("System", "an " + params + " wird ab sofort wieder gesendet.");
								}
								buildLabel (item);
								userListPanel.validate ();
							} else {
								if (notSendToList.containsKey(params.toLowerCase ())) {
									notSendToList.remove(params.toLowerCase ());
									addSysMsg ("System", "an " + params + " wird ab sofort wieder gesendet.");
								} else {
									notSendToList.put(params.toLowerCase (), params);
									addSysMsg ("System", "an " + params + " wird ab sofort nicht mehr gesendet.");
								}
							}//if
						}//if
					// Sound an/aus
					} else if (command.equals("/sound")) {
						
						params = params.toLowerCase();
						if (params.equals("")) {
							sound = !sound;
						} else if (params.equals("1")
						 || params.equals("an") || params.equals("on")
						 || params.equals("true") || params.equals("wahr")
						 || params.charAt(0) == 'y' || params.charAt(0) == 'j') {
							sound = true;
						} else if (params.equals("0")
						 || params.equals("aus") || params.equals("off")
						 || params.equals("false") || params.equals("falsch")
						 || params.charAt(0) == 'n') {
							sound = false;
						} else {
							addSysMsg("System", "Unbekannter Zustand: " + params);
						}
						
						if (sound) {
							addSysMsg("System", "Sound ist jetzt: an");
						} else {
							addSysMsg("System", "Sound ist jetzt: aus");
						}
					// nix gltiges
					} else {
						addSysMsg ("Hinweis",
							data[1] + " ist kein gltiger Befehl."
						);
					}
					
				} else if (data[0].equals ("notInChatRoom")) {
					
					addSysMsg ("Hinweis",
						"Sie mssen in einem Chatraum angemeldet" +
						" sein, um diese Funktion zu nutzen."
					);
					
				} else if (data[0].equals ("notLoggedIn")) {
					
					addSysMsg ("Hinweis", "Sie mssen an einem Server angemeldet sein, um diese Funktion zu nutzen.");
					
				} else if (data[0].equals ("waitReconnect")) {
					
					addSysMsg ("Hinweis", "Warte 30 Sekunden bis zum nchsten Verbindungsversuch...");
					
				} else if (data[0].equals ("noPingReply")) {
					
					addSysMsg ("Hinweis", "Server hat innerhalb von 60 Sekunden nicht auf ping geantwortet...");
					
				} else {
					
					addSysMsg ("Hinweis", data[0]);
					
				}//if
				
			//private Nachricht
			} else if (event.equals ("privateMessage")) {
				
				boolean ignore = false;
				UserListItem item = (UserListItem)userList.get (data[0].toLowerCase ());
				if (item != null) {
					
					ignore = item.isIgnored ();
					
				}//if
				
				if (data[0].equals (chatName) || !ignore) {
					
					addMessage (data[0] + " @ " + data[1], data[2], Tools.long2Color(data[3]), true);
					if (!data[0].equals (chatName)) {
						
						beep();
						
					}
					
				}//if
				
			//Nachricht vom Server
			} else if (event.equals ("serverMessage")) {
				
				addSysMsg ("Server", data[0]);
				
			// Server hat falsche Protokollversion
			} else if (event.equals("serverProtoVersion")) {
				
				addError("System", "Der Server lehnt die Protokollversion dieses Clients" +
					" (" + data[0] + ") ab"
				);
				
			//Awayodus per Kommandozeile gendert
			} else if (event.equals ("setAway")) {
				
				//data[0] - Awaymodus
				if (data[0].equals ("0")) {
					
					awayButton.setLabel ("Awaymodus aktivieren");
					
				} else {
					
					awayButton.setLabel ("Awaymodus deaktivieren");
					
				}//if
				
			//"schrei"-Nachricht
			} else if (event.equals ("shoutMessage")) {
				
				boolean ignore = false;
				UserListItem item = (UserListItem)userList.get (data[0].toLowerCase ());
				if (item != null) {
					
					ignore = item.isIgnored ();
					
				}//if
				
				if (data[0].equals (chatName) || !ignore) {
					
					addMessage ("System", data[0] + " schreit \"" + data[1].toUpperCase () + "\".", Tools.long2Color(data[2]));
					if (!data[0].equals (chatName)) {
						
						beep();
						
					}
					
				}//if
				
			//unkebannte Befehle (Server/Frontend)
			} else if (event.startsWith ("unknown")) {
				
				addError ("Fehler", event + ": " + data[0] + "\nBitte benachrichtigen Sie den Programmierer.");
				
			//Userdetails
			} else if (event.equals("userDetails")) {
				
				addSysMsg("System", "Userdetails von " + data[0]);
				addText(
					ColorTextArea.makeUnderline("Person\n")
					+ "Name: " + (data[1].equals("") ? "nicht angegeben" : data[1]) + "\n"
					+ "Geschlecht: " + (data[2].equals("1") ? "mnnlich" : data[2].equals("2") ? "weiblich" : "nicht angegeben") + "\n"
					+ "Geburtstag: " + formatBirthday(data[3]) + "\n"
					+ "Hobbies: " + (data[4].equals("") ? "nicht angegeben" : data[4]) + "\n"
					+ "ICQ: " + (data[5].equals("") ? "nicht angegeben" : data[5]) + "\n"
					+ "AIM: " + (data[6].equals("") ? "nicht angegeben" : data[6]) + "\n"
					+ "YIM: " + (data[7].equals("") ? "nicht angegeben" : data[7]) + "\n"
					+ "MSN: " + (data[8].equals("") ? "nicht angegeben" : data[8]) + "\n"
					+ "eMail: " + (data[9].equals("") ? "nicht angegeben" : data[9]) + "\n"
					+ "Homepage: " + (data[10].equals("") ? "nicht angegeben" : "http://" + data[10]) + "\n"
					+ "Sprachen: " + (data[11].equals("") ? "nicht angegeben" : data[11]) + "\n"
					+ "Sonstiges: " + (data[12].equals("") ? "nicht angegeben" : data[12]) + "\n"
					+ "Job: " + (data[13].equals("") ? "nicht angegeben" : data[13]) + "\n"
					+ "Bildlink: " + (data[14].equals("") ? "nicht angegeben" : data[14]) + "\n"
					+ ColorTextArea.makeUnderline("Hauptwohnsitz\n")
					+ "Land: " + (data[15].equals("") ? "nicht angegeben" : data[15]) + "\n"
					+ "Bundesland: " + (data[16].equals("") ? "nicht angegeben" : data[16]) + "\n"
					+ "Strae: " + (data[17].equals("") ? "nicht angegeben" : data[17]) + "\n"
					+ "Postleitzahl: " + (data[18].equals("") ? "nicht angegeben" : data[18]) + "\n"
					+ "Ort: " + (data[19].equals("") ? "nicht angegeben" : data[19]) + "\n"
					+ ColorTextArea.makeUnderline("Nebenwohnsitz\n")
					+ "Land: " + (data[20].equals("") ? "nicht angegeben" : data[20]) + "\n"
					+ "Bundesland: " + (data[21].equals("") ? "nicht angegeben" : data[21]) + "\n"
					+ "Strae: " + (data[22].equals("") ? "nicht angegeben" : data[22]) + "\n"
					+ "Postleitzahl: " + (data[23].equals("") ? "nicht angegeben" : data[23]) + "\n"
					+ "Ort: " + (data[24].equals("") ? "nicht angegeben" : data[24]) + "\n"
					+ ColorTextArea.makeUnderline("Rechner\n")
					+ "CPU: " + (data[25].equals("") ? "nicht angegeben" : data[25]) + "\n"
					+ "RAM: " + (data[26].equals("") ? "nicht angegeben" : data[26]) + "\n"
					+ "Grafikkarte: " + (data[27].equals("") ? "nicht angegeben" : data[27]) + "\n"
					+ "Soundkarte: " + (data[28].equals("") ? "nicht angegeben" : data[28]) + "\n"
					+ "Festplatte: " + (data[29].equals("") ? "nicht angegeben" : data[29]) + "\n"
					+ "Verbindung: " + formatConnection(data[30]) + "\n"
					+ "Betriebssystem: " + (data[31].equals("") ? "nicht angegeben" : data[31]) + "\n"
					+ "Sonstiges: " + (data[32].equals("") ? "nicht angegeben" : data[32]) + "\n"
				);
				
			//Userdetails-Fehler
			} else if (event.equals("userDetailsError")) {
				
				if (data[0].equals("usernotfound")) {
					addError("Userdetails", "Der Benutzer " + data[1] + " wurde nicht gefunden");
				} else if (data[0].equals("pluginnotfound")) {
					addError("Userdetails", "Der Benutzer " + data[1] + " hat das Plugin nicht (aktiviert)");
				}
				
			//User hinzufgen
			} else if (event.equals ("userJoined")) {
				
				//data[0] - Name
				//data[1] - Farbe
				//data[2] - Awaystatus
				//data[3] - Adminstatus
				
				Color c = Tools.long2Color(data[1]);
				
				Checkbox checkbox = new Checkbox (data[0], true);
				checkbox.addItemListener(this);
				checkbox.setName(data[0]);
				checkbox.setBackground(userListPanel.getBackground());
				UserListItem item = new UserListItem (data[0], !data[2].equals ("0"), !data[3].equals ("0"), false, true, c, checkbox);
				if (ignoreList.containsKey(data[0].toLowerCase())) {
					item.setIgnored(true);
					item.setSendTo(false);
					notSendToList.put(data[0].toLowerCase(), data[0]);
				} else if (notSendToList.containsKey(data[0].toLowerCase())) {
					item.setSendTo(false);
				} else {
					addSysMsg ("System", data[0] + " hat den Chat betreten.");
				}
				buildLabel (item);
				userList.put (data[0].toLowerCase (), item);
				userListPanel.validate ();
				
			//User entfernen
			} else if (event.equals ("userLeft")) {
				
				//data[0] - Name
				//data[1] - neuer Chatraum (oder ! > Kick)
				//data[2] - Grund
				
				UserListItem item = (UserListItem)userList.get (data[0].toLowerCase ());
				if (item != null) {
					
					userListPanel.remove (item.getCheckbox ());
					userList.remove (data[0].toLowerCase ());
					
				}//if
				userListPanel.validate ();
				
				boolean ignored = false;
				if (item != null) {
					ignored = item.isIgnored();
				}
				if (!ignored) {
					if (data[1].equals ("")) {
						
						if (data[2].equals ("")) {
							
							addSysMsg ("System", data[0] + " hat den Chat verlassen.");
							
						} else {
							
							addSysMsg ("System", data[0] + " hat den Chat verlassen (Grund: " + data[2] + ")");
							
						}//if
						
					} else if (data[1].equals("!")) {
						
						addSysMsg ("System", data[0] + " wurde vom Server gekickt.");
						
					} else {
						
						addSysMsg ("System", data[0] + " hat in den Chat " + data[1] + " gewechselt.");
						
					}//if
				}
				
			//Userliste empfangen
			} else if (event.equals ("userList")) {
				
				//data[0] - Name
				//data[1] - Farbe
				//data[2] - away
				//data[3] - admin
				
				userListPanel.removeAll ();
				userList.clear();
				for (int i = 0; i < data.length / 4; i++) {
					
					String name = data[4 * i];
					Color c = Tools.long2Color(data[4 * i + 1]);
					boolean away = !data[4 * i + 2].equals ("0");
					boolean admin = !data[4 * i + 3].equals ("0");
					boolean ignored = ignoreList.containsKey(name.toLowerCase());
					boolean sendTo = !notSendToList.containsKey(name.toLowerCase());
					
					Checkbox checkbox = new Checkbox (data[4 * i], true);
					checkbox.addItemListener(this);
					checkbox.setName(data[4 * i]);
					checkbox.setBackground(userListPanel.getBackground());
					UserListItem item = new UserListItem (name, away, admin, ignored, sendTo, c, checkbox);
					buildLabel (item);
					userList.put (name.toLowerCase (), item);
					
				}//for
				userListPanel.validate ();
				
			//Willkommensnachricht
			} else if (event.equals ("welcome")) {
				
				addSysMsg ("System", "Verbindung hergestellt");
				addText (data[0]);
				
			//Unbekannte Befehle
			} else {
				
				addError ("Fehler", "unbekannter Befehl: " + event);
				
			}//if
			
		}//if
		
	}//EC3CallBack(...)
	
	/* 
	 * (non-Javadoc)
	 * @see java.applet.Applet#init()
	 */
	public void init() {
		
		String beepWaveFile = getParameter("NotifyWaveFile");
		if (beepWaveFile == null) {
			
			final String beepWaveCommand = getParameter("NotifyWaveCommand");
			
			if (beepWaveCommand == null) {
				
				beepAudioClip = getAudioClip(getClass().getClassLoader().getResource("ec3/client/applet/resources/notify.wav"));
				
			} else {
				
				beepAudioClip = new AudioClip() {
					
					public void loop() {
						// leer
					}
					
					public void play() {
						try {
							Runtime.getRuntime().exec(beepWaveCommand);
						} catch (IOException e) {
							e.printStackTrace();
						}
					}
					
					public void stop() {
						// leer
					}
				};
				
			}
			
		} else {
			
			beepAudioClip = getAudioClip(getDocumentBase(), beepWaveFile);
			
		}
		
		//Buttons
		connectButton = new Button ("neu Verbinden");
		connectButton.addActionListener(this);
		awayButton = new Button ("Awaymodus aktivieren");
		awayButton.addActionListener(this);
		chatroomButton = new Button ("Chatraumliste anzeigen");
		chatroomButton.addActionListener(this);
		awayButton.setEnabled (false);
		chatroomButton.setEnabled (false);
		
		//Panel fr die Buttons
		buttonPanel = new Panel ();
		buttonPanel.setLayout (new GridLayout (0, 3));
		buttonPanel.add (connectButton);
		buttonPanel.add (awayButton);
		buttonPanel.add (chatroomButton);
		
		//Eingabezeile
		inputTextArea = new TextArea("", 1, 1, TextArea.SCROLLBARS_VERTICAL_ONLY);
		inputTextArea.addKeyListener(this);
		sendButton = new Button ("Senden");
		sendButton.addActionListener(this);
		inputPanel = new Panel ();
		inputPanel.setLayout (new BorderLayout ());
		inputPanel.add ("East", sendButton);
		inputPanel.add ("Center", inputTextArea);
		
		//Chattext
		chatTextArea = new ColorTextArea ();
		textAreaScrollPane = new SimpleScrollPane();
		textAreaScrollPane.add(chatTextArea);
		textAreaScrollPane.setUnitIncrement(10);
		textAreaScrollPane.setBlockIncrement(100);
		chatTextArea.setUrlFgColor(null);
		//textArea.addURLPrefix("ec3link://");
		chatTextArea.addURLListener(this);
		if (getParameter("Font") != null) {
			String[] split = Tools.split(getParameter("Font"), ":");
			if (split.length == 2) {
				int size = 12;
				size = Integer.parseInt(split[1]);
				if (size > 0) {
					chatTextArea.setFont(new Font(split[0], Font.PLAIN, size));
				}
			}
		}
		if (getParameter("XSelection") != null) {
			chatTextArea.addSelectionListener(this);
		} else {
			chatTextArea.addKeyListener(this);
		}
		if (getParameter("Antialiasing") != null) {
			chatTextArea.setAntialiasing(true);
		}
		if (getParameter("NoScrollRepaint") != null) {
			chatTextArea.setScrollRepaint(false);
		}
		if (getParameter("NoSound") != null) {
			sound = false;
		}
		if ("0".equals(getParameter("MultiLine"))) {
			multiline = 0;
		} else if ("1".equals(getParameter("MultiLine"))) {
			multiline = 1;
		} else if ("2".equals(getParameter("MultiLine"))) {
			multiline = 2;
		} else if (getParameter("MultiLine") != null) {
			multiline = 1;
		}
		try {
			if (getParameter("NoDefaultSmilies") == null) {
				URL res = getClass().getClassLoader().getResource("ec3/client/applet/resources/smilies.cfg");
				if (res != null) {
					loadSmilies(res);
				}
			}
			String smilieUrl = getParameter("SmiliesCfg");
			if (smilieUrl != null) {
				URL url = new URL(smilieUrl);
				loadSmilies(url);
			}
		} catch (IOException e1) {
			addError("IO-Fehler", "Problem beim Laden der Smilies");
		}
		
		mainSplitter = new Splitter(Splitter.VERTICAL_SPLIT, textAreaScrollPane, inputPanel);
		mainSplitter.setSplitterResizeWeight(1);
		mainSplitter.setConstraint(25);
		
		//Userliste
		userListPanel = new Panel () {
			public Insets getInsets() {
				Insets i = super.getInsets();
				i.left += 3;
				return i;
			}
		};
		userListPanel.setBackground (Color.white);
		userListPanel.setLayout (new MyBoxLayout ());
		//userListPanel.setLayout (new GridLayout (0, 1));
		userListScrollPane = new ScrollPane ();
		userListScrollPane.setBackground (buttonPanel.getBackground());
		userListScrollPane.add (userListPanel);
		
		splitter2 = new Splitter(Splitter.HORIZONTAL_SPLIT);
		splitter2.setConstraint(50);
		if (getParameter("RightUserlist") != null) {
			splitter2.setFirstComponent(mainSplitter);
			splitter2.setSecondComponent(userListScrollPane);
			splitter2.setSplitterResizeWeight(1);
		} else {
			splitter2.setFirstComponent(userListScrollPane);
			splitter2.setSecondComponent(mainSplitter);
			splitter2.setSplitterResizeWeight(0);
		}
		
		// Hintergrundfarbe setzen
		if (getParameter("BGColor") != null) {
			Color c = Tools.long2Color(Tools.parseColor(getParameter("BGColor")));
			int sum = c.getRed() + c.getGreen() + c.getBlue();
			if (sum >= 255) {
				// nur helle Farben als Hintergrund zulassen 
				chatTextArea.setBackground(c);
				userListPanel.setBackground(c);
				inputTextArea.setBackground(c);
			}
		}
		
		//Das Applet selbst
		setLayout (new BorderLayout ());
		add ("North", buttonPanel);
		add ("Center", splitter2);
		
		inputTextArea.requestFocus ();
		
		//Protokoll-Objekt erstellen
		Vector extensions = new Vector();
		if (getParameter("noDefaultExtensions") != null) {
			// nur angegebene Extensions laden
			if (getParameter("loadExtension") != null) {
				String[] split = Tools.split(getParameter("loadExtension"), ",");
				for (int i = 0; i < split.length; i++) {
					String temp = split[i].trim();
					if (temp.length() > 0) {
						extensions.add(temp);
					}
				}
			}
		} else {
			// User-Extensions, die als "zuerst laden" markiert sind
			if (getParameter("loadExtensionFirst") != null) {
				String[] split = Tools.split(getParameter("loadExtensionFirst"), ",");
				for (int i = 0; i < split.length; i++) {
					String temp = split[i].trim();
					if (temp.length() > 0) {
						extensions.add(temp);
					}
				}
			}
			// die 2 Standardextensions, ohne die nix geht
			extensions.add("ec3.client.AwaySys");
			extensions.add("ec3.client.Standard");
			// normale User-Extensions
			if (getParameter("loadExtension") != null) {
				String[] split = Tools.split(getParameter("loadExtension"), ",");
				for (int i = 0; i < split.length; i++) {
					String temp = split[i].trim();
					if (temp.length() > 0) {
						extensions.add(temp);
					}
				}
			}
			// Reconnect und UserDetails nur, falls nicht deaktiviert
			if (getParameter("noReconnect") == null) {
				extensions.add("ec3.client.Reconnect");
			}
			if (getParameter("noUserDetails") == null) {
				extensions.add("ec3.client.UserDetails");
			}
			// Deny immer laden
			extensions.add("ec3.client.Deny");
			// User-Extensions, die als "zuletzt laden" markiert sind
			if (getParameter("loadExtensionLast") != null) {
				String[] split = Tools.split(getParameter("loadExtensionLast"), ",");
				for (int i = 0; i < split.length; i++) {
					String temp = split[i].trim();
					if (temp.length() > 0) {
						extensions.add(temp);
					}
				}
			}
		}
		// erzeuge Implementation mit angegebenen Extensions
		impl = new ClientImpl(this, (String[])extensions.toArray(new String[extensions.size()]));
		// frage, ob Verschlsselung aktiv
		String[] res = new String[] {null};
		implCallBack ("encryptionSupported", res);
		
		userList = new Hashtable ();
		awayInput = new InputFrame ("Awaynachricht eingeben", this);
		roomList = new ChatRoomList (this, res[0].equals("1"));
		ignoreList = new Hashtable();
		notSendToList = new Hashtable();
		
		String historySize = getParameter("HistorySize");
		if (historySize != null) {
			try {
				int hS = Integer.parseInt(historySize);
				history = new StringFIFO(hS, true);
			} catch (NumberFormatException e) {
				history = new StringFIFO(10, true);
			}
		} else {
			history = new StringFIFO(10, true);
		}
		historyPos = -1;
		
		//Parameter holen, null-Werte verhindern
		chatName = getParameter ("ChatName");
		if (chatName == null) {
			
			chatName = "";
			
		}//if
		
		chatPassword = getParameter ("ChatPassword");
		if (chatPassword == null) {
			
			chatPassword = "";
			
		}//if
		
		chatColor = getParameter ("ChatColor");
		if (chatColor == null) {
			
			chatColor = "0";
			
		} else {
			
			chatColor = Tools.parseColor (chatColor);
			
		}//if
		
		server = getParameter ("Server");
		if (server == null) {
			
			server = "";
			
		}//if
		
		String temp = getParameter ("Port");
		port = 0;
		if (temp != null) {
			
			try {
				
				port = Integer.parseInt (temp);
				if ((port > 65535) || (port < 0)) {
					
					port = 0;
					
				}//if
				
			} catch (NumberFormatException e) {
				
				port = 0;
				
			}//try
			
		}//if
		
		chatRoom = getParameter ("ChatRoom");
		if (chatRoom == null) {
			
			chatRoom = "";
			
		}//if
		
		chatRoomPassword = getParameter ("ChatRoomPassword");
		if (chatRoomPassword == null) {
			
			chatRoomPassword = "";
			
		}//if
		
		impl.setInitData (server, port, !chatName.equals (""));
	}//init()
	
	public void inputFrameCallBack (InputFrame source, String text) {
		
		//Awaynachricht eingegeben
		if (source == awayInput) {
			
			implCallBack ("setAway", new String[] {"1", text});
			
		}//if
		
	}//inputFrameCallBack(...)
	
	/* (non-Javadoc)
	 * @see java.awt.event.KeyListener#keyPressed(java.awt.event.KeyEvent)
	 */
	public void keyPressed(KeyEvent e) {
		if (e.getSource() == chatTextArea) {
			if ((e.getKeyCode() == KeyEvent.VK_C) && (e.getModifiers() == InputEvent.CTRL_MASK)) {
				e.consume();
				StringSelection s = new StringSelection(chatTextArea.getSelectedText());
				Toolkit.getDefaultToolkit().getSystemClipboard().setContents(s, s);
			} else if ((e.getKeyCode() == KeyEvent.VK_INSERT) && (e.getModifiers() == InputEvent.CTRL_MASK)) {
				e.consume();
				StringSelection s = new StringSelection(chatTextArea.getSelectedText());
				Toolkit.getDefaultToolkit().getSystemClipboard().setContents(s, s);
			}
		} else if (e.getSource() == inputTextArea) {
			if ((e.getKeyCode() == KeyEvent.VK_S) && (e.getModifiers() == InputEvent.ALT_MASK)) {
				e.consume();
				String text = inputTextArea.getText ();
				inputTextArea.setText ("");
				inputTextArea.requestFocus ();
				sendMessage (text);
			} else if ((e.getKeyCode() == KeyEvent.VK_A) && (e.getModifiers() == InputEvent.CTRL_MASK)) {
				e.consume();
				inputTextArea.setSelectionStart(0);
				inputTextArea.setSelectionEnd(inputTextArea.getText().length());
			} else if ((e.getKeyCode() == KeyEvent.VK_UP) && (e.getModifiers() == InputEvent.CTRL_MASK)) {
				e.consume();
				int historyMax = history.getElementCount() - 1;
				if (historyPos < historyMax) {
					historyPos++;
					inputTextArea.setText(history.get(historyPos));
				}
			} else if ((e.getKeyCode() == KeyEvent.VK_DOWN) && (e.getModifiers() == InputEvent.CTRL_MASK)) {
				e.consume();
				if (historyPos > -1) {
					historyPos--;
					if (historyPos > -1) {
						inputTextArea.setText(history.get(historyPos));
					} else {
						inputTextArea.setText("");
					}
				}
			} else if ((e.getKeyCode() == KeyEvent.VK_ENTER) && (e.getModifiers() == InputEvent.CTRL_MASK)) {
				if (this.multiline == 0) {
					// Multiline deaktiviert -> Ctrl+Enter erzeugt Zeilenumbruch
					int pos = this.inputTextArea.getCaretPosition();
					int selStart = this.inputTextArea.getSelectionStart();
					int selEnd = this.inputTextArea.getSelectionEnd();
					if (selEnd > selStart) {
						this.inputTextArea.replaceRange("\n", selStart, selEnd);
					} else {
						this.inputTextArea.insert("\n", pos);
					}
				} else {
					e.consume();
					String text = inputTextArea.getText ();
					inputTextArea.setText ("");
					inputTextArea.requestFocus ();
					sendMessage (text);
				}
			} else if (e.getKeyCode() == KeyEvent.VK_ENTER) {
				switch (this.multiline) {
					case 0:
						e.consume();
						String text = inputTextArea.getText ();
						inputTextArea.setText ("");
						inputTextArea.requestFocus ();
						sendMessage (text);
						break;
					case 1:
						break;
					case 2:
						if (lastEnter) {
							e.consume();
							text = inputTextArea.getText ();
							inputTextArea.setText ("");
							inputTextArea.requestFocus ();
							sendMessage (text);
						} else {
							lastEnter = true;
							return;
						}
				}
			}
			lastEnter = false;
		}
	}
	
	/* (non-Javadoc)
	 * @see java.awt.event.KeyListener#keyReleased(java.awt.event.KeyEvent)
	 */
	public void keyReleased(KeyEvent e) {
		if ((e.getSource() == chatTextArea) && ((e.getKeyCode() == KeyEvent.VK_C) || (e.getKeyCode() == KeyEvent.VK_INSERT)) && (e.getModifiers() == InputEvent.CTRL_MASK)) {
			e.consume();
		} else if ((e.getSource() == inputTextArea) && (e.getKeyCode() == KeyEvent.VK_S) && (e.getModifiers() == InputEvent.ALT_MASK)) {
			e.consume();
		} else if ((e.getSource() == inputTextArea) && (e.getKeyCode() == KeyEvent.VK_A || e.getKeyCode() == KeyEvent.VK_UP || e.getKeyCode() == KeyEvent.VK_DOWN || e.getKeyCode() == KeyEvent.VK_ENTER) && (e.getModifiers() == InputEvent.CTRL_MASK)) {
			e.consume();
		}
	}
	
	/* (non-Javadoc)
	 * @see java.awt.event.KeyListener#keyTyped(java.awt.event.KeyEvent)
	 */
	public void keyTyped(KeyEvent e) {
		// nix
	}
	
	/* 
	 * (non-Javadoc)
	 * @see java.applet.Applet#start()
	 */
	public void start () {
		if (server.equals ("")) {
			
			addError("Fehler", "kein Server angegeben!\n" +
			 "Bitte informieren Sie den Betreiber dieser Seite");
			
		} else if (chatName.equals ("")) {
			
			addError ("Fehler", "kein Chatname angegeben!\n" +
			 "Bitte informieren Sie den Betreiber dieser Seite");
			
		} else {
			
			impl.connect (server, port);
			
		}//if
		
		mainSplitter.setSplitterPos(mainSplitter.getHeight() - 30);
		if (getParameter("RightUserlist") != null) {
			splitter2.setSplitterPos(splitter2.getWidth() - 120);
		} else {
			splitter2.setSplitterPos(120);
		}
		inputTextArea.requestFocus();
		
	}//start()
	
	/* (non-Javadoc)
	 * @see ec4.tools.ui.URLListener#urlActionPerformed(ec4.tools.ui.URLEvent)
	 */
	public void urlActionPerformed(URLEvent e) {
		try {
			getAppletContext().showDocument(new URL(e.getURL()), "_new");
		} catch (MalformedURLException e1) {
			// ignoriert
		}
	}
	
	/**
	 * Ldt die in der URL angegebenen Smilies in die chatTextArea.
	 * Die Konfigdatei hat folgendes Format:
	 * <pre>
	 * Dateiname
	 * Smiliecode1 Smiliecode2 ...
	 * nchster Dateiname
	 * usw...
	 * </pre>
	 * z.B.
	 * <pre>
	 * angry.gif
	 * >:-( >:(
	 * sad.gif
	 * :-( :(
	 * </pre>
	 * Als Dateinamen kommen in Frage:
	 * <ul>
	 * <li>nur der Name (wird als Resource in ec3/client/applet/resources/
	 *     gesucht)</li>
	 * <li>res://pfad/.../Dateiname (wird als Resource gesucht)</li>
	 * <li>eine andere gltige URL (z.B. http://...; wird von dort geladen</li>
	 * </ul>
	 * 
	 * @param url die zu ladende URL.
	 * @throws IOException wenn ein Fehler auftritt.
	 */
	private void loadSmilies(URL url) throws IOException {
		URLConnection conn = url.openConnection();
		InputStream is = conn.getInputStream();
		// Lnge lesen; es kann -1 zurckkommen!
		int len = conn.getContentLength();
		// Puffer reservieren (min 256 Byte)
		byte[] buf = new byte[(len > 256? len : 256)];
		byte[] data = new byte[(len > 256? len : 256)];
		int dataPos = 0;
		int readBytes = is.read(buf);
		// Einlesen, Puffer falls ntig vergrern
		while (readBytes >= 0) {
			if (readBytes + dataPos >= data.length) {
				byte[] temp = new byte[data.length * 2];
				System.arraycopy(data, 0, temp, 0, data.length);
				data = temp;
			}
			System.arraycopy(buf, 0, data, dataPos, readBytes);
			dataPos += readBytes;
			readBytes = is.read(buf);
		}
		String[] input = Tools.split(Tools.join(Tools.split(new String(data, 0, dataPos), "\r\n"), "\n"), "\n");
		for (int i = 0; i < input.length - 1; i += 2) {
			String file = input[i];
			if (file.endsWith("\r")) {
				file = file.substring(0, file.length() - 1);
			}
			String codes = input[i + 1];
			try {
				URL res;
				if (file.indexOf(':') < 0) {
					res = getClass().getClassLoader().getResource(
						"ec3/client/applet/resources/" + file);
				} else if (file.startsWith("res://")) {
					res = getClass().getClassLoader().getResource(file.substring(6));
				} else {
					res = new URL(file);
				}
				if (res != null) {
					chatTextArea.putSmiley(Tools.split(codes, " "), res);
				}
			} catch (IOException e) {
				addError("IO-Fehler", "kann Smilie " + file + " nicht laden");
			}
		}
	}
	
	/**
	 * formatiert eine Fehlermeldung und fgt sie in den Chat ein
	 * 
	 * @param type der Typ des Fehlers
	 * @param text der Fehlertext
	 */
	private void addError(String type, String text) {
		Calendar cal = Calendar.getInstance();
		String hours = "" + cal.get(Calendar.HOUR_OF_DAY);
		while (hours.length() < 2) hours = "0" + hours;
		String minutes = "" + cal.get(Calendar.MINUTE);
		while (minutes.length() < 2) minutes = "0" + minutes;
		String date = "[" + hours + ":" + minutes + "]";
		addText(ColorTextArea.makeBold(ColorTextArea.makeFgColor(date + " " + type + ": " + text, Color.red)));
	}
	
	/**
	 * formatiert eine normale Chatnachricht und fgt sie in den Chat ein
	 *
	 * @param from der Absender
	 * @param text der einzufgende Text
	 * @param color die Farbe der NAchricht
	 */
	private void addMessage (String from, String text, Color color) {
		addMessage(from, text, color, false);
	}//addTest (String, String)
	
	/**
	 * formatiert eine normale Chatnachricht und fgt sie in den Chat ein
	 *
	 * @param from der Absender
	 * @param text der einzufgende Text
	 * @param color die Farbe der Nachricht
	 * @param italic gibt an, ob die Nachricht kursiv ist
	 */
	private void addMessage (String from, String text, Color color, boolean italic) {
		
		Calendar cal = Calendar.getInstance();
		String hours = "" + cal.get(Calendar.HOUR_OF_DAY);
		while (hours.length() < 2) hours = "0" + hours;
		String minutes = "" + cal.get(Calendar.MINUTE);
		while (minutes.length() < 2) minutes = "0" + minutes;
		String date = "[" + hours + ":" + minutes + "]";
		String _text = ColorTextArea.makeFgColor(date + " " + from + ": " + text, color);
		if (italic) {
			_text = ColorTextArea.makeItalic(_text);
		}
		addText (_text);
		
	}//addTest (String, String)
	
	/**
	 * formatiert eine Systemnachricht und fgt sie in den Chat ein
	 * 
	 * @param type der Typ der Nachricht
	 * @param text der Fehlertext
	 */
	private void addSysMsg(String type, String text) {
		Calendar cal = Calendar.getInstance();
		String hours = "" + cal.get(Calendar.HOUR_OF_DAY);
		while (hours.length() < 2) hours = "0" + hours;
		String minutes = "" + cal.get(Calendar.MINUTE);
		while (minutes.length() < 2) minutes = "0" + minutes;
		String date = "[" + hours + ":" + minutes + "]";
		addText(ColorTextArea.makeBold(date + " " + type + ": " + text));
	}
	
	/**
	 * fgt Text in den Chat ein
	 *
	 * @param text der einzufgende Text
	 */
	private synchronized void addText (String text) {
		// Scroll-Position ermitteln
		int pos = textAreaScrollPane.getScrollPosition();
		int max = textAreaScrollPane.getMaximumScrollPosition()
				- textAreaScrollPane.getVisibleAmount();
		// Text einfgen
		chatTextArea.append(text);
		// Scrollen, falls ntig
		if (pos > max - 100) {
			max = textAreaScrollPane.getMaximumScrollPosition()
				- textAreaScrollPane.getVisibleAmount();
			textAreaScrollPane.setScrollPosition(max);
		}
	}//addText (String)
	
	/**
	 * ...
	 */
	private void beep() {
		
		if ((beepAudioClip != null)
		 && impl.getProperty ("client.AwaySys.away").equals ("0")
		 && sound) {
			
			beepAudioClip.play();
			
		}
		
	}
	
	/**
	 * erstellt die Labels der Checkboxen in der Userliste
	 *
	 * @param item der zu bearbeitende Eintrag in der Userliste
	 */
	private void buildLabel (UserListItem item) {
		
		//Name
		String name = item.getName ();
		//Admin?
		if (item.isAdmin ()) {
			
			name = "@" + name;
			
		}//if
		//Away?
		if (item.isAway ()) {
			
			//ignoriert?
			if (item.isIgnored ()) {
				
				name = name + " (away, ignoriert)";
				
			} else {
				
				name = name + " (away)";
				
			}//if
			
		//ignoriert?
		} else if (item.isIgnored ()) {
			
			name = name + " (ignoriert)";
			
		}//if
		//Checkbox updaten...
		Checkbox checkbox = item.getCheckbox ();
		checkbox.setLabel (name);
		checkbox.setState (item.sendTo ());
		checkbox.setForeground(item.getColor());
		
		//...entfernen...
		userListPanel.remove(checkbox);
		
		//...und neu einsortieren
		Component[] temp = userListPanel.getComponents ();
		int i = 0;
		while ((i < temp.length) && (((Checkbox)temp[i]).getLabel ().compareTo (name) < 0)) {
			
			i++;
			
		}//while
		userListPanel.add (checkbox, i);
		
		userListScrollPane.validate();
		
	}//buildLabel
	
	/**
	 * schickt ein EC3Event an die Impl
	 * 
	 * @param event der Vefehl
	 * @param data die Daten zum Befehl
	 */
	private void implCallBack (String event, String[] data) {
		
		impl.EC3CallBack (new EC3Event (this, event, data));
		
	}//implCallBack(...)
	
	/**
	 * Nachricht an alle User abschicken, bei denen sendTo aktiviert ist
	 * 
	 * @param text der zu sendende Text
	 */
	private void sendMessage (String text) {
		
		// zur History hinzufgen
		history.put(text);
		historyPos = -1;
		
		//alle User durchgehen
		Vector liste = new Vector (userList.size () + 1);
		for (Enumeration e = userList.keys (); e.hasMoreElements ();) {
			
			UserListItem item = (UserListItem)userList.get (e.nextElement ());
			if (item.sendTo ()) {
				
				liste.addElement (item.getName ());
				
			}//if
			
		}//for
		
		//immer zumindest an den User selbst senden
		if (liste.size () == 0) {
			
			implCallBack ("message", new String[] {text, chatName});
			
		//an alle Zielpersonen
		} else {
			
			liste.insertElementAt (text, 0);
			String[] to = new String[liste.size ()];
			liste.copyInto (to);
			implCallBack ("message", to);
			
		}//if
		
	}//sendMessage(...)

	/* 
	 * (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		// Nachricht senden
		if (e.getSource() == sendButton) {
			String text = inputTextArea.getText ();
			inputTextArea.setText ("");
			inputTextArea.requestFocus ();
			sendMessage (text);
		// neu verbinden
		} else if (e.getSource() == connectButton) {
			awayButton.setLabel ("Awaymodus aktivieren");
			impl.connect (server, port);
			inputTextArea.requestFocus ();
		// Away-Modus an/aus
		} else if (e.getSource() == awayButton) {
			
			if (impl.getProperty ("client.AwaySys.away").equals ("0")) {
				
				inputTextArea.requestFocus ();
				awayInput.setVisible(true);
				
			} else {
				
				implCallBack ("setAway", new String[] {"0"});
				inputTextArea.requestFocus ();
				
			}//if
		// Chatraumliste anzeigen
		} else if (e.getSource() == chatroomButton) {
			implCallBack ("getChatRoomList", Tools.emptyStringArray);
			inputTextArea.requestFocus ();
		}//if
	}

	/* 
	 * (non-Javadoc)
	 * @see java.awt.event.ItemListener#itemStateChanged(java.awt.event.ItemEvent)
	 */
	public void itemStateChanged(ItemEvent e) {
		String name = ((Checkbox)e.getSource()).getName ();
		UserListItem item = (UserListItem)userList.get (name.toLowerCase ());
		if (item != null) {
			item.setSendTo (e.getStateChange() == ItemEvent.SELECTED);
			if (item.sendTo()) {
				notSendToList.remove(item.getName().toLowerCase());
			} else {
				notSendToList.put(item.getName().toLowerCase(), item.getName());
			}
		}//if
	}

	/* 
	 * (non-Javadoc)
	 * @see ec4.tools.ui.SelectionListener#selectionChanged(ec4.tools.ui.SelectionEvent)
	 */
	public void selectionChanged(SelectionEvent e) {
		StringSelection s = new StringSelection(chatTextArea.getSelectedText()) {
			public void lostOwnership(Clipboard clipboard, Transferable contents) {
				if (!chatTextArea.isInSelection()) {
					chatTextArea.removeSelection();
				}
			}
		};
		Clipboard clip = null;
		try {
			clip = Toolkit.getDefaultToolkit().getSystemSelection();
			if (clip == null) {
				clip = Toolkit.getDefaultToolkit().getSystemClipboard();
			}
		} catch (SecurityException ex) {
			// nix
		}
		if (clip != null) {
			clip.setContents(s, s);
		}
	}
	
	/**
	 * Formatiert die Verbindungs-Angabe fr das Userdetails-Plugin.
	 * 
	 * @param data die Angabe.
	 * @return der formatierte String.
	 */
	private String formatConnection(String data) {
		if (data.equals("1")) return "14.4 kBit Modem";
		if (data.equals("2")) return "28.8 kBit Modem";
		if (data.equals("3")) return "33.6 kBit Modem";
		if (data.equals("4")) return "56 kBit Modem";
		if (data.equals("5")) return "64 kBit ISDN";
		if (data.equals("6")) return "128 kBit ISDN";
		if (data.equals("7")) return "768 kBit DSL";
		if (data.equals("8")) return "1500 kBit DSL";
		if (data.equals("9")) return "10 MBit LAN";
		if (data.equals("10")) return "11 MBit WLAN";
		if (data.equals("11")) return "22 MBit WLAN";
		if (data.equals("12")) return "44 MBit WLAN";
		if (data.equals("13")) return "100 MBit LAN";
		if (data.equals("14")) return "> 100 MBit LAN";
		if (data.indexOf('/') > 0) {
			return data.substring(data.indexOf('/') + 1);
		}
		return "nicht angegeben";
	}
	
	/**
	 * Formatiert das Geburtsdatum fr die User-Details.
	 * 
	 * @param data die Datums-Angabe.
	 * @return der formatierte String.
	 */
	private String formatBirthday(String data) {
		System.out.println(data);
		if (data.equals("00000000")) {
			return "nicht angegeben";
		}
		String day = data.substring(0, 2);
		if (day.equals("00")) {
			day = "-";
		}
		String month = data.substring(2, 4);
		if (month.equals("00")) {
			month = "-";
		}
		String year = data.substring(4, 8);
		if (year.equals("0000")) {
			year = "-";
		}
		return day + "." + month + "." + year;
	}
}