package ec3.client.applet;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

/**
 * Fenster zur Eingabe eines Strings, wahlweise mit Sternchen
 *
 * @since 0.4
 */
public class InputFrame extends Frame implements WindowListener, ActionListener,
KeyListener {
	
	/**
	 * Konstruktor mit Titel und Rckrufobjekt
	 *
	 * @param titel der Fenstertitel
	 * @param target das Rckrufobjekt
	 */
	public InputFrame (String titel, InputFrameCallBack target) {
		super (titel);
		
		this.target = target;
		
		addKeyListener(this);
		addWindowListener(this);
		
		//Fenster bauen
		setSize (200, 120);
		setLayout (new FlowLayout ());
		
		textField = new TextField ("", 20);
		textField.addActionListener(this);
		textField.addKeyListener(this);
		add (textField);
		
		okButton = new Button ("OK");
		okButton.addActionListener(this);
		okButton.addKeyListener(this);
		add (okButton);
		cancelButton = new Button ("Abbrechen");
		cancelButton.addActionListener(this);
		cancelButton.addKeyListener(this);
		add (cancelButton);
		
	}//InputFrame (String, InputFrameCallBack)
	
	/**
	 * Konstruktor mit Titel, Rckrufobjekt und Passowrteigenschaft
	 *
	 * @param titel der Fenstertitel
	 * @param target das Rckrufobjekt
	 * @param password sollen Sternchen angezeigt werden?
	 */
	public InputFrame (String titel, InputFrameCallBack target, boolean password) {
		this(titel, target);
		
		if (password) { textField.setEchoChar ('x'); }
		add (textField);
		
	}//InputFrame (String, InputFrameCallBack, boolean)
	
	public void show () {
		super.show ();
		textField.requestFocus ();
	}//show()
	
	/**
	 * das Eingabefeld
	 */
	private TextField textField;
	
	/**
	 * der "OK"-Button
	 */
	private Button okButton;
	
	/**
	 * der "Abbrechen"-Button
	 */
	private Button cancelButton;
	
	/**
	 * das zu benschrichtigende Callback-Objekt
	 */
	private final InputFrameCallBack target;

	/* 
	 * (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowActivated(java.awt.event.WindowEvent)
	 */
	public void windowActivated(WindowEvent e) {
		// nix
	}

	/* 
	 * (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowClosed(java.awt.event.WindowEvent)
	 */
	public void windowClosed(WindowEvent e) {
		// nix
	}

	/* 
	 * (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowClosing(java.awt.event.WindowEvent)
	 */
	public void windowClosing(WindowEvent e) {
		setVisible(false);
		textField.setText ("");
	}

	/* 
	 * (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowDeactivated(java.awt.event.WindowEvent)
	 */
	public void windowDeactivated(WindowEvent e) {
		// nix
	}

	/* 
	 * (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowDeiconified(java.awt.event.WindowEvent)
	 */
	public void windowDeiconified(WindowEvent e) {
		// nix
	}

	/* 
	 * (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowIconified(java.awt.event.WindowEvent)
	 */
	public void windowIconified(WindowEvent e) {
		// nix
	}

	/* 
	 * (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowOpened(java.awt.event.WindowEvent)
	 */
	public void windowOpened(WindowEvent e) {
		// nix
	}

	/* 
	 * (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		//Eingabe fertig
		if ((e.getSource() == okButton) || (e.getSource() == textField)) {
			setVisible(false);
			target.inputFrameCallBack (this, textField.getText ());
			textField.setText ("");
		//Eingabe abbrechen
		} else if (e.getSource() == cancelButton) {
			setVisible(false);
			textField.setText ("");
		}//if
	}

	/* 
	 * (non-Javadoc)
	 * @see java.awt.event.KeyListener#keyPressed(java.awt.event.KeyEvent)
	 */
	public void keyPressed(KeyEvent e) {
		//Escape -> Fenster verstecken
		if ((e.getModifiers() == 0) && (e.getKeyCode() == KeyEvent.VK_ESCAPE)) {
			setVisible(false);
			textField.setText ("");
			e.consume();
		} else if ((e.getModifiers() == 0)
		 && (e.getKeyCode() == KeyEvent.VK_ENTER)
		 && (e.getSource() == okButton)) {
			setVisible(false);
			target.inputFrameCallBack (this, textField.getText ());
			textField.setText ("");
			e.consume();
		} else if ((e.getModifiers() == 0)
			 && (e.getKeyCode() == KeyEvent.VK_ENTER)
			 && (e.getSource() == cancelButton)) {
			setVisible(false);
				textField.setText ("");
				e.consume();
		}//if
	}

	/* 
	 * (non-Javadoc)
	 * @see java.awt.event.KeyListener#keyReleased(java.awt.event.KeyEvent)
	 */
	public void keyReleased(KeyEvent e) {
		// nix
	}

	/* 
	 * (non-Javadoc)
	 * @see java.awt.event.KeyListener#keyTyped(java.awt.event.KeyEvent)
	 */
	public void keyTyped(KeyEvent e) {
		// nix
	}
	
}