package ec3.client.applet;

import java.awt.Checkbox;
import java.awt.Color;

/**
 * Reprsentiert einen Eintrag in der Userliste
 *
 * @since 0.4
 */
public class UserListItem {
	
	/**
	 * der Admin-Status des Users
	 */
	private boolean admin;
	
	/**
	 * der Away-Status des Users
	 */
	private boolean away;
	
	/**
	 * die Chechbox zu diesem User
	 */
	private Checkbox checkbox;
	
	/**
	 * die Farbe dieses Users
	 */
	private Color color;
	
	/**
	 * der "ignogiert"-Status des Users
	 */
	private boolean ignored;
	
	/**
	 * der Name des Users
	 */
	private String name;
	
	/**
	 * der "senden an"-Status des Users
	 */
	private boolean sendTo;
	
	/**
	 * erstellt ein neues UserListItem
	 * 
	 * @param name der Name des Users
	 * @param away true, wenn der User away ist
	 * @param admin true, wenn der User Admin ist
	 * @param ignored true, wenn der User ignoriert werden soll
	 * @param sendTo true, wenn Nachrichten zu diesem User gesendet werden
	 *        sollen (=> die Checkbox ist markiert)
	 * @param color die Farbe dieses Users
	 * @param checkbox die Checkbox, die zu diesem User zugeordnet ist
	 */
	public UserListItem (String name, boolean away, boolean admin,
	 boolean ignored, boolean sendTo, Color color, Checkbox checkbox) {
		this.name = name;
		this.away = away;
		this.admin = admin;
		this.ignored = ignored;
		this.sendTo = sendTo;
		this.color = color;
		this.checkbox = checkbox;
	}
	
	/**
	 * erstellt ein neues UserListItem
	 * 
	 * @param name der Name des Users
	 * @param checkbox die Checkbox, die zu diesem User zugeordnet ist
	 */
	public UserListItem (String name, Checkbox checkbox) {
		this.name = name;
		away = false;
		admin = false;
		ignored = false;
		sendTo = true;
		color = Color.black;
		this.checkbox = checkbox;
	}
	
	/**
	 * fragt die Checkbox zu diesem User ab
	 * 
	 * @return die Checkbox
	 */
	public Checkbox getCheckbox () {
		return checkbox;
	}
	
	/**
	 * fragt die Farbe des Users ab
	 * 
	 * @return die Farbe
	 */
	public Color getColor() {
		return color;
	}
	
	/**
	 * fragt den Namen ab
	 * 
	 * @return der Name
	 */
	public String getName () {
		return name;
	}
	
	/**
	 * fragt ab, ob der User Admin ist
	 * 
	 * @return true, wenn der User Admin ist
	 */
	public boolean isAdmin () {
		return admin;
	}
	
	/**
	 * fragt ab, ob der User away ist
	 * 
	 * @return true, wenn der User away ist
	 */
	public boolean isAway () {
		return away;
	}
	
	/**
	 * fragt ab, ob der User ignoriert wird
	 * 
	 * @return true, wenn der User ignoriert wird
	 */
	public boolean isIgnored () {
		return ignored;
	}
	
	/**
	 * fragt ab, ob an diesen User gesendet werden soll
	 * 
	 * @return true, wenn an diesen User gesendet werden soll
	 */
	public boolean sendTo () {
		return sendTo;
	}
	
	/**
	 * setzt den Admin-Status
	 * 
	 * @param admin der neue Admin-Status
	 */
	public void setAdmin (boolean admin) {
		this.admin = admin;
	}
	
	/**
	 * setzt den Away-Status
	 * 
	 * @param away der neue Away-Status
	 */
	public void setAway (boolean away) {
		this.away = away;
	}
	
	/**
	 * setzt die Farbe des Users
	 * 
	 * @param color die neue Farbe
	 */
	public void setColor(Color color) {
		this.color = color;
	}
	
	/**
	 * setzt den "ignoriert"-Status
	 * 
	 * @param ignored der neue "ignoriert"-Status
	 */
	public void setIgnored (boolean ignored) {
		this.ignored = ignored;
	}
	
	/**
	 * legt fest, ob an diesen User gesendet werden soll
	 * 
	 * @param sendTo der neue Status
	 */
	public void setSendTo (boolean sendTo) {
		this.sendTo = sendTo;
	}
}