package ec3.client.applet.launcher;

import java.applet.Applet;
import java.applet.AppletContext;
import java.applet.AppletStub;
import java.applet.AudioClip;
import java.awt.BorderLayout;
import java.awt.Frame;
import java.awt.Image;
import java.awt.Label;
import java.awt.Toolkit;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.NoSuchElementException;
import java.util.Vector;

import ec3.client.applet.ColorApplet;
import ec3.shared.Tools;


/**
 * Launcher fhrt ein Applet im Fenster aus.
 * <br><br>
 * Das Fenster kann ber spezielle showDocument-Aufrufe gesteuert werden, die
 * die Form showDocument(getCodeBase(), key) haben.
 * <table>
 * <tr><th>key</th><th>Wirkung</th></tr>
 * <tr><td>close.html</td><td>schliet das Fenster</td></tr>
 * </table>
 * 
 * @author AM
 */
public class Launcher extends Frame implements AppletStub, AppletContext {
	
	/**
	 * Der Parametername, der den zu verwendenden Browser angibt.
	 */
	private static final String BROWSER_KEY = "browser";
	
	/**
	 * Der Parametername, der zu lesende Konfigdateien angibt.
	 */
	private static final String INCLUDE_KEY = "include";
	
	/**
	 * Der Parametername, der, wenn gesetzt, das automatische Laden von
	 * DEFAULT_CONFIG verhindert. Mu zwingend als erster Parameter benutzt
	 * werden, um Wirkung zu haben.
	 */
	private static final String NO_DEFAULT_CONFIG_KEY = "nodefaultconfig";
	
	/**
	 * fr AppletStub.isActive(): ist das Applet aktiv?
	 */
	private boolean active;
	
	/**
	 * Hashtable fr die Applet-Parameter
	 */
	private Hashtable args_table;
	
	/**
	 * der Pfad zu dieser Klasse
	 */
	private static final String MY_PATH
	// = "/ec3/client/applet/launcher/Launcher.class";
	 = "/" + Launcher.class.getName().replaceAll("\\.", "/") + ".class";
	
	/**
	 * Eine Standard-Konfigurationsdatei, die beim Start immer geladen wird.
	 */
	private static final String DEFAULT_CONFIG
	 = System.getProperty("user.home") + System.getProperty("file.separator")
	 + ".ec3_applet";
	
	/**
	 * das Applet, welches verwaltet wird
	 */
	final Applet a;
	
	/**
	 * Hashtable fr die InputStreams des Applets
	 */
	private static Hashtable streams = new Hashtable();
	
	/**
	 * Vektor, der inkludierte Dateien enthlt, um Endlosschleifen zu vermeiden.
	 */
	private Vector includedFiles = new Vector();
	
	/**
	 * der Einsprungpunkt zum Starten
	 * 
	 * @param args die Kommandozeilenparameter
	 */
	public static void main(String[] args) {
		if (args.length > 0 && args[0].indexOf(':') < 0) {
			if (args[0].toLowerCase().indexOf("edit") >= 0) {
				System.out.println("editiere " + DEFAULT_CONFIG);
				new Launcher(new String[] {NO_DEFAULT_CONFIG_KEY + ":true", "edit:" + DEFAULT_CONFIG}, new EditorApplet(), "EasyChat Konfigurationsdatei editieren");
			} else if (args[0].toLowerCase().indexOf("help") >= 0) {
				System.out.println("java -jar EC3Applet.jar edit editiert die Konfigdatei (" + DEFAULT_CONFIG + ")");
				System.out.println("java -jar EC3Applet.jar help zeigt die Hilfe");
				System.out.println("java -jar EC3Applet.jar [schlssel:wert schlssel:wert ...] startet das Applet");
			} else {
				System.out.println("unbekannte Option: " + args[0]);
				System.out.println("java -jar EC3Applet.jar edit editiert die Konfigdatei (" + DEFAULT_CONFIG + ")");
				System.out.println("java -jar EC3Applet.jar help zeigt die Hilfe");
				System.out.println("java -jar EC3Applet.jar [schlssel:wert schlssel:wert ...] startet das Applet");
			}
		} else {
			// Launcher erstellen
			new Launcher(args, new ColorApplet(), "EasyChat");
		}
	}
	
	/**
	 * erstellt ein neuen Launcher
	 * 
	 * @param args die Parameter fr das Applet
	 * @param a das Applet
	 * @param title der Fenstertitel
	 */
	public Launcher(String[] args, Applet a, String title) {
		super(title);
		active = false;
		args_table = new Hashtable();
		this.a = a;
		if (args.length == 0 || !args[0].toLowerCase().startsWith(NO_DEFAULT_CONFIG_KEY + ":")) {
			handleParams(new String[] {INCLUDE_KEY + ":" + DEFAULT_CONFIG});
		}
		handleParams(args);
		// Applet initalisieren
		a.setStub(this);
		a.init();
		// Launcher initalisieren
		setLayout(new BorderLayout());
		add(a, BorderLayout.CENTER);
		setSize(640, 480);
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		});
		setVisible(true);
		active = true;
		a.start();
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see java.applet.AppletStub#appletResize(int, int)
	 */
	public void appletResize(int width, int height) {
		setSize(width, height);
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see java.applet.AppletContext#getApplet(java.lang.String)
	 */
	public Applet getApplet(String name) {
		return null;
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see java.applet.AppletStub#getAppletContext()
	 */
	public AppletContext getAppletContext() {
		return this;
	}
	
	/* 
	 * (non-Javadoc)
	 * @see java.applet.AppletContext#getApplets()
	 */
	public Enumeration getApplets() {
		return new Enumeration() {
			boolean hasRead = false;
			public boolean hasMoreElements() {
				return !hasRead;
			}

			public Object nextElement() {
				if (!hasRead) {
					hasRead = true;
					return a;
				}
				throw new NoSuchElementException();
			}
		};
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see java.applet.AppletContext#getAudioClip(java.net.URL)
	 */
	public AudioClip getAudioClip(URL url) {
		return Applet.newAudioClip(url);
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see java.applet.AppletStub#getCodeBase()
	 */
	public URL getCodeBase() {
		try {
			Class clazz = Launcher.class;
			URL res = clazz.getResource(MY_PATH);
			String r = res.toString();
			int i = r.lastIndexOf(MY_PATH) + 1;
			String r2 = r.substring(0, i);
			URL res2 = new URL(r2);
			return res2;
		} catch (MalformedURLException e) {
			return null;
		}
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see java.applet.AppletStub#getDocumentBase()
	 */
	public URL getDocumentBase() {
		try {
			return new File(".").getAbsoluteFile().toURL();
		} catch (MalformedURLException e) {
			return null;
		}
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see java.applet.AppletContext#getImage(java.net.URL)
	 */
	public Image getImage(URL url) {
		try {
			// URL holen
			URLConnection c = url.openConnection();
			InputStream is = c.getInputStream();
			// Lnge lesen; es kann -1 zurckkommen!
			int len = c.getContentLength();
			// Puffer reservieren (min 256 Byte)
			byte[] buf = new byte[(len > 256? len : 256)];
			byte[] data = new byte[(len > 256? len : 256)];
			int dataPos = 0;
			int readBytes = is.read(buf);
			// Einlesen, Puffer falls ntig vergrern
			while (readBytes >= 0) {
				if (readBytes + dataPos >= data.length) {
					byte[] temp = new byte[data.length * 2];
					System.arraycopy(data, 0, temp, 0, data.length);
					data = temp;
				}
				System.arraycopy(buf, 0, data, dataPos, readBytes);
				dataPos += readBytes;
				readBytes = is.read(buf);
			}
			// Image erstellen
			Image image = Toolkit.getDefaultToolkit().createImage(data, 0,
				dataPos
			);
			return image;
		} catch (IOException e) {
			return null;
		}
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see java.applet.AppletStub#getParameter(java.lang.String)
	 */
	public String getParameter(String name) {
		return (String)args_table.get(name.toLowerCase());
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see java.applet.AppletContext#getStream(java.lang.String)
	 */
	public InputStream getStream(String key) {
		return (InputStream)streams.get(key);
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see java.applet.AppletContext#getStreamKeys()
	 */
	public Iterator getStreamKeys() {
		return new EnumerationIterator(streams.keys());
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see java.applet.AppletStub#isActive()
	 */
	public boolean isActive() {
		return active;
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see java.applet.AppletContext#setStream(java.lang.String,
	 *      java.io.InputStream)
	 */
	public void setStream(String key, InputStream stream) throws IOException {
		streams.put(key, stream);
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see java.applet.AppletContext#showDocument(java.net.URL)
	 */
	public void showDocument(URL url) {
		showDocument(url, "");
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see java.applet.AppletContext#showDocument(java.net.URL,
	 *      java.lang.String)
	 */
	public void showDocument(URL url, String target) {
		URL closeURL;
		try {
			closeURL = new URL(getCodeBase(), "close.html");
		} catch (MalformedURLException e1) {
			closeURL = null;
		}
		if (url.equals(closeURL)) {
			a.stop();
			active = false;
			a.destroy();
			System.exit(0);
			return;
		}
		String urlString = url.toString();
		String browser = getParameter(BROWSER_KEY);
		if (browser != null) {
			if (browser.indexOf("%1") > 0) {
				//browser = browser.replaceAll("%1", urlString);
				browser = Tools.join(Tools.split(browser, "%1"), urlString);
			} else {
				browser = browser + " " + urlString;
			}
			try {
				Runtime.getRuntime().exec(browser);
				return;
			} catch (IOException e) {
				System.err.println("can't exec " + browser);
			}
		}
		// kein Browser gegeben (oder gegebener nicht verfgbar) -> OS abfragen
		if (System.getProperty("os.name").toLowerCase().indexOf("windows") >= 0) {
			// Windows -> Standardbrowser oder ie versuchen
			try {
				Runtime.getRuntime().exec(url.toString());
				args_table.put(BROWSER_KEY, "%1");
				return;
			} catch (IOException e) {
				System.err.println("can't exec \"" + url.toString() + "\"");
			}
			try {
				Runtime.getRuntime().exec("iexplore " + url.toString());
				args_table.put(BROWSER_KEY, "iexplore %1");
				return;
			} catch (IOException e) {
				System.err.println("can't exec \"iexplore " + url.toString() + "\"");
			}
		} else {
			// kein Windows -> netscape, mozilla, opera, firefox, konqueror probieren
			try {
				Runtime.getRuntime().exec("netscape " + url.toString());
				args_table.put(BROWSER_KEY, "netscape %1");
				return;
			} catch (IOException e) {
				System.err.println("can't exec \"netscape " + url.toString() + "\"");
			}
			try {
				Runtime.getRuntime().exec("mozilla " + url.toString());
				args_table.put(BROWSER_KEY, "mozilla %1");
				return;
			} catch (IOException e) {
				System.err.println("can't exec \"mozilla " + url.toString() + "\"");
			}
			try {
				Runtime.getRuntime().exec("opera " + url.toString());
				args_table.put(BROWSER_KEY, "opera %1");
				return;
			} catch (IOException e) {
				System.err.println("can't exec \"opera " + url.toString() + "\"");
			}
			try {
				Runtime.getRuntime().exec("firefox " + url.toString());
				args_table.put(BROWSER_KEY, "firefox %1");
			} catch (IOException e) {
				System.err.println("can't exec \"firefox " + url.toString() + "\"");
			}
			try {
				Runtime.getRuntime().exec("konqueror " + url.toString());
				args_table.put(BROWSER_KEY, "konqueror %1");
			} catch (IOException e) {
				System.err.println("can't exec konqueror " + url.toString());
			}
		}
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see java.applet.AppletContext#showStatus(java.lang.String)
	 */
	public void showStatus(String status) {
		add(new Label(status), BorderLayout.SOUTH);
	}
	
	/**
	 * Extrahiert Parameter aus einem String-Array
	 * 
	 * @param args das zu verarbeitende String-Array
	 */
	private void handleParams(String[] args) {
		for (int i = 0; i < args.length; i++) {
			String temp = args[i].trim();
			if (temp.startsWith("#") || temp.startsWith(";")) {
				continue;
			}
			int pos = args[i].indexOf(':');
			String key = args[i].substring(0, pos).toLowerCase().trim();
			String value = args[i].substring(pos + 1);
			if (key.equals(INCLUDE_KEY)) {
				// Datei schon gelesen?
				if (includedFiles.contains(value)) {
					continue;
				}
				includedFiles.add(value);
				try {
					// Datei einlesen
					FileInputStream fis = new FileInputStream(value);
					BufferedReader br = new BufferedReader(new InputStreamReader(fis));
					ArrayList al = new ArrayList();
					while (br.ready()) {
						al.add(br.readLine());
					}
					String[] array = (String[])al.toArray(new String[al.size()]);
					handleParams(array);
				} catch (FileNotFoundException e) {
					if (!value.equals(DEFAULT_CONFIG) || args_table.containsKey(NO_DEFAULT_CONFIG_KEY)) {
						System.err.println("Kann Datei " + value + " nicht lesen: " + e.getMessage());
					}
				} catch (IOException e) {
					System.err.println("Kann Datei " + value + " nicht lesen: " + e.getMessage());
				}
			} else {
				args_table.put(key, value);
			}
		}
	}
}