/*
 * Created on 07.07.2004
 */
package ec3.shared;


/**
 * StringFIFO stellt einen Ringpuffer dar, der bei berschreiten der maximalen
 * Eintragsanzahl ent weder den ltesten Eintrag entfernt oder eine Exception
 * wirft
 * 
 * @author AM
 */
public class StringFIFO {
	
	/**
	 * der aktuelle Inhalt des Puffers
	 */
	private String[] content;
	
	/**
	 * die maximale Gre des Puffers
	 */
	private int capacity;
	
	/**
	 * ein Zeiger auf das Feld, wo als nchstes eingefgt werden soll
	 */
	private int head;
	
	/**
	 * ein Zeiger auf das letzte belegte Feld
	 */
	private int tail;
	
	/**
	 * true, wenn bei berlauf das lteste Element entfernt werden soll
	 */
	private boolean ignoreOverflow;
	
	/**
	 * true, wenn der Puffer voll ist
	 */
	private boolean full;
	
	/**
	 * true, wenn der Puffer leer ist
	 */
	private boolean empty;
	
	/**
	 * erstellt eine neue StringFIFO mit der angegebene Kapazitt
	 * 
	 * @param capacity die Maximale Gre der FIFO
	 */
	public StringFIFO(int capacity) {
		this(capacity, false);
	}
	
	/**
	 * erstellt eine neue StringFIFO mit der angegebene Kapazitt
	 * 
	 * @param capacity die Maximale Gre der FIFO
	 * @param ignoreOverflow true, wenn bei berlauf das lteste Element
	 *        entfernt werden soll
	 */
	public StringFIFO(int capacity, boolean ignoreOverflow) {
		super();
		this.capacity = capacity;
		this.ignoreOverflow = ignoreOverflow;
		content = new String[capacity];
		head = 0;
		tail = 0;
		full = false;
		empty = true;
	}
	
	/**
	 * fragt das neueste Element ab
	 * 
	 * @return das neueste Element
	 */
	public String getHead() {
		return get(0);
	}
	
	/**
	 * fragt das lteste Element ab
	 * 
	 * @return das lteste Element
	 */
	public String getTail() {
		return get(getElementCount() - 1);
	}
	
	/**
	 * fragt ein Element ab
	 * 
	 * @param index der Index des gewnschten Elements
	 *        (0 bis getElementCount() - 1)
	 * @return das Element
	 */
	public String get(int index) {
		if (empty || (index >= capacity) || (index < 0)) {
			throw new IndexOutOfBoundsException();
		}
		int myIndex = (capacity + head - index - 1) % capacity;
		if (((head > tail) && (myIndex >= head) && (myIndex < tail))
			|| (head <= tail) && (myIndex <= head) && (myIndex > tail)
		) {
			throw new IndexOutOfBoundsException();
		}
		return content[myIndex];
	}
	
	/**
	 * fgt ein neues Element ein
	 * 
	 * @param s das neue Element
	 */
	public void put(String s) {
		if (!ignoreOverflow && full) {
			throw new IndexOutOfBoundsException();
		}
		content[head] = s;
		head++;
		head %= capacity;
		if (full) {
			tail++;
			tail %= capacity;
		}
		if (head == tail) {
			full = true;
		}
		empty = false;
	}
	
	/**
	 * entfernt das lteste Element
	 * 
	 * @return das lteste Element
	 */
	public String removeTail() {
		if (empty) {
			throw new IndexOutOfBoundsException();
		}
		String temp = content[tail];
		tail++;
		tail %= capacity;
		if (head == tail) {
			empty = true;
		}
		full = false;
		return temp;
	}
	
	/**
	 * leert den Puffer
	 */
	public void clear() {
		head = 0;
		tail = 0;
		empty = true;
		full = false;
	}
	
	/**
	 * fragt die maximale Puffergre ab
	 * 
	 * @return die Kapazitt des Puffers
	 */
	public int getCapacity() {
		return capacity;
	}
	
	/**
	 * fragt ab, wie viele Elemente sich gegenwrtig im Puffer befinden
	 * 
	 * @return die Elementzahl
	 */
	public int getElementCount() {
		if (full) {
			return capacity;
		}
		return (capacity + head - tail) % capacity;
	}
	
	/*
	public static void main(String[] args) {
		StringFIFO f = new StringFIFO(3);
		f.put("abc");
		System.out.println(f.getElementCount());
		f.put("def");
		System.out.println(f.getElementCount());
		f.put("ghi");
		System.out.println(f.getElementCount());
		System.out.println(f.getHead());
		System.out.println(f.get(0));
		System.out.println(f.get(1));
		System.out.println(f.get(2));
		f.removeTail();
		System.out.println(f.get(0));
		System.out.println(f.get(1));
		try {
			System.out.println(f.get(2));
		} catch (IndexOutOfBoundsException e) {
			System.out.println("gibts net");
		}
		f = new StringFIFO(3, true);
		f.put("abc");
		f.put("def");
		f.put("ghi");
		System.out.println(f.getElementCount());
		f.put("jkl");
		System.out.println(f.getElementCount());
		System.out.println(f.getHead());
		System.out.println(f.get(0));
		System.out.println(f.get(1));
		System.out.println(f.get(2));
	}
	*/
	
	/**
	 * fragt den Status des ignoreOverflow-Flags ab
	 * 
	 * @return true, wenn bei berlauf das lteste Element gelscht wird
	 */
	public boolean isIgnoreOverflow() {
		return ignoreOverflow;
	}
	
	/**
	 * setzt das ignoreOverflow-Flag neu
	 * 
	 * @param ignoreOverflow true, wenn bei berlauf das lteste Element
	 *        gelscht werden soll
	 */
	public void setIgnoreOverflow(boolean ignoreOverflow) {
		this.ignoreOverflow = ignoreOverflow;
	}
	
	/**
	 * fragt ab, ob die FIFO leer ist
	 * 
	 * @return true, wenn die FIFO leer sit
	 */
	public boolean isEmpty() {
		return empty;
	}
	
	/**
	 * fragt ab, ob die FIFO voll ist
	 * 
	 * @return true, wenn die FIFO voll ist
	 */
	public boolean isFull() {
		return full;
	}
}
