package ec3.shared;

import java.util.Vector;
//import java.util.ArrayList;
import java.awt.Color;

/**
 * Sammlung verschiedener Methoden, die immer wieder gebraucht werden
 */
public final class Tools {
	
	/**
	 * ein leeres Stringarray
	 */
	public static final String[] emptyStringArray = new String[] {};
	
	/**
	 * Konstante "ASCII-Zeichen 0"
	 */
	public static final String CHR0 = new Character ((char)0).toString ();
	
	/**
	 * Konstante "ASCII-Zeichen 1"
	 */
	public static final String CHR1 = "\u0001";
	
	/**
	 * Konstante "ASCII-Zeichen 2"
	 */
	public static final String CHR2 = "\u0002";
	
	/**
	 * Konstante "ASCII-Zeichen 13 und ASCII-Zeichen 10"
	 */
	public static final String CRLF = "\r\n";
	
	/**
	 * ersetzt String.split, weil das vor java 1.4 noch nicht vorhanden ist
	 *
	 * @param source der zu zerteilende String
	 * @param at der Teilstring, an dem gesplittet wird
	 * @return die Teile des Strings
	 */
	public static String[] split (String source, String at) {
		String temp = source;
		boolean run = true;
		Vector list = new Vector ();
		//solange noch was zu tun ist
		while (run) {
			//suche nchstes Vorkommen
			int pos = temp.indexOf (at);
			//nicht gefunden => Abbruch, letztes Element noch speichern
			if (pos == -1) {
				run = false;
				list.addElement (temp);
			//am Anfang gefunden => Leerstring einfgen
			} else if (pos == 0) {
				list.addElement ("");
				temp = temp.substring (at.length ());
			//gefunden => String auseinanderklamsern
			} else {
				list.addElement (temp.substring (0, pos));
				temp = temp.substring (pos + at.length ());
			}//if
		}//while
		//in Array umwandeln und zurckgeben
		String[] list2array = new String[list.size ()];
		list.copyInto (list2array);
		return list2array;
	}//split (...)
	
	/**
	 * verbindet den Inhalt eines Stringarrays mit glue
	 *
	 * @param source das zu verbindende Array
	 * @param glue der zwischen die Arrayelemente eungefgte String
	 * @return der zusammengefgte String
	 */
	public static String join (String[] source, String glue) {
		if ((source == null) || (source.length == 0)) {
			return "";
		}
		if (glue == null) {
			glue = "";
		}
		StringBuffer temp = new StringBuffer (source[0]);
		for (int i = 1; i < source.length; i++) {
			temp.append (glue).append (source[i]);
		}
		return temp.toString ();
	}
	
	/**
	 * wandelt einen EC-Farbwert(als String) in ein Color-Objekt
	 *
	 * @param val der Farbwert als String
	 * @return der Farbwert als Color-Objekt
	 */
	public static Color long2Color (String val) {
		//Versuchen, in int umzuwandeln
		int intVal = 0;
		try {
			intVal = Integer.parseInt (val);
		} catch (NumberFormatException e) {
			// ignoriert
		}
		//andere Variante aufrufen
		return long2Color (intVal);
	}//long2Color (String val)
	
	/**
	 * wandelt einen EC-Farbwert(als int) in ein Color-Objekt
	 *
	 * @param val der Farbwert als int
	 * @return der Farbwert als Color-Objekt
	 */
	public static Color long2Color (int val) {
		//einzelne Komponenten extrahieren
		int r = val & 0xff;
		int g = (val & 0xff00) >> 8;
		int b = (val & 0xff0000) >> 16;
		//Color-Objekt bauen
		return new Color (r, g, b);
	}//long2Color (int val)
	
	/**
	 * wandelt ein Color-Objekt in ein EC-Farbwert
	 *
	 * @param c der Farbwert als Color-Objekt
	 * @return der Farbwert als String
	 */
	public static String color2Long (Color c) {
		int col = (c.getBlue () << 16) + (c.getGreen () << 8) + c.getRed ();
		return String.valueOf (col);
	}//color2Long (...)
	
	/**
	 * wandelt eine durch Rot-, Grn- und Blauanteile gegebene Farbe in einen
	 * EC-Farbwert
	 * 
	 * @param red der Rot-Anteil der Farbe
	 * @param green
	 * @param blue
	 * @return der Farbwert als String
	 */
	public static String color2Long (int red, int green, int blue) {
		int col = (blue << 16) + (green << 8) + red;
		return String.valueOf (col);
	}//color2Long (...)
	
	/**
	 * versucht, aus einem beliebigen String einen Farbwert zu gewinnen
	 * 
	 * @param col der zu parsende String (als Integerdarstellung, HTML-Farbwert
	 *        oder Konstante)
	 * @return der Farbwert als String oder "0", falls kein Farbwert gewonnen
	 *         werden konnte
	 */
	public static String parseColor (String col) {
		// Leerstring u..
		if ((col == null) || col.equals("")) {
			return "0";
		}
		//1. "richtiger" Farbwert (= integer-Wert)
		try {
			int col_int = Integer.parseInt (col);
			if ((col_int < 0) || (col_int > 16777215)) {
				return "0";
			}// if
			return col;
		} catch (NumberFormatException e) {
			// ignoriert
		}
		//2. HTML-Farbwert
		if (col.charAt (0) == '#') {
			if (col.length () < 7) {
				return "0";
			}
			try {
				int red = Integer.parseInt (col.substring (1, 3), 16);
				int green = Integer.parseInt (col.substring (3, 5), 16);
				int blue = Integer.parseInt (col.substring (5, 7), 16);
				int color = (blue * 256 + green) * 256 + red;
				return String.valueOf (color);
			} catch (NumberFormatException e) {
				return "0";
			}
		}
		//3. Konstanten (Java- und HTML-Farbwerte)
		col = col.toLowerCase ();
		if (col.equals ("black")) {
			return color2Long (Color.black);
		} else if (col.equals ("blue")) {
			return color2Long (Color.blue);
		} else if (col.equals ("cyan")) {
			return color2Long (Color.cyan);
		} else if (col.equals ("darkgray")) {
			return color2Long (Color.darkGray);
		} else if (col.equals ("gray")) {
			return color2Long (Color.gray);
		} else if (col.equals ("green")) {
			return color2Long (Color.green);
		} else if (col.equals ("lightgray")) {
			return color2Long (Color.lightGray);
		} else if (col.equals ("magenta")) {
			return color2Long (Color.magenta);
		} else if (col.equals ("orange")) {
			return color2Long (Color.orange);
		} else if (col.equals ("pink")) {
			return color2Long (Color.pink);
		} else if (col.equals ("red")) {
			return color2Long (Color.red);
		} else if (col.equals ("white")) {
			return color2Long (Color.white);
		} else if (col.equals ("yellow")) {
			return color2Long (Color.yellow);
		} else if (col.equals("html-aliceblue")) {
			return color2Long(0xf0, 0xf8, 0xff);
		} else if (col.equals("html-antiquewhite")) {
			return color2Long(0xfa, 0xeb, 0xd7);
		} else if (col.equals("html-aqua")) {
			return color2Long(0x00, 0xff, 0xff);
		} else if (col.equals("html-aquamarine")) {
			return color2Long(0x7f, 0xff, 0xd4);
		} else if (col.equals("html-azure")) {
			return color2Long(0xf0, 0xff, 0xff);
		} else if (col.equals("html-beige")) {
			return color2Long(0xf5, 0xf5, 0xdc);
		} else if (col.equals("html-bisque")) {
			return color2Long(0xff, 0xe4, 0xc4);
		} else if (col.equals("html-black")) {
			return color2Long(0x00, 0x00, 0x00);
		} else if (col.equals("html-blanchedalmond")) {
			return color2Long(0xff, 0xeb, 0xcd);
		} else if (col.equals("html-blue")) {
			return color2Long(0x00, 0x00, 0xff);
		} else if (col.equals("html-blueviolet")) {
			return color2Long(0x8a, 0x2b, 0xe2);
		} else if (col.equals("html-brown")) {
			return color2Long(0xa5, 0x2a, 0x2a);
		} else if (col.equals("html-burlywood")) {
			return color2Long(0xde, 0xb8, 0x87);
		} else if (col.equals("html-cadetblue")) {
			return color2Long(0x5f, 0x9e, 0xa0);
		} else if (col.equals("html-chartreuse")) {
			return color2Long(0x7f, 0xff, 0x00);
		} else if (col.equals("html-chocolate")) {
			return color2Long(0xd2, 0x69, 0x1e);
		} else if (col.equals("html-coral")) {
			return color2Long(0xff, 0x7f, 0x50);
		} else if (col.equals("html-cornflowerblue")) {
			return color2Long(0x64, 0x95, 0xed);
		} else if (col.equals("html-cornsilk")) {
			return color2Long(0xff, 0xf8, 0xdc);
		} else if (col.equals("html-crimson")) {
			return color2Long(0xdc, 0x14, 0x3c);
		} else if (col.equals("html-cyan")) {
			return color2Long(0x00, 0xff, 0xff);
		} else if (col.equals("html-darkblue")) {
			return color2Long(0x00, 0x00, 0x8b);
		} else if (col.equals("html-darkcyan")) {
			return color2Long(0x00, 0x8b, 0x8b);
		} else if (col.equals("html-darkgoldenrod")) {
			return color2Long(0xb8, 0x86, 0x0b);
		} else if (col.equals("html-darkgray")) {
			return color2Long(0xa9, 0xa9, 0xa9);
		} else if (col.equals("html-darkgreen")) {
			return color2Long(0x00, 0x64, 0x00);
		} else if (col.equals("html-darkkhaki")) {
			return color2Long(0xbd, 0xb7, 0x6b);
		} else if (col.equals("html-darkmagenta")) {
			return color2Long(0x8b, 0x00, 0x8b);
		} else if (col.equals("html-darkolivegreen")) {
			return color2Long(0x55, 0x6b, 0x2f);
		} else if (col.equals("html-darkorange")) {
			return color2Long(0xff, 0x8c, 0x00);
		} else if (col.equals("html-darkorchid")) {
			return color2Long(0x99, 0x32, 0xcc);
		} else if (col.equals("html-darkred")) {
			return color2Long(0x8b, 0x00, 0x00);
		} else if (col.equals("html-darksalmon")) {
			return color2Long(0xe9, 0x96, 0x7a);
		} else if (col.equals("html-darkseagreen")) {
			return color2Long(0x8f, 0xbc, 0x8f);
		} else if (col.equals("html-darkslateblue")) {
			return color2Long(0x48, 0x3d, 0x8b);
		} else if (col.equals("html-darkslategray")) {
			return color2Long(0x2f, 0x4f, 0x4f);
		} else if (col.equals("html-darkturquoise")) {
			return color2Long(0x00, 0xce, 0xd1);
		} else if (col.equals("html-darkviolet")) {
			return color2Long(0x94, 0x00, 0xd3);
		} else if (col.equals("html-deeppink")) {
			return color2Long(0xff, 0x14, 0x93);
		} else if (col.equals("html-deepskyblue")) {
			return color2Long(0x00, 0xbf, 0xff);
		} else if (col.equals("html-dimgray")) {
			return color2Long(0x69, 0x69, 0x69);
		} else if (col.equals("html-dodgerblue")) {
			return color2Long(0x1e, 0x90, 0xff);
		} else if (col.equals("html-firebrick")) {
			return color2Long(0xb2, 0x22, 0x22);
		} else if (col.equals("html-floralwhite")) {
			return color2Long(0xff, 0xfa, 0xf0);
		} else if (col.equals("html-forestgreen")) {
			return color2Long(0x22, 0x8b, 0x22);
		} else if (col.equals("html-fuchsia")) {
			return color2Long(0xff, 0x00, 0xff);
		} else if (col.equals("html-gainsboro")) {
			return color2Long(0xdc, 0xdc, 0xdc);
		} else if (col.equals("html-ghostwhite")) {
			return color2Long(0xf8, 0xf8, 0xff);
		} else if (col.equals("html-gold")) {
			return color2Long(0xff, 0xd7, 0x00);
		} else if (col.equals("html-goldenrod")) {
			return color2Long(0xda, 0xa5, 0x20);
		} else if (col.equals("html-gray")) {
			return color2Long(0x80, 0x80, 0x80);
		} else if (col.equals("html-green")) {
			return color2Long(0x00, 0x80, 0x00);
		} else if (col.equals("html-greenyellow")) {
			return color2Long(0xad, 0xff, 0x2f);
		} else if (col.equals("html-honeydew")) {
			return color2Long(0xf0, 0xff, 0xf0);
		} else if (col.equals("html-hotpink")) {
			return color2Long(0xff, 0x69, 0xb4);
		} else if (col.equals("html-indianred")) {
			return color2Long(0xcd, 0x5c, 0x5c);
		} else if (col.equals("html-indigo")) {
			return color2Long(0x4b, 0x00, 0x82);
		} else if (col.equals("html-ivory")) {
			return color2Long(0xff, 0xff, 0xf0);
		} else if (col.equals("html-khaki")) {
			return color2Long(0xf0, 0xe6, 0x8c);
		} else if (col.equals("html-lavender")) {
			return color2Long(0xe6, 0xe6, 0xfa);
		} else if (col.equals("html-lavenderblush")) {
			return color2Long(0xff, 0xf0, 0xf5);
		} else if (col.equals("html-lawngreen")) {
			return color2Long(0x7c, 0xfc, 0x00);
		} else if (col.equals("html-lemonchiffon")) {
			return color2Long(0xff, 0xfa, 0xcd);
		} else if (col.equals("html-lightblue")) {
			return color2Long(0xad, 0xd8, 0xe6);
		} else if (col.equals("html-lightcoral")) {
			return color2Long(0xf0, 0x80, 0x80);
		} else if (col.equals("html-lightcyan")) {
			return color2Long(0xe0, 0xff, 0xff);
		} else if (col.equals("html-lightgoldenrodyellow")) {
			return color2Long(0xfa, 0xfa, 0xd2);
		} else if (col.equals("html-lightgreen")) {
			return color2Long(0x90, 0xee, 0x90);
		} else if (col.equals("html-lightgrey")) {
			return color2Long(0xd3, 0xd3, 0xd3);
		} else if (col.equals("html-lightpink")) {
			return color2Long(0xff, 0xb6, 0xc1);
		} else if (col.equals("html-lightsalmon")) {
			return color2Long(0xff, 0xa0, 0x7a);
		} else if (col.equals("html-lightseagreen")) {
			return color2Long(0x20, 0xb2, 0xaa);
		} else if (col.equals("html-lightskyblue")) {
			return color2Long(0x87, 0xce, 0xfa);
		} else if (col.equals("html-lightslategray")) {
			return color2Long(0x77, 0x88, 0x99);
		} else if (col.equals("html-lightsteelblue")) {
			return color2Long(0xb0, 0xc4, 0xde);
		} else if (col.equals("html-lightyellow")) {
			return color2Long(0xff, 0xff, 0xe0);
		} else if (col.equals("html-lime")) {
			return color2Long(0x00, 0xff, 0x00);
		} else if (col.equals("html-limegreen")) {
			return color2Long(0x32, 0xcd, 0x32);
		} else if (col.equals("html-linen")) {
			return color2Long(0xfa, 0xf0, 0xe6);
		} else if (col.equals("html-magenta")) {
			return color2Long(0xff, 0x00, 0xff);
		} else if (col.equals("html-maroon")) {
			return color2Long(0x80, 0x00, 0x00);
		} else if (col.equals("html-mediumaquamarine")) {
			return color2Long(0x66, 0xcd, 0xaa);
		} else if (col.equals("html-mediumblue")) {
			return color2Long(0x00, 0x00, 0xcd);
		} else if (col.equals("html-mediumorchid")) {
			return color2Long(0xba, 0x55, 0xd3);
		} else if (col.equals("html-mediumpurple")) {
			return color2Long(0x93, 0x70, 0xdb);
		} else if (col.equals("html-mediumseagreen")) {
			return color2Long(0x3c, 0xb3, 0x71);
		} else if (col.equals("html-mediumslateblue")) {
			return color2Long(0x7b, 0x68, 0xee);
		} else if (col.equals("html-mediumspringgreen")) {
			return color2Long(0x00, 0xfa, 0x9a);
		} else if (col.equals("html-mediumturquoise")) {
			return color2Long(0x48, 0xd1, 0xcc);
		} else if (col.equals("html-mediumvioletred")) {
			return color2Long(0xc7, 0x15, 0x85);
		} else if (col.equals("html-midnightblue")) {
			return color2Long(0x19, 0x19, 0x70);
		} else if (col.equals("html-mintcream")) {
			return color2Long(0xf5, 0xff, 0xfa);
		} else if (col.equals("html-mistyrose")) {
			return color2Long(0xff, 0xe4, 0xe1);
		} else if (col.equals("html-moccasin")) {
			return color2Long(0xff, 0xe4, 0xb5);
		} else if (col.equals("html-navajowhite")) {
			return color2Long(0xff, 0xde, 0xad);
		} else if (col.equals("html-navy")) {
			return color2Long(0x00, 0x00, 0x80);
		} else if (col.equals("html-oldlace")) {
			return color2Long(0xfd, 0xf5, 0xe6);
		} else if (col.equals("html-olive")) {
			return color2Long(0x80, 0x80, 0x00);
		} else if (col.equals("html-olivedrab")) {
			return color2Long(0x6b, 0x8e, 0x23);
		} else if (col.equals("html-orange")) {
			return color2Long(0xff, 0xa5, 0x00);
		} else if (col.equals("html-orangered")) {
			return color2Long(0xff, 0x45, 0x00);
		} else if (col.equals("html-orchid")) {
			return color2Long(0xda, 0x70, 0xd6);
		} else if (col.equals("html-palegoldenrod")) {
			return color2Long(0xee, 0xe8, 0xaa);
		} else if (col.equals("html-palegreen")) {
			return color2Long(0x98, 0xfb, 0x98);
		} else if (col.equals("html-paleturquoise")) {
			return color2Long(0xaf, 0xee, 0xee);
		} else if (col.equals("html-palevioletred")) {
			return color2Long(0xdb, 0x70, 0x93);
		} else if (col.equals("html-papayawhip")) {
			return color2Long(0xff, 0xef, 0xd5);
		} else if (col.equals("html-peachpuff")) {
			return color2Long(0xff, 0xda, 0xb9);
		} else if (col.equals("html-peru")) {
			return color2Long(0xcd, 0x85, 0x3f);
		} else if (col.equals("html-pink")) {
			return color2Long(0xff, 0xc0, 0xcb);
		} else if (col.equals("html-plum")) {
			return color2Long(0xdd, 0xa0, 0xdd);
		} else if (col.equals("html-powderblue")) {
			return color2Long(0xb0, 0xe0, 0xe6);
		} else if (col.equals("html-purple")) {
			return color2Long(0x80, 0x00, 0x80);
		} else if (col.equals("html-red")) {
			return color2Long(0xff, 0x00, 0x00);
		} else if (col.equals("html-rosybrown")) {
			return color2Long(0xbc, 0x8f, 0x80);
		} else if (col.equals("html-royalblue")) {
			return color2Long(0x41, 0x69, 0xe1);
		} else if (col.equals("html-saddlebrown")) {
			return color2Long(0x8b, 0x45, 0x13);
		} else if (col.equals("html-salmon")) {
			return color2Long(0xfa, 0x80, 0x72);
		} else if (col.equals("html-sandybrown")) {
			return color2Long(0xf4, 0xa4, 0x60);
		} else if (col.equals("html-seagreen")) {
			return color2Long(0x2e, 0x8b, 0x57);
		} else if (col.equals("html-seashell")) {
			return color2Long(0xff, 0xf5, 0xee);
		} else if (col.equals("html-sienna")) {
			return color2Long(0xa0, 0x52, 0x2d);
		} else if (col.equals("html-silver")) {
			return color2Long(0xc0, 0xc0, 0xc0);
		} else if (col.equals("html-skyblue")) {
			return color2Long(0x87, 0xce, 0xeb);
		} else if (col.equals("html-slateblue")) {
			return color2Long(0x6a, 0x5a, 0xcd);
		} else if (col.equals("html-slategray")) {
			return color2Long(0x70, 0x80, 0x90);
		} else if (col.equals("html-snow")) {
			return color2Long(0xff, 0xfa, 0xfa);
		} else if (col.equals("html-springgreen")) {
			return color2Long(0x00, 0xff, 0x7f);
		} else if (col.equals("html-steelblue")) {
			return color2Long(0x46, 0x82, 0xb4);
		} else if (col.equals("html-tan")) {
			return color2Long(0xd2, 0xb4, 0x8c);
		} else if (col.equals("html-teal")) {
			return color2Long(0x00, 0x80, 0x80);
		} else if (col.equals("html-thistle")) {
			return color2Long(0xd8, 0xbf, 0xd8);
		} else if (col.equals("html-tomato")) {
			return color2Long(0xff, 0x63, 0x47);
		} else if (col.equals("html-turquoise")) {
			return color2Long(0x40, 0xe0, 0xd0);
		} else if (col.equals("html-violet")) {
			return color2Long(0xee, 0x82, 0xee);
		} else if (col.equals("html-wheat")) {
			return color2Long(0xf5, 0xde, 0xb3);
		} else if (col.equals("html-white")) {
			return color2Long(0xff, 0xff, 0xff);
		} else if (col.equals("html-whitesmoke")) {
			return color2Long(0xf5, 0xf5, 0xf5);
		} else if (col.equals("html-yellow")) {
			return color2Long(0xff, 0xff, 0x00);
		} else if (col.equals("html-yellowgreen")) {
			return color2Long(0x9a, 0xcd, 0x32);
		}
		//4. Standard-Fall
		return "0";
	}//parseColor (...)
	
	/**
	 * Testet einen Farbwert auf Gltigkeit im Sinne von EasyChat
	 * 
	 * @param c der Farbwert
	 * @return true, wenn der Farbwert gltig ist
	 */
	public static boolean testColor(Color c) {
		int sum = c.getRed() + c.getGreen() + c.getBlue();
		return sum <= 2*255;
	}
	/*
	public static String sha256Hash (String in) {
		
		SHA256 sha256 = new SHA256 ();
		return new String (UTF8.hexString (sha256.digest (string2Bytes (in)))).toUpperCase ();
		
	}//sha256Hash (...)
	
	public static byte[] string2Bytes (String in) {
		
		byte[] abc = new byte[in.length ()];
		for (int i = 0; i < in.length (); i++) {
			
			abc[i] = (byte)(in.charAt (i));
			
		}//for
		
		return abc;
		
	}//string2Bytes (...)
	*/
	/**
	 * ersetzt im String source alle Vorkommen von find durch replacement
	 * 
	 * @param source der String, in dem gesucht wird
	 * @param find der zu suchende Teilstring
	 * @param replacement der Ersetzungsausdruck
	 * @return der genderte String
	 */
	public static String replace (String source, String find,
	 String replacement) {
		int i = source.indexOf (find);
		while (i > -1) {
			source = source.substring (0, i) + replacement
			 + source.substring (i + 2);
			i = source.indexOf (find);
		}
		return source;
	}
	
	/**
	 * versucht, aus einem String die gegebene Anzahl Augumente zu extrahieren
	 * 
	 * in Anfhrungszeichen gesetzte Textteile werden als zusammengehrig betrachtet
	 * es wird an Leerzeichen gesplittet
	 * fehlende Teile werden durch Leerstrings ersetzt
	 * 
	 * @param source der zu splittende String
	 * @param count die Anzahl der Teile
	 * @return die Teile des Strings bzw. null wenn count negativ ist
	 */
	public static String[] splitArguments (String source, int count) {
		String[] res = null;
		if (count > 0) {
			res = new String[count];
			if (source != null) {
				String[] split1 = split (source, "\"");
				splitArguments1 (split1, 0, res);
			} else {
				for (int i = 0; i < res.length; i++) {
					res[i] = "";
				}
			}
		}
		return res;
	}
	
	/**
	 * interne Hilfsfunktion fr splitArgumnents; ruft sich selbst rekursiv auf
	 * 
	 * @param split1 der zu bearbeitende String, an " gesplittet
	 * @param pos die Position im Resultatarray
	 * @param res das Resultat
	 */
	private static void splitArguments1 (String[] split1, int pos,
	 String[] res) {
		// letzter Parameter?
		if (pos + 1 == res.length) {
			res[pos] = join (split1, "\"").trim ();
		} else {
			// erster Teil gleich "" => nchsten anfangen
			if (split1[0].equals ("")) {
				// wenn noch was da
				if (split1.length > 1) {
					// nchster Parameter in "" angegeben => komplett benutzen
					res[pos] = split1[1];
					// entfernen
					String[] split1new = new String[split1.length - 2];
					// mit Rest rekursiv aufrufen
					System.arraycopy (split1, 2, split1new, 0,
						split1new.length
					);
					splitArguments1 (split1new, pos + 1, res);
				} else {
					// nix mehr da => rest mit "" fllen
					for (int i = pos; i < res.length; i++) {
						res[i] = "";
					}
				}
			} else {
				// erster Teil ist nicht leer => am Leerzeichen splitten
				String[] split2 = split(split1[0], " ");
				int i = 0;
				// leere Teile bergehen
				for (i = 0; (i < split2.length) && (split2[i].equals (""));
				 i++) {
					// nix
				}
				// alles leer => nur Leerzeichen vorhanden
				if (i == split2.length) {
					// wenn noch was da ist
					if (split1.length > 1) {
						// nchster Parameter in "" angegeben => komplett nehmen
						res[pos] = split1[1];
						// entfernen
						String[] split1new = new String[split1.length - 2];
						// mit Rest rekursiv aufrufen
						System.arraycopy (split1, 2, split1new, 0,
							split1new.length
						);
						splitArguments1 (split1new, pos + 1, res);
					} else {
						// nix mehr da => mit "" auffllen
						for (int j = pos; j < res.length; j++) {
							res[j] = "";
						}
					}
				} else {
					// einen nichtleeren Teil gefunden
					res[pos] = split2[i];
					// rest wieder zusammenfgen
					String[] temp = new String[split2.length - (i + 1)];
					System.arraycopy (split2, i + 1, temp, 0, temp.length);
					split1[0] = join (temp, " ");
					// und rekursiv aufrufen
					splitArguments1 (split1, pos + 1, res);
				}
			}
		}
	}
	
	/**
	 * Maskiert 0-Zeichen in einem String.
	 * 
	 * @param s der zu bearbeitende String.
	 * @return der String, mit maskierten 0-Zeichen und mit " umschlossen.
	 */
	public static String escape(String s) {
		return "\"" + join(split(s, CHR0), "\" + chr$(0) + \"") + "\"";
	}
	
	/**
	 * Maskiert nicht druckbare Zeichen in einem Zeichenarray.
	 * 
	 * @param c das zu bearbeitende Array.
	 * @param start ab wo soll das Array verarbeitet werden (inkliusiv).
	 * @param end bis wo soll das Array verarbeitet werden (exklusiv).
	 * @return der aus dem Array gebildete String, mit maskierten 0-Zeichen
	 *         und mit " umschlossen.
	 */
	public static String escape(char[] c, int start, int end) {
		StringBuffer s = new StringBuffer("\"");
		for (int i = start; i < end; i++) {
			if (c[i] < 32) {
				s.append("\" + chr$(" + (int)c[i] + ") + \"");
			} else {
				s.append(c[i]);
			}
		}
		return s.append("\"").toString();
	}
}//Tools