/*
 * Created on 14.06.2004
 */
package ec4.tools.ui;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

// TODO: Zoom
// bercksichtigt bei:
// - Line.paint() -> AffineTransform?
// - sizeCalc -> width anpassen
// - ???
// Unterstrzung: /zoom und Strg+Mausrad

/**
 * Anzeigefeld fr mehrfarbigen Text und eingebettete Bilder
 * 
 * @author AM
 */
public class ColorTextArea extends Canvas
implements ComponentListener, MouseListener, MouseMotionListener, Scrollable {
	
	/**
	 * Kommando: Hintergrundfarbe setzen
	 */
	public static final char BG_COLOR = 'b';
	
	/**
	 * Kommando: Hintergrundfarbe rcksetzen
	 */
	public static final char BG_COLOR_DEFAULT = 'B';
	
	/**
	 * Kommando: fett umschalten
	 */
	public static final char BOLD = 'd';
	
	/**
	 * Startzeichen fr alle Kommandos
	 */
	public static final char CHR0 = '\u0000';
	
	/**
	 * Eigenschaft nie benutzen
	 */
	public static final int FALSE = 0;
	
	/**
	 * Kommando: Vordergrundfarbe setzen
	 */
	public static final char FG_COLOR = 'f';
	
	/**
	 * Kommando: Vordergrundfarbe rcksetzen
	 */
	public static final char FG_COLOR_DEFAULT = 'F';
	
	/**
	 * Eigenschaftswert intertieren
	 */
	public static final int INVERT = 3;
	
	/**
	 * Kommando: kursiv umschalten
	 */
	public static final char ITALIC = 'i';
	
	/**
	 * Eigenschaft benutzen, wenn im Kontext benutzt
	 */
	public static final int NO_CHANGE = 2;
	
	/**
	 * Kommando: fett, kursiv, unterstrichen rcksetzen
	 */
	public static final char RESET = 'r';
	
	/**
	 * Eigenschaft immer benutzen
	 */
	public static final int TRUE = 1;
	
	/**
	 * Kommando: unterstrichen umschalten
	 */
	public static final char UNDERLINE = 'u';
	
	/**
	 * Position unmittelbar vor einem Element
	 */
	static final int ABOVE = -1;
	
	/**
	 * Position unmittelbar nach einem Element
	 */
	static final int BELOW = Integer.MAX_VALUE; 
	
	/**
	 * ungltige Position
	 */
	static final int INVALID = Integer.MIN_VALUE;
	
	/**
	 * die Standardschrift
	 */
	private static final Font DEFAULT_FONT = new Font(
		"Dialog", //$NON-NLS-1$
		Font.PLAIN, 12
	);
	
	/**
	 * true, wenn Antialiasing benutzt werden soll
	 */
	private boolean antialiasing;
	
	/**
	 * die Eigenschaften der aktuellen Schrift
	 */
	private FontHolder font;
	
	/**
	 * die berechnete Hhe der Komponente
	 */
	private int height;
	
	/**
	 * true, wenn gerade mit der Maus markiert wird
	 */
	private boolean inSelection;
	
	/**
	 * die Randabstnde
	 */
	private Insets insets;
	
	/**
	 * die Liste aller logischen Zeilen
	 */
	private Vector lines;
	
	/**
	 * die maximale Lnge der Smileycodes
	 */
	private int maxSmileyCodeLength;
	
	/**
	 * die alte Breite der Komponente (fr resize)
	 */
	private int oldWidth;
	
	/**
	 * Hintergrundfarbe der markierten Auswahl oder null, wenn die normale
	 * Vordergrundfarbe werden soll
	 */
	private Color selectionBgColor;
	
	/**
	 * Vordergrundfarbe der markierten Auswahl oder null, wenn die normale
	 * Hintergrundfarbe benutzt werden soll
	 */
	private Color selectionFgColor;
	
	/**
	 * die Liste aller SelectionListener
	 */
	private Vector selectionListeners;
	
	/**
	 * die logische Zeile, in der die Markierung endet
	 */
	private int selEndLineIndex;
	
	/**
	 * die Zeichenposition, an der die Markierung endet
	 */
	private int selEndPos;
	
	/**
	 * die logische Zeile, in der die Markierung beginnt
	 */
	private int selStartLineIndex;
	
	/**
	 * die Zeichenposition, an der die Markierung beginnt
	 */
	private int selStartPos;
	
	/**
	 * die Liste aller Smileys
	 */
	private Hashtable smileys;
	
	/**
	 * die Einrckung fr Folgezeilen
	 */
	private int subsequentIndent;
	
	/**
	 * die logische Zeile, in der mit der Markierung angefangen wurde
	 */
	private int tempSelStartLineIndex;
	
	/**
	 * die Zeichenposition, in der mit der Markierung angefangen wurde
	 */
	private int tempSelStartPos;
	
	/**
	 * Hintergrundfarbe der URLs oder null, wenn keine spezielle Farbe benutzt
	 * werden soll
	 */
	private Color urlBgColor;
	
	/**
	 * sollen URLs fett angezeigt werden?
	 */
	private int urlBold;
	
	/**
	 * Vordergrundfarbe der URLs oder null, wenn keine spezielle Farbe benutzt
	 * werden soll
	 */
	private Color urlFgColor;
	
	/**
	 * sollen URLs kursiv angezeigt werden?
	 */
	private int urlItalic;
	
	/**
	 * die Liste aller URLListener
	 */
	private Vector urlListeners;
	
	/**
	 * die Liste aller URL-Prfixe
	 */
	private Vector urlPrefix;
	
	/**
	 * sollen URLs unterstrichen werden?
	 */
	private int urlUnderline;
	
	/**
	 * die Position, zu der gescrollt wurde
	 */
	private int scrollPos;
	
	/**
	 * true, wenn beim scrollen alles neu gezeichnet werden soll
	 */
	private boolean scrollRepaint;
	
	/**
	 * wandelt ein Color-Objekt in eine Stringreprsentation um
	 * 
	 * @param c die Farbe
	 * @return eine Stringreprsentation der Farbe
	 */
	public static String color2String(Color c) {
		int col = (c.getBlue () << 16) + (c.getGreen () << 8) + c.getRed ();
		return String.valueOf (col);
	}
	
	/**
	 * formatiert einen String mit einer bestimmten Hintergrundfarbe
	 * 
	 * @param s der zu formatierende String
	 * @param c die Farbe
	 * @return der formatierte String
	 */
	public static String makeBgColor(String s, Color c) {
		return "" + CHR0 + BG_COLOR + color2String(c) + CHR0 + s //$NON-NLS-1$
		+ CHR0 + BG_COLOR_DEFAULT;
	}
	
	/**
	 * formatiert einen String fett
	 * 
	 * @param s der zu formatierende String
	 * @return der formatierte String
	 */
	public static String makeBold(String s) {
		return "" + CHR0 + BOLD + s + CHR0 + BOLD; //$NON-NLS-1$
	}
	
	/**
	 * formatiert einen String mit einer bestimmten Farbe
	 * 
	 * @param s der zu formatierende String
	 * @param c die Farbe
	 * @return der formatierte String
	 */
	public static String makeFgColor(String s, Color c) {
		return "" + CHR0 + FG_COLOR + color2String(c) + CHR0 + s //$NON-NLS-1$
		+ CHR0 + FG_COLOR_DEFAULT;
	}
	
	/**
	 * formatiert einen String kursiv
	 * 
	 * @param s der zu formatierende String
	 * @return der formatierte String
	 */
	public static String makeItalic(String s) {
		return "" + CHR0 + ITALIC + s + CHR0 + ITALIC; //$NON-NLS-1$
	}
	
	/**
	 * formatiert einen String unterstrichen
	 * 
	 * @param s der zu formatierende String
	 * @return der formatierte String
	 */
	public static String makeUnderline(String s) {
		return "" + CHR0 + UNDERLINE + s + CHR0 + UNDERLINE; //$NON-NLS-1$
	}
	
	/**
	 * erzeugt ein Color-Objekt aud dessen Stringreprsentation
	 * 
	 * @param s die Stringreprsentation
	 * @return das Color-Objekt
	 */
	public static Color string2Color(String s) {
		//Versuchen, in int umzuwandeln
		int intVal;
		try {
			intVal = Integer.parseInt (s);
		} catch (NumberFormatException e) {
			intVal = 0;
		}
		//einzelne Komponenten extrahieren
		int r = intVal & 0xff;
		int g = (intVal & 0xff00) >> 8;
		int b = (intVal & 0xff0000) >> 16;
		//Color-Objekt bauen
		return new Color (r, g, b);
	}
	
	/**
	 * erstellt eine neue ColorTextArea
	 */
	public ColorTextArea() {
		super();
		// Felder initalisieren
		this.smileys = new Hashtable();
		this.maxSmileyCodeLength = 0;
		this.urlPrefix = new Vector();
		this.urlListeners = new Vector();
		this.selectionListeners = new Vector();
		this.lines = new Vector();
		this.subsequentIndent = 12;
		this.insets = new Insets(3, 3, 3, 3);
		setFont(DEFAULT_FONT);
		this.urlBold = NO_CHANGE;
		this.urlItalic = NO_CHANGE;
		this.urlUnderline = TRUE;
		this.urlFgColor = Color.blue;
		this.selStartPos = INVALID;
		this.selEndPos = INVALID;
		this.tempSelStartPos = INVALID;
		this.inSelection = false;
		this.antialiasing = false;
		setBackground(Color.white);
		setForeground(Color.black);
		setCursor(Cursor.getPredefinedCursor(Cursor.TEXT_CURSOR));
		this.oldWidth = 0;
		this.scrollPos = 0;
		this.scrollRepaint = true;
		// Standardprfixe
		addURLPrefix("WWW."); //$NON-NLS-1$
		addURLPrefix("HTTP://"); //$NON-NLS-1$
		addURLPrefix("HTTPS://"); //$NON-NLS-1$
		addURLPrefix("FTP."); //$NON-NLS-1$
		addURLPrefix("FTP://"); //$NON-NLS-1$
		// Listener registrieren
		addComponentListener(this);
		addMouseListener(this);
		addMouseMotionListener(this);
		// Gre berechnen
		sizeCalc();
		if (getParent() != null) {
			// neue Gre bekanntgeben
			getParent().doLayout();
		}
	}
	
	/**
	 * fgt einen SelectionListener hinzu
	 * 
	 * @param l der Listener
	 */
	public void addSelectionListener(SelectionListener l) {
		if (!this.selectionListeners.contains(l)) {
			this.selectionListeners.addElement(l);
		}
	}
	
	/**
	 * fgt einen URLListener hinzu
	 * 
	 * @param l der Listener
	 */
	public void addURLListener(URLListener l) {
		if (!this.urlListeners.contains(l)) {
			this.urlListeners.addElement(l);
		}
	}
	
	/**
	 * fgt ein URL-Prfix in die Liste ein
	 * 
	 * @param prefix das Prfix
	 */
	public void addURLPrefix(String prefix) {
		this.urlPrefix.addElement(prefix.toUpperCase());
	}
	
	/**
	 * hngt einen String an
	 * 
	 * @param s der String
	 */
	public synchronized void append(String s) {
		Line l = new Line(this, s);
		int lineHeight = l.sizeCalc();
		int oldHeight = this.height;
		this.height += lineHeight;
		this.lines.addElement(l);
		if (getParent() != null) {
			// neue Gre bekanntgeben
			getParent().doLayout();
		}
		// nur die neue Zeile muss gezeichnet werden
		repaint(0, oldHeight + getInsets().top, this.getSize().width,
			lineHeight
		);
	}
	
	/**
	 * clear lscht alle Zeilen aus der TextArea
	 */
	public void clear() {
		// lschen
		this.lines.removeAllElements();
		// internen Status Updaten
		sizeCalc();
		updateSelection(INVALID, INVALID, INVALID, INVALID);
		if (getParent() != null) {
			// neue Gre bekanntgeben
			getParent().doLayout();
		}
		// neu zeichnen
		repaint();
		// FIXME neuzeichnen nach cls funzt net (manchmal)
	}
	
	/* (non-Javadoc)
	 * @see java.awt.event.ComponentListener#componentHidden(java.awt.event.ComponentEvent)
	 */
	public void componentHidden(ComponentEvent e) {
		// nix
	}
	
	/* (non-Javadoc)
	 * @see java.awt.event.ComponentListener#componentMoved(java.awt.event.ComponentEvent)
	 */
	public void componentMoved(ComponentEvent e) {
		// nix
	}
	
	/* (non-Javadoc)
	 * @see java.awt.event.ComponentListener#componentResized(java.awt.event.ComponentEvent)
	 */
	public void componentResized(ComponentEvent e) {
		if (e.getSource() != this) return;
		int newWidth = getSize().width;
		if (newWidth != this.oldWidth) {
			sizeCalc();
			if (getParent() != null) {
				// neue Gre bekanntgeben
				getParent().doLayout();
			}
			this.oldWidth = newWidth;
		}
	}
	
	/* (non-Javadoc)
	 * @see java.awt.event.ComponentListener#componentShown(java.awt.event.ComponentEvent)
	 */
	public void componentShown(ComponentEvent e) {
		// nix
	}

	/* (non-Javadoc)
	 * @see java.awt.MenuContainer#getFont()
	 */
	public Font getFont() {
		return this.font.getUsedFont(FontHolder.PLAIN);
	}
	
	/**
	 * fragt die Rnder ab, die nicht beschrieben werden drfen
	 * 
	 * @return die Rnder
	 */
	public Insets getInsets() {
		return this.insets;
	}

	/* (non-Javadoc)
	 * @see java.awt.Component#getMaximumSize()
	 */
	public Dimension getMaximumSize() {
		return getPreferredSize();
	}
	
	/**
	 * fragt die maximale Smiley-Code-Lnge ab
	 * 
	 * @return die maximale Smiley-Code-Lnge
	 */
	public int getMaxSmileyCodeLength(){
		return this.maxSmileyCodeLength;
	}

	/* (non-Javadoc)
	 * @see java.awt.Component#getMinimumSize()
	 */
	public Dimension getMinimumSize() {
		return getPreferredSize();
	}
	
	/* (non-Javadoc)
	 * @see java.awt.Component#getPreferredSize()
	 */
	public synchronized Dimension getPreferredSize() {
		Dimension d = getSize();
		d.height = this.height + getInsets().top + getInsets().bottom;
		return d;
	}
	
	/**
	 * gibt den gerade markierten Text zurck
	 * 
	 * @return der markierte Text
	 */
	public String getSelectedText() {
		// keine Markierung?
		if ((this.selStartLineIndex == INVALID) || (this.selStartPos == INVALID)
		 || (this.selEndLineIndex == INVALID) || (this.selEndPos == INVALID)) {
			return ""; //$NON-NLS-1$
		}
		String newLine = "\n"; //$NON-NLS-1$
		int start = Math.max(0, this.selStartLineIndex);
		int end = Math.min(this.lines.size() - 1, this.selEndLineIndex);
		StringBuffer s = new StringBuffer();
		// durchgehen
		for (int i = start; i <= end; i++) {
			Line l = (Line)this.lines.elementAt(i);
			String text = l.getText() + newLine;
			if ((i == this.selEndLineIndex) && (this.selEndPos >= 0)
			 && (this.selEndPos < text.length())) {
				text = text.substring(0, this.selEndPos);
			} else if ((i == this.selEndLineIndex) && (this.selEndPos == ABOVE)) {
				// vor der aktuellen Zeile
				text = ""; //$NON-NLS-1$
			}
			if ((i == this.selStartLineIndex) && (this.selStartPos >= 0)
			 && (this.selStartPos < text.length())) {
				text = text.substring(this.selStartPos);
			}
			s.append(text);
		}
		return s.toString();
	}
	
	/**
	 * fragt ab, mit welcher Hintergrundfarbe die Markierung dargestellt wird
	 * 
	 * @return die Markierungs-Hintergrundfarbe oder null, wenn die
	 * Vordergrundfarbe benutzt werden soll 
	 */
	public Color getSelectionBgColor() {
		return this.selectionBgColor;
	}
	
	/**
	 * fragt ab, in welcher Farbe die Markierung dargestellt wird
	 * 
	 * @return die Markierungs-Farbe oder null, wenn die Hintergrundfarbe
	 * benutzt werden soll
	 */
	public Color getSelectionFgColor() {
		return this.selectionFgColor;
	}
	
	/**
	 * fragt das Image zu einem Smiley-Code ab
	 * 
	 * @param key der Smiley-Code
	 * @return das Image oder null, wenn keins vorhanden
	 */
	public Image getSmiley(String key) {
		return (Image)this.smileys.get(key.toLowerCase());
	}
	
	/**
	 * fragt die Einrckung folgender Zeilen ab
	 * 
	 * @return die Einrckung
	 */
	public int getSubsequentIndent() {
		return this.subsequentIndent;
	}
	
	/**
	 * fragt ab, mit welcher Hintergrundfarbe URLs dargestellt werden
	 * 
	 * @return die URL-Hintergrundfarbe oder null, wenn keine festgelegt 
	 */
	public Color getUrlBgColor() {
		return this.urlBgColor;
	}
	
	/**
	 * fragt ab, ob URLs fett dargestellt werden
	 * 
	 * @return FALSE, TRUE, INVERT oder NO_CHANGE
	 */
	public int getUrlBold() {
		return this.urlBold;
	}
	
	/**
	 * fragt ab, in welcher Farbe URLs dargestellt werden
	 * 
	 * @return die URL-Farbe oder null, wenn keine festgelegt
	 */
	public Color getUrlFgColor() {
		return this.urlFgColor;
	}
	
	/**
	 * fragt ab, ob URLs kursiv dargestellt werden
	 * 
	 * @return FALSE, TRUE, INVERT oder NO_CHANGE
	 */
	public int getUrlItalic() {
		return this.urlItalic;
	}
	
	/**
	 * gibt eine Liste aller URL-Prfixe zurck
	 * 
	 * @return die Liste
	 */
	public String[] getURLPrefixes() {
		String[] res = new String[this.urlPrefix.size()];
		this.urlPrefix.copyInto(res);
		return res;
	}
	
	/**
	 * fragt ab, ob URLs unterstrichen werden
	 * 
	 * @return FALSE, TRUE, INVERT oder NO_CHANGE
	 */
	public int getUrlUnderline() {
		return this.urlUnderline;
	}
	
	/**
	 * fragt ab, ob Antialiasing benutzt wird
	 * 
	 * @return true, wenn Antialiasing benutzt wird
	 */
	public boolean isAntialiasing() {
		return this.antialiasing;
	}
	
	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
	 */
	public void mouseClicked(MouseEvent e) {
		if (e.getSource() != this) return;
		if (!this.inSelection) {
			Point p = e.getPoint();
			p.y += this.scrollPos;
			String url = getURL(p);
			// wenn URL da
			if (url != null) {
				// URL-Klick
				fireURLActionPerformed(url);
			} else {
				// Markierung entfernen
				updateSelection(INVALID, INVALID, INVALID, INVALID);
				fireSelectionChanged();
			}
		}
	}
	
	/* (non-Javadoc)
	 * @see java.awt.event.MouseMotionListener#mouseDragged(java.awt.event.MouseEvent)
	 */
	public void mouseDragged(MouseEvent e) {
		if (e.getSource() != this) return;
		if ((this.tempSelStartPos != INVALID)
		 && (this.tempSelStartLineIndex != INVALID)) {
			this.inSelection = true;
			Point p = e.getPoint();
			p.y += this.scrollPos;
			// Endposition ermitteln
			int tempSelEndLineIndex = getLineIndex(p);
			int tempSelEndPos = getStringPos(p);
			// Markierung updaten
			updateSelection(this.tempSelStartLineIndex, this.tempSelStartPos,
				tempSelEndLineIndex, tempSelEndPos
			);
		}
	}
	
	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
	 */
	public void mouseEntered(MouseEvent e) {
		// nix
	}
	
	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
	 */
	public void mouseExited(MouseEvent e) {
		// nix
	}
	
	/* (non-Javadoc)
	 * @see java.awt.event.MouseMotionListener#mouseMoved(java.awt.event.MouseEvent)
	 */
	public void mouseMoved(MouseEvent e) {
		if (e.getSource() != this) return;
		Point p = e.getPoint();
		p.y += this.scrollPos;
		String url = getURL(p);
		// Cursor anpassen, wenn URL da
		if (url != null) {
			setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
		} else {
			setCursor(Cursor.getPredefinedCursor(Cursor.TEXT_CURSOR));
		}
	}
	
	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
	 */
	public void mousePressed(MouseEvent e) {
		if (e.getSource() != this) return;
		Point p = e.getPoint();
		p.y += this.scrollPos;
		// Startposition fr Markierung merken
		this.tempSelStartLineIndex = getLineIndex(p);
		this.tempSelStartPos = getStringPos(p);
	}
	
	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
	 */
	public void mouseReleased(MouseEvent e) {
		if (e.getSource() != this) return;
		// wenn Markierung, dann Event senden
		if (this.inSelection) {
			fireSelectionChanged();
		}
		// Werte zurcksetzen
		this.tempSelStartLineIndex = INVALID;
		this.tempSelStartPos = INVALID;
		this.inSelection = false;
	}
	
	/* (non-Javadoc)
	 * @see java.awt.Component#paint(java.awt.Graphics)
	 */
	public synchronized void paint(Graphics g) {
		Rectangle clipBounds = g.getClipBounds();
		// Werte ermitteln
		int y = this.insets.top;
		int size = this.lines.size();
		int top = clipBounds.y + this.scrollPos;
		int bottom = top + clipBounds.height;
		int w = getSize().width;
		// alle lines durchgehen
		for (int i = 0; i < size; i++) {
			Line l = (Line)this.lines.elementAt(i);
			int lineHeight = l.getHeight();
			// Zeile ber dem sichtbaren Bereich?
			if (y + lineHeight < top) {
				y += lineHeight;
				continue;
			}
			// Zeile unter dem sichtbaren Bereich?
			if (y > bottom) {
				break;
			}
			// Zeichnen
			l.paint(g, y - this.scrollPos);
			y += lineHeight;
		}
		// evtl. Flche unterhalb
		if (y < bottom) {
			g.setColor(getBackground());
			g.fillRect(0, y - this.scrollPos, w, bottom - y);
		}
		// oberer Rand, sofern im clip
		if (top < getInsets().top) {
			g.setColor(getBackground());
			g.fillRect(0, top - this.scrollPos, w, getInsets().top - top);
		}
	}
	
	/**
	 * trgt ein Smiley ein
	 * 
	 * @param keys die Smiley-Codes
	 * @param image das Image zum Smiley
	 */
	public void putSmiley(String[] keys, Image image) {
		// Image laden
		image.getWidth(null);
		// in Hashtable ablegen
		for (int i = 0; i < keys.length; i++) {
			this.smileys.put(keys[i].toLowerCase(), image);
			this.maxSmileyCodeLength
			 = Math.max(this.maxSmileyCodeLength, keys[i].length());
		}
	}
	
	/**
	 * trgt ein Smiley ein
	 * 
	 * @param keys die Smiley-Codes
	 * @param imageFile der Dateiname des Smileys
	 * @throws IOException wenn ein Fehler auftritt.
	 */
	public void putSmiley(String[] keys, String imageFile) throws IOException {
		// wenn was zu tun
		if (keys.length > 0) {
			// Datei laden
			File f = new File(imageFile);
			if (f.exists() && f.isFile() && f.canRead()) {
				InputStream is = new FileInputStream(f);
				byte[] data = new byte[(int)f.length()];
				// Datei einlesen
				is.read(data);
				// Image erstellen
				Image image = Toolkit.getDefaultToolkit().createImage(data);
				putSmiley(keys, image);
			}
		}
	}
	
	/**
	 * trgt ein Smiley ein
	 * 
	 * @param keys die Smiley-Codes
	 * @param imageURL die URL zum Smiley
	 * @throws IOException wenn ein Fehler auftritt.
	 */
	public void putSmiley(String[] keys, URL imageURL) throws IOException {
		// wenn was zu tun
		if (keys.length > 0) {
			// URL holen
			URLConnection c = imageURL.openConnection();
			InputStream is = c.getInputStream();
			// Lnge lesen; es kann -1 zurckkommen!
			int len = c.getContentLength();
			// Puffer reservieren (min 256 Byte)
			byte[] buf = new byte[(len > 256? len : 256)];
			byte[] data = new byte[(len > 256? len : 256)];
			int dataPos = 0;
			int readBytes = is.read(buf);
			// Einlesen, Puffer falls ntig vergrern
			while (readBytes >= 0) {
				if (readBytes + dataPos >= data.length) {
					byte[] temp = new byte[data.length * 2];
					System.arraycopy(data, 0, temp, 0, data.length);
					data = temp;
				}
				System.arraycopy(buf, 0, data, dataPos, readBytes);
				dataPos += readBytes;
				readBytes = is.read(buf);
			}
			// Image erstellen
			Image image = Toolkit.getDefaultToolkit().createImage(data, 0,
				dataPos
			);
			if (image == null) {
				throw new IOException("buuuuug"); //$NON-NLS-1$
			}
			putSmiley(keys, image);
		}
	}
	
	/**
	 * entfernt einen SelectionListener
	 * 
	 * @param l der Listener
	 */
	public void removeSelectionListener(SelectionListener l) {
		this.selectionListeners.removeElement(l);
	}
	
	/**
	 * entfernt einen URLListener
	 * 
	 * @param l der Listener
	 */
	public void removeURLListener(URLListener l) {
		this.urlListeners.removeElement(l);
	}
	
	/**
	 * legt fest, ob Antialiasing benutzt werden soll oder nicht
	 * 
	 * @param antialiasing true, wenn ANtialiasing benutzt werden soll
	 */
	public void setAntialiasing(boolean antialiasing) {
		this.antialiasing = antialiasing;
		if (getParent() != null) {
			// neue Gre bekanntgeben
			getParent().doLayout();
		}
	}
	
	/* (non-Javadoc)
	 * @see java.awt.Component#setFont(java.awt.Font)
	 */
	public void setFont(Font f) {
		this.font = new FontHolder(f);
		super.setFont(this.font.getUsedFont(FontHolder.PLAIN));
		sizeCalc();
		if (getParent() != null) {
			// neue Gre bekanntgeben
			getParent().doLayout();
		}
		repaint();
	}
	
	/**
	 * setzt die nicht beschreibbaren Rnder neu
	 * 
	 * @param insets das neue Rand-Objekt
	 */
	public void setInsets(Insets insets) {
		this.insets = (Insets)insets.clone();
		// Gre berechnen
		sizeCalc();
		if (getParent() != null) {
			// neue Gre bekanntgeben
			getParent().doLayout();
		}
	}
	
	/**
	 * legt fest, mit welcher Hintergrundfarbe die Markierung dargestellt wird
	 * 
	 * @param color die Markierungs-Hintergrundfarbe oder null, wenn die
	 * Vordergrundfarbe benutzt werden soll (Default: null)
	 */
	public void setSelectionBgColor(Color color) {
		this.selectionBgColor = color;
	}
	
	/**
	 * legt fest, in welcher Farbe die Markierung dargestellt wird
	 * 
	 * @param color die Markierungs-Farbe oder null, wenn die Hintergrundfarbe
	 * benutzt werden soll (Default: null)
	 */
	public void setSelectionFgColor(Color color) {
		this.selectionFgColor = color;
	}
	
	/**
	 * setzt die Einrckung folgender Zeilen
	 * 
	 * @param i die neue Einrckung
	 */
	public void setSubsequentIndent(int i) {
		this.subsequentIndent = i;
		// Gre berechnen
		sizeCalc();
		if (getParent() != null) {
			// neue Gre bekanntgeben
			getParent().doLayout();
		}
	}
	
	/**
	 * legt fest, mit welcher Hintergrundfarbe URLs dargestellt werden
	 * 
	 * @param color die URL-Hintergrundfarbe oder null, wenn die aktuelle
	 * Hintergrundfarbe benutzt werden soll (Default: null)
	 */
	public void setUrlBgColor(Color color) {
		this.urlBgColor = color;
	}
	
	/**
	 * legt fest, ob URLs fett dargestellt werden
	 * 
	 * mgliche Werte sind:
	 * <ul>
	 * <li>FALSE - URLs sind nie fett</li>
	 * <li>TRUE - URLs sind immer fett</li>
	 * <li>INVERT - URLs sind genau dann fett, wenn es normaler Text an dieser
	 * Stelle nicht wre</li>
	 * <li>NO_CHANGE - URLs sind fett, wenn das im eingefgten String so
	 * steht (Default)</li>
	 * </ul>
	 * 
	 * @param i FALSE, TRUE, INVERT oder NO_CHANGE
	 */
	public void setUrlBold(int i) {
		if (i < 0 || i > 3) {
			throw new IllegalArgumentException();
		}
		this.urlBold = i;
	}
	
	/**
	 * legt fest, in welcher Farbe URLs dargestellt werden
	 * 
	 * @param color die URL-Farbe oder null, wenn die aktuelle Zeichenfarbe
	 * benutzt werden soll (Default: Color.blue)
	 */
	public void setUrlFgColor(Color color) {
		this.urlFgColor = color;
	}
	
	/**
	 * legt fest, ob URLs kursiv dargestellt werden
	 * 
	 * mgliche Werte sind:
	 * <ul>
	 * <li>FALSE - URLs sind nie kursiv</li>
	 * <li>TRUE - URLs sind immer kursiv</li>
	 * <li>INVERT - URLs sind genau dann kursiv, wenn es normaler Text an dieser
	 * Stelle nicht wre</li>
	 * <li>NO_CHANGE - URLs sind kursiv, wenn das im eingefgten String so
	 * steht (Default)</li>
	 * </ul>
	 * 
	 * @param i FALSE, TRUE, INVERT oder NO_CHANGE
	 */
	public void setUrlItalic(int i) {
		if (i < 0 || i > 3) {
			throw new IllegalArgumentException();
		}
		this.urlItalic = i;
	}
	
	/**
	 * legt fest, ob URLs unterstrichen werden
	 * 
	 * mgliche Werte sind:
	 * <ul>
	 * <li>FALSE - URLs sind nie unterstrichen</li>
	 * <li>TRUE - URLs sind immer unterstrichen (Default)</li>
	 * <li>INVERT - URLs sind genau dann unterstrichen, wenn es normaler Text an
	 * dieser Stelle nicht wre</li>
	 * <li>NO_CHANGE - URLs sind unterstrichen, wenn das im eingefgten String
	 * so steht</li>
	 * </ul>
	 * 
	 * @param i FALSE, TRUE, INVERT oder NO_CHANGE
	 */
	public void setUrlUnderline(int i) {
		if (i < 0 || i > 2) {
			throw new IllegalArgumentException();
		}
		this.urlUnderline = i;
	}
	
	/* 
	 * (non-Javadoc)
	 * @see java.awt.Component#update(java.awt.Graphics)
	 */
	public void update(Graphics g) {
		paint(g);
		//super.update(g);
	}
	
	/**
	 * gibt den FontHolder mit den Daten zur aktuell verwendeten Schrift zurck
	 * 
	 * @return das FontHolder-Objekt
	 */
	FontHolder getFontHolder() {
		return this.font;
	}
	
	/**
	 * berechnet Start- und Endposition fr repaint()
	 * 
	 * @param changed wurde die aktuelle Line gendert?
	 * @param pos ein Array aus 3 Elementen: startY, endY, position
	 * @param h die Hhe der aktuellen Line
	 */
	private void calcPos(boolean changed, int[] pos, int h) {
		// nicht gendert => nur die Position hochzhlen
		if (!changed) {
			pos[2] += h; //position
			return;
		}
		// gendert, aber start == -1 => 1. zu zeichnendes feld; Start und Ende
		// setzen
		if (pos[0] == -1) { //startY
			pos[0] = pos[2]; //startY, position
			pos[2] += h; //position
			pos[1] = pos[2]; //endY, position
			return;
		}
		// nur Ende setzen
		pos[2] += h; //position
		pos[1] = pos[2]; //endY, position
	}
	
	/**
	 * versendet selectionChanged an alle registrierten Klienten
	 */
	private void fireSelectionChanged() {
		if (this.selectionListeners.size() > 0) {
			SelectionEvent ev = new SelectionEvent(this);
			Enumeration e = this.selectionListeners.elements();
			while(e.hasMoreElements()) {
				SelectionListener l = (SelectionListener)e.nextElement();
				l.selectionChanged(ev);
			}
		}
	}
	
	/**
	 * versendet urlActionPerformed an alle registrierten Klienten
	 * 
	 * @param url die URL, die geklickt wurde
	 */
	private void fireURLActionPerformed(String url){
		if (this.urlListeners.size() > 0){
			URLEvent ev = new URLEvent(this, url);
			Enumeration e = this.urlListeners.elements();
			while(e.hasMoreElements()){
				URLListener l = (URLListener)e.nextElement();
				l.urlActionPerformed(ev);
			}
		}
	}
	
	/**
	 * bestimmt die Line an einem gegebenen Punkt
	 * 
	 * @param p der Punkt
	 * @return die Line an diesem Punkt oder null, wenn keine vorhanden
	 */
	private int getLineIndex (Point p) {
		// kein LinePart da
		if (this.lines.size() == 0) {
			return ABOVE;
		}
		int y = getInsets().top;
		int i = -1;
		while ((y < p.y) && (++i < this.lines.size())) {
			Line l = (Line)this.lines.elementAt(i);
			y += l.getHeight();
		}
		// vor erster Line
		if (i == -1) {
			return ABOVE;
		}
		// alle durchsucht, nix gefunden
		if (i == this.lines.size()) {
			return BELOW;
		}
		return i;
	}
	
	/**
	 * berechnet zu einem gegebenen Punkt den LinePart und dort die Position in
	 * dessen String
	 * 
	 * @param p der Punkt
	 * @return die Position im String (oder BELOW bzw. ABOVE, wenn p auerhalb
	 *         ist)
	 */
	private int getStringPos (Point p) {
		int lineIndex = getLineIndex(p);
		// keine Line
		if ((lineIndex < 0) || (lineIndex >= this.lines.size())) {
			return lineIndex;
		}
		// Rckgabe
		return ((Line)this.lines.elementAt(lineIndex)).getStringPos(
			pointTranslate(p, lineIndex)
		);
	}
	
	/**
	 * bestimmt die URL des LineParts an einem gegebenen Punkt
	 * 
	 * @param p der Punkt
	 * @return die URL oder null, wenn kein LinePart oder keine URL vorhanden
	 */
	private String getURL(Point p) {
		int lineIndex = getLineIndex(p);
		if ((lineIndex < 0) || (lineIndex >= this.lines.size())) {
			return null;
		}
		p = pointTranslate(p, lineIndex);
		return ((Line)this.lines.elementAt(lineIndex)).getURL(p);
	}
	
	/**
	 * transformiert einen Punkt in das lokale Koordinatensystem einer Line
	 * 
	 * @param p der zu transformierte Punkt
	 * @param lineIndex die Line, in dessen Koordinatensystem transformiert
	 *        werden soll
	 * @return der transformierte Punkt
	 */
	private Point pointTranslate(Point p, int lineIndex) {
		// Nullwerte abfangen
		if ((lineIndex < 0) || (lineIndex >= this.lines.size()) || (p == null)) {
			return null;
		}
		// Position im lokate Koordinaten umrechnen
		int y = getInsets().top;
		int i = 0;
		while (i < lineIndex) {
			y += ((Line)this.lines.elementAt(i++)).getHeight();
		}
		// Rckgabe
		return new Point(p.x, p.y - y);
	}
	
	/**
	 * berechnet die Gre dieser Komponente
	 */
	private void sizeCalc() {
		this.height = 0;
		int lineCount = this.lines.size();
		for (int i = 0; i < lineCount; i++){
			Line l = (Line)this.lines.elementAt(i);
			this.height += l.sizeCalc();
		}
	}
	
	/**
	 * setzt neue Werte fr die MArkierung und teilt sie den Kindobjekten mit
	 * 
	 * @param startLineIndex die Nummer der Startzeile der Markierung
	 * @param startPos die Startposition innerhalb der Startzeile
	 * @param endLineIndex die Nummer der Endzeile der Markierung
	 * @param endPos die Endposition innerhalb der Endzeile
	 */
	synchronized private void updateSelection(int startLineIndex, int startPos,
	 int endLineIndex, int endPos) {
		// vertauschen falls ntig
		if ((startLineIndex > endLineIndex)
		 || ((startLineIndex == endLineIndex) && (startPos > endPos))) {
			int temp = startLineIndex;
			startLineIndex = endLineIndex;
			endLineIndex = temp;
			temp = startPos;
			startPos = endPos;
			endPos = temp;
		}
		// ein Wert == INVALID => alles INVALID
		if ((startLineIndex == INVALID) || (startPos == INVALID)
		 || (endLineIndex == INVALID) || (endPos == INVALID)) {
			startLineIndex = INVALID;
			startPos = INVALID;
			endLineIndex = INVALID;
			endPos = INVALID;
		}
		// trivialer Fall: nix hat sich gendert
		if ((startLineIndex == this.selStartLineIndex) && (startPos == this.selStartPos)
		 && (endLineIndex == this.selEndLineIndex) && (endPos == this.selEndPos)) {
			return;
		}
		// ein Wert == INVALID => Markierung lschen
		if (startLineIndex == INVALID) {
			int startY = getInsets().top;
			int yRange = 0;
			for (int i = 0; i < this.lines.size(); i++) {
				Line l = (Line)this.lines.elementAt(i);
				int lineHeight = l.getHeight();
				if (i < this.selStartLineIndex) {
					startY += lineHeight;
					continue;
				}
				yRange += lineHeight;
				l.setSelection(INVALID, INVALID);
				if (i >= this.selEndLineIndex) {
					break;
				}
			}
			repaint(0, startY - this.scrollPos, getSize().width, yRange);
			this.selStartLineIndex = INVALID;
			this.selStartPos = INVALID;
			this.selEndLineIndex = INVALID;
			this.selEndPos = INVALID;
			return;
		}
		// Markierung neu berechnen
		int i = 0;
		int[] pos = new int[3];
		pos[0] = -1; //startY
		pos[1] = -1; //endY
		pos[2] = getInsets().top; //position
		// nichtmarkierte Lines vorher
		if ((startLineIndex >= 0) && (startLineIndex < this.lines.size())) {
			for (; i < this.lines.size(); i++) {
				Line l = (Line)this.lines.elementAt(i);
				if (i == startLineIndex) {
					if (i == endLineIndex) {
						calcPos(l.setSelection(startPos, endPos), pos,
						 l.getHeight());
					} else {
						calcPos(l.setSelection(startPos, l.getText().length()),
						 pos, l.getHeight());
					}
					i++;
					break;
				}
				calcPos(l.setSelection(INVALID, INVALID), pos, l.getHeight());
			}
		}
		// markierte Lines
		if (((startLineIndex >= 0) && startLineIndex < this.lines.size())
		 || ((startPos == ABOVE) && (endPos != ABOVE))) {
			if ((i <= endLineIndex) || (endLineIndex < 0)) {
				for (; i < this.lines.size(); i++) {
					Line l = (Line)this.lines.elementAt(i);
					if (i == endLineIndex) {
						calcPos(l.setSelection(0, endPos), pos, l.getHeight());
						i++;
						break;
					}
					calcPos(l.setSelection(0,  l.getText().length()), pos,
					 l.getHeight());
				}
			}
		}
		// nichtmarkierte Lines nachher
		for (; i < this.lines.size(); i++) {
			Line l = (Line)this.lines.elementAt(i);
			calcPos(l.setSelection(INVALID, INVALID), pos, l.getHeight());
		}
		// Werte merken
		this.selStartLineIndex = startLineIndex;
		this.selStartPos = startPos;
		this.selEndLineIndex = endLineIndex;
		this.selEndPos = endPos;
		// nur bentigte Lines neu zeichnen
		repaint(0, pos[0] - this.scrollPos, getSize().width, pos[1] - pos[0] + 1);
	}
	
	/**
	 * entfernt die Markierung
	 */
	public void removeSelection() {
		updateSelection(INVALID, INVALID, INVALID, INVALID);
	}
	
	/**
	 * fragt ab, ob gegenwrtig markiert wird
	 * 
	 * @return true, wenn gerade markiert wird
	 */
	public boolean isInSelection() {
		return this.inSelection;
	}
	
	/* 
	 * (non-Javadoc)
	 * @see ec4.tools.ui.Scrollable#scrollTo(int)
	 */
	public synchronized void scrollTo(int pos) {
		// pos < 0 geht net
		if (pos < 0) {
			throw
			 new IllegalArgumentException("pos has to be >= 0"); //$NON-NLS-1$
		}
		// alten Wert speichern
		int oldScrollPos = this.scrollPos;
		this.scrollPos = pos;
		Dimension d = getSize();
		int h = d.height;
		int w = d.width;
		// Maximum bercksichtigen
		int max = this.height + this.insets.top + this.insets.bottom - h;
		if (this.scrollPos > max) {
			this.scrollPos = max;
		}
		// gleiche Position => fertig
		if (oldScrollPos == this.scrollPos) {
			return;
		}
		// mehr als 1 Seite gescrollt => neuzeichnen, fertig
		if (Math.abs(this.scrollPos - oldScrollPos) >= h) {
			repaint();
			return;
		}
		// Zeichnen
		Graphics g = getGraphics();
		if (g == null) {
			repaint();
			return;
		}
		// das, was sich nicht ndert kopieren
		try {
			g.copyArea(0, 0, w, h, 0, oldScrollPos - this.scrollPos);
		} catch (NullPointerException e) {
			repaint();
			return;
		}
		// den Rest bestimmen und zeichnen
		int paintTop;
		int paintHeight;
		if (this.scrollRepaint) {
			paintTop = 0;
			paintHeight = h;
		} else {
			// nach oben verschoben (scrollPos wurde grer)
			if (this.scrollPos > oldScrollPos) {
				paintTop = h - (this.scrollPos - oldScrollPos) - 1;
				paintHeight = this.scrollPos - oldScrollPos + 1;
			// nach unten verschoben
			} else {
				paintTop = 0;
				paintHeight = oldScrollPos - this.scrollPos + 1;
			}
		}
		repaint(0, paintTop, w, paintHeight);
	}
	
	/**
	 * fragt ab, ob beim Scrollen alles neu gezeichnet wird oder nicht
	 * 
	 * @return true, wenn alles neu gezeichnet wird
	 */
	public boolean isScrollRepaint() {
		return this.scrollRepaint;
	}
	
	/**
	 * legt fest, ob beim Scrollen alles neu gezeichnet wird oder nicht
	 * 
	 * @param scrollRepaint true, wenn alles neu gezeichnet werden soll
	 */
	public void setScrollRepaint(boolean scrollRepaint) {
		this.scrollRepaint = scrollRepaint;
	}
}