/*
 * Created on 13.06.2004
 */
package ec4.tools.ui;

import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Toolkit;

/**
 * stellt eine Schriftart mit ihren Abwandlungen(fett, kursiv) und ihren
 * Abmessungen bereit
 * 
 * @author AM
 */
class FontHolder {
	
	/**
	 * normale Schrift
	 */
	static final int PLAIN = 0;
	
	/**
	 * fette Schrift
	 */
	static final int BOLD = 1;
	
	/**
	 * kursive Schrift
	 */
	static final int ITALIC = 2;
	
	/**
	 * fette und kursive Schrift
	 */
	static final int BOLD_ITALIC = BOLD | ITALIC;
	
	/**
	 * die benutzten Schriften
	 */
	private Font usedFonts[];
	
	/**
	 * die FomtMetric-Objekte der benutzten Schriften
	 */
	private FontMetrics fontMetrics[];
	
	/**
	 * der maximale Aufstrich
	 */
	private int fontMetricAscent;
	
	/**
	 * die Aufstriche der einzelnen Schriftvarianten
	 */
	private int fontMetricAscentFonts[];
	
	/**
	 * der maximale Abstrich
	 */
	private int fontMetricDescent;
	
	/**
	 * die Abstriche der einzelnen Schriftvarianten
	 */
	private int fontMetricDescentFonts[];
	
	/**
	 * der maximale Zeilenzwischenraum
	 */
	private int fontMetricLeading;
	
	/**
	 * die Zeilenhhe
	 */
	private int fontMetricLineHeight;
	
	/**
	 * die Breite eines Leerzeichens
	 */
	private int fontMetricSpaceWidth;
	
	/**
	 * berechnet eine Kombination aus fett und kirsiv
	 * 
	 * @param bold fett?
	 * @param italic kursiv?
	 * @return eine der Konstanten PLAIN, BOLD, ITALIC, BOLD_ITALIC
	 */
	static int combine(boolean bold, boolean italic) {
		return (bold ? BOLD : PLAIN) | (italic ? ITALIC : PLAIN);
	}
	
	/**
	 * erstellt ein neues FontHolder-Objekt
	 * 
	 * @param f der zu verwendende Font
	 */
	FontHolder(Font f) {
		this(f.getName(), f.getSize());
	}
	
	/**
	 * erstellt ein neues FontHolder-Objekt
	 * 
	 * @param name der zu verwendende Font
	 * @param size die zu verwendende Schriftgre
	 */
	FontHolder(String name, int size) {
		super();
		this.usedFonts = new Font[4];
		this.fontMetrics = new FontMetrics[4];
		this.fontMetricDescentFonts = new int[4];
		this.fontMetricAscentFonts = new int[4];
		prepareFont(name, size);
	}
	
	/**
	 * gibt eine Font-Variante zurck
	 * 
	 * @param i die Id des gewnschten FOnts
	 * @return den gewnschten Font
	 */
	Font getUsedFont(int i) {
		return this.usedFonts[i];
	}
	
	/**
	 * gibt ein FontMetrics-Objekt einer Schriftvariante (normal, fett, kursiv,
	 * fett+kursiv)zurck
	 * 
	 * @param i die Id des FontMetrics-Objekt
	 * @return das FontMetrics-Objekt
	 */
	FontMetrics getFontMetrics(int i) {
		return this.fontMetrics[i];
	}
	
	/**
	 * gibt den maximalen Aufstrich der Schriftvarianten (normal, fett, kursiv,
	 * fett+kursiv) zurck
	 * 
	 * @return der maximale Aufstrich
	 */
	int getFontMetricAscent() {
		return this.fontMetricAscent;
	}
	
	/**
	 * gibt den Aufstrich einer Schriftvariante (normal, fett, kursiv,
	 * fett+kursiv)zurck
	 * 
	 * @param i die Variante
	 * @return der Aufstrich
	 */
	int getFontMetricAscent(int i) {
		return this.fontMetricAscentFonts[i];
	}
	
	/**
	 * gibt den maximalen Abstrich der Schriftvarianten (normal, fett, kursiv,
	 * fett+kursiv) zurck
	 * 
	 * @return der maximale Abstrich
	 */
	int getFontMetricDescent() {
		return this.fontMetricDescent;
	}
	
	/**
	 * gibt den Abstrich einer Schriftvariante (normal, fett, kursiv,
	 * fett+kursiv)zurck
	 * 
	 * @param i die Variante
	 * @return der Abstrich
	 */
	int getFontMetricDescent(int i) {
		return this.fontMetricDescentFonts[i];
	}
	
	/**
	 * gibt den maximalen Zeilenzwischenraum der Schriftvarianten (normal, fett,
	 * kursiv, fett+kursiv) zurck
	 * 
	 * @return der Zeilenzwischenraum
	 */
	int getFontMetricLeading() {
		return this.fontMetricLeading;
	}
	
	/**
	 * gibt die Hhe (incl. Zeilenzwischenraum) der gewhlten Schrift zurck
	 * 
	 * @return die Schrifthhe
	 */
	int getFontMetricLineHeight() {
		return this.fontMetricLineHeight;
	}
	
	/**
	 * gibt die durchschnittliche Leerzeichenbreite der Schriftvarianten
	 * (normal, fett, kursiv, fett+kursiv) zurck
	 * 
	 * @return die Leerzeichenbreite
	 */
	int getFontMetricSpaceWidth() {
		return this.fontMetricSpaceWidth;
	}
	
	/**
	 * ermittelt die Abmessungen der bergebenen Schriftart und setzt die
	 * internen Felder entsprechend 
	 * 
	 * @param name der Name der Schriftart
	 * @param size die Schriftgre
	 */
	private void prepareFont(String name, int size) {
		// Schriftvatianten ableiten
		this.usedFonts[0] = new Font(name, Font.PLAIN, size);
		this.usedFonts[1] = new Font(name, Font.BOLD, size);
		this.usedFonts[2] = new Font(name, Font.ITALIC, size);
		this.usedFonts[3] = new Font(name, Font.BOLD | Font.ITALIC, size);
		// Mae initalisieren
		this.fontMetricLineHeight = 0;
		this.fontMetricDescent = 0;
		this.fontMetricAscent = 0;
		this.fontMetricLeading = 9999;
		// alle Schriftvarianten durchgehen
		for (int i = 0; i < 4; i++) {
			// FontMetrics beschaffen
			FontMetrics fm
			 = Toolkit.getDefaultToolkit().getFontMetrics(usedFonts[i]);
			this.fontMetrics[i] = fm;
			// Mae beschaffen
			this.fontMetricSpaceWidth += fm.stringWidth(" "); //$NON-NLS-1$
/*			fontMetricDescentFonts[i] = fm.getMaxDescent();
			fontMetricAscentFonts[i] = fm.getMaxAscent();
			// fontMetricAscent ist Maximum aller Werte
			if (fontMetricAscent < fm.getMaxAscent()) {
				fontMetricAscent = fm.getMaxAscent();
			}
			// fontMetricDescent ist Maximum aller Werte
			if (fontMetricDescent < fm.getMaxDescent()) {
				fontMetricDescent = fm.getMaxDescent();
			}*/
			this.fontMetricDescentFonts[i] = fm.getDescent();
			this.fontMetricAscentFonts[i] = fm.getAscent();
			// fontMetricAscent ist Maximum aller Werte
			if (this.fontMetricAscent < fm.getAscent()) {
				this.fontMetricAscent = fm.getAscent();
			}
			// fontMetricDescent ist Maximum aller Werte
			if (this.fontMetricDescent < fm.getDescent()) {
				this.fontMetricDescent = fm.getDescent();
			}
			// fontMetricLeading ist Mimimum aller Werte
			if (this.fontMetricLeading > fm.getLeading()) {
				this.fontMetricLeading = fm.getLeading();
			}
		}
		// mindestens 0
		if (this.fontMetricLeading < 0) {
			this.fontMetricLeading = 0;
		}
		// Zeilenhhe berechnen
		this.fontMetricLineHeight = this.fontMetricDescent + this.fontMetricAscent
		 + this.fontMetricLeading;
		// +2 zum Runden; Durchschnitt bilden
		this.fontMetricSpaceWidth = (this.fontMetricSpaceWidth + 2) / 4;
	}
	
}
