/*
 * Created on 14.06.2004
 */
package ec4.tools.ui;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Point;
import java.util.Vector;

/**
 * ein LinePart, der ein Bild darstellen kann
 * 
 * @author AM
 */
class ImageLinePart implements LinePart {
	
	/**
	 * die ColorTextArea, zu der dieser LinePart gehrt
	 */
	private ColorTextArea cTA;
	
	/**
	 * der Text, der diesem LinePart zugeordnet ist
	 */
	private String text;
	
	/**
	 * das Bild, das diesem LinePart zugeordnet ist
	 */
	private Image image;
	
	/**
	 * die Vordergrundfarbe dieses LineParts
	 */
	private Color fgColor;
	
	/**
	 * die Vordergrundfarbe dieses LineParts
	 */
	private Color bgColor;
	
	/**
	 * die Startzeile dieses LineParts
	 */
	private int startLine;
	
	/**
	 * die Endzeile dieses LineParts
	 */
	private int endLine;
	
	/**
	 * die Startposition dieses LineParts
	 */
	private int startX;
	
	/**
	 * die Endposition dieses LinePArts
	 */
	private int endX;
	
	/**
	 * true, wenn dieser LinePart markiert ist
	 */
	private boolean selected;
	
	/**
	 * erstellt einen neuen ImageLinePart
	 * 
	 * @param cTA die ColorTextArea, zu der dieses Line-Objekt gehrt
	 * @param text der Text zum Bild
	 * @param image das anzuzeigende Bild
	 * @param fgColor die Vordergrundfarbe
	 * @param bgColor die Hintergrundfarbe
	 */
	ImageLinePart(ColorTextArea cTA, String text, Image image,
	 Color fgColor, Color bgColor) {
		super();
		this.cTA = cTA;
		this.text = text;
		this.image = image;
		this.fgColor = fgColor;
		this.bgColor = bgColor;
	}
	
	/* (non-Javadoc)
	 * @see ec4.tools.ui.LinePart#getText()
	 */
	public String getText() {
		return this.text;
	}

	/* (non-Javadoc)
	 * @see ec4.tools.ui.LinePart#getURL()
	 */
	public String getURL() {
		return null;
	}

	/* (non-Javadoc)
	 * @see ec4.tools.ui.LinePart#paint(java.awt.Graphics)
	 */
	public void paint(Graphics g, Vector lineHeights) {
		// vertikale Position
		int y = 0;
		// Schriftmae holen
		FontHolder font = this.cTA.getFontHolder();
		int descent = font.getFontMetricDescent(FontHolder.PLAIN);
		// Positionsberechnung
		int height = this.image.getHeight(null);
		int width = this.image.getWidth(null);
		int lineHeight = ((Integer)lineHeights.elementAt(this.startLine)).intValue();
		// Hhen der vorhergehenden Zeilen addieren
		for (int i = 0; i < this.startLine; i++) {
			y += ((Integer)lineHeights.elementAt(i)).intValue();
		}
		// unteres Zeilenende
		int yOff = lineHeight;
		// Basislinie der Zeile
		yOff -= descent;
		// Bild um Basislinie zentrieren
		yOff -= height / 2;
		// aber nicht unter die Zeile schieben
		if (yOff + height > lineHeight) {
			yOff = lineHeight - height;
		}
		// Hintergrund zeichnen
		g.setColor(this.selected ? this.fgColor : this.bgColor);
		//g.fillRect(startX, y + yOff, width, yOff + height);
		g.fillRect(this.startX, y + yOff, width, height);
		// Bild zeichnen
		g.drawImage(this.image, this.startX, y + yOff, null);
	}

	/* (non-Javadoc)
	 * @see ec4.tools.ui.LinePart#sizeCalc()
	 */
	public int sizeCalc(int x, Vector lineHeights) {
		// Schriftmae holen
		FontHolder font = this.cTA.getFontHolder();
		int leading = font.getFontMetricLeading();
		if (leading < 2) {
			leading = 2;
		}
		// Anfangsposition speichern
		this.startLine = lineHeights.size() - 1;
		this.startX = x;
		// wenn kein Platz in aktueller Zeile => Umbruch
		if (x > this.cTA.getSize().width - this.image.getWidth(null)) {
			this.startLine++;
			this.startX = this.cTA.getInsets().left + this.cTA.getSubsequentIndent();
			lineHeights.addElement(new Integer(0));
		}
		// Zeilenhhe anpassen
		int lineHeight
		 = ((Integer)lineHeights.elementAt(this.startLine)).intValue();
		lineHeight
		 = Math.max(lineHeight, this.image.getHeight(null) + leading);
		lineHeights.setElementAt(new Integer(lineHeight), this.startLine);
		// Endkoordinaten berechnen
		this.endX = this.startX + this.image.getWidth(null);
		this.endLine = this.startLine;
		return this.endX;
	}

	/* (non-Javadoc)
	 * @see ec4.tools.ui.LinePart#getStringPos(java.awt.Point)
	 */
	public int getStringPos(Point p) {
		// Zeilennummer vergleichen
		if (p.y < this.startLine || p.y > this.endLine) {
			return -1;
		}
		// x-Koordinate vergleichen
		if (p.x < this.startX || p.x > this.endX) {
			return -1;
		}
		// Berechnung
		if (p.x - this.startX < this.image.getWidth(null) / 2) {
			return 0;
		}
		return this.text.length();
	}

	/* (non-Javadoc)
	 * @see ec4.tools.ui.LinePart#getStartX()
	 */
	public int getStartX() {
		return this.startX;
	}

	/* (non-Javadoc)
	 * @see ec4.tools.ui.LinePart#getEndX()
	 */
	public int getEndX() {
		return this.endX;
	}

	/* (non-Javadoc)
	 * @see ec4.tools.ui.LinePart#getStartLine()
	 */
	public int getStartLine() {
		return this.startLine;
	}

	/* (non-Javadoc)
	 * @see ec4.tools.ui.LinePart#getEndLine()
	 */
	public int getEndLine() {
		return this.endLine;
	}

	/* (non-Javadoc)
	 * @see ec4.tools.ui.LinePart#setSelection(int, int)
	 */
	public boolean setSelection(int start, int end) {
		boolean oldSelected = this.selected;
		this.selected = (start >= 0) && (end >= 0) && (start <= end)
		 && (end <= this.text.length()) && (end - start > 0);
		return this.selected != oldSelected;
	}

}
