/*
 * Created on 14.06.2004
 */
////#INCLUDE ../../../../defines
package ec4.tools.ui;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Point;
import java.util.Vector;

import ec4.tools.version.VersionController;

/**
 * Line stellt eine logische Zeile dar, die aus mehreren LineParts besteht und
 * auf mehrere physische Zeilen verteilt ausgegeben wird
 *
 * @author AM
 */
final class Line {

	/**
	 * die ColorTextArea, zu der diese Line gehrt
	 */
	private ColorTextArea cTA;

	/**
	 * der Text dieser Line
	 */
	private String text;

	/**
	 * die LineParts, mit denen diese Line gerendert wird
	 */
	private Vector lineParts;

	/**
	 * die Hhe dieser Line (in Pixel)
	 */
	private int height;

	/**
	 * die Hhen der einzelnen physischen Zeilen (in Pixel)
	 */
	private Vector lineHeights;

	/**
	 * der Puffer, auf den gezeichnet wird
	 */
	private Image buffer;

	/**
	 * true, wenn der Puffer gltig ist
	 */
	private boolean bufferValid;

	/**
	 * erstellt ein neues Line-Objekt
	 *
	 * @param cTA die ColorTextArea, zu der dieses Line-Objekt gehrt
	 * @param text der String, der den Inhalt dieses Line-Objekts dartstellt
	 */
	Line(ColorTextArea cTA, String text) {
		// 1. NewLines umformen
		text = join(split(join(split(text, "\r\n"), "\n"), "\r"), "\n"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		this.cTA = cTA;
		this.text = strip(text);
		if (text.equals("")) { //$NON-NLS-1$
			text = " "; //$NON-NLS-1$
		}
		this.lineParts = new Vector();
		this.height = 0;
		this.lineHeights = new Vector();
		this.buffer = null;
		this.bufferValid = false;
		// Lnge von s zwischenspeichern
		int len = text.length();
		// Ende eines Links
		int hrefEnd = -1;
		// Anfang des aktuellen Textteils
		int start = 0;
		// Formatierungseinstellungen
		boolean bold = false;
		boolean italic = false;
		boolean underline = false;
		Color fgColor = cTA.getForeground();
		Color bgColor = cTA.getBackground();
		// URL-Formatierung
		int urlBold = cTA.getUrlBold();
		int urlItalic = cTA.getUrlItalic();
		int urlUnderline = cTA.getUrlUnderline();
		Color urlFgColor = cTA.getUrlFgColor();
		Color urlBgColor = cTA.getUrlBgColor();
		for(int i = 0; i < len; i++) {
			// E-Mail-Adressen suchen
			if (i > hrefEnd) {
				// Ende des Teils vor @ finden
				hrefEnd = findEndOfHref(text, i);
				// wenn @ folgt => Mail
				if (i < hrefEnd && hrefEnd < len
				 && text.charAt(hrefEnd) == '@') {
					// evtl Text vorher noch ausgeben
					if (start < i) {
						this.lineParts.addElement(new TextLinePart(
							cTA, text.substring(start, i), bold, italic,
							underline, fgColor, bgColor
						));
					}
					// Ende der Adresse suchen
					int emailEnd = findEndOfHref(text, hrefEnd + 1);
					// String extrahieren
					String emailStr = text.substring(i, emailEnd);
					// URL bauen
					String emailUrl = emailStr;
					// evtl. mailto: anfgen
					if (!emailUrl.toUpperCase()
					 .startsWith("MAILTO:")) { //$NON-NLS-1$
						emailUrl = "mailto:" + emailUrl; //$NON-NLS-1$
					}
					// LinePart erzeugen
					this.lineParts.addElement(new TextLinePart(
						cTA, emailStr, decode(urlBold, bold),
						decode(urlItalic, italic),
						decode(urlUnderline, underline),
						(urlFgColor == null ? fgColor : urlFgColor),
						(urlBgColor == null ? bgColor : urlBgColor),
						emailUrl
					));
					// i => Ende der Adresse
					i = emailEnd - 1;
					// start => Ende der Adresse
					start = emailEnd;
					// Schleife nach der Adresse erneut starten
					continue;
				}
			}//if Email
			// URLs suchen; eine URL beginnt nie mitten im Wort
			if (isHref(text.substring(i)) && !Character.isLetterOrDigit(text.charAt(i - 1))) {
				// evtl Text vorher noch ausgeben
				if (start < i) {
					this.lineParts.addElement(new TextLinePart(
						cTA, text.substring(start, i), bold, italic,
						underline, fgColor, bgColor
					));
				}
				// String extrahieren
				String urlStr = text.substring(i, hrefEnd);
				// URL bauen
				String url = urlStr;
				// evtl. Protokoll anfgen
				if (url.toUpperCase().startsWith("WWW.")) { //$NON-NLS-1$
					url = "http://" + url; //$NON-NLS-1$
				} else if (url.toUpperCase().startsWith("FTP.")) { //$NON-NLS-1$
					url = "ftp://" + url; //$NON-NLS-1$
				}
				// LinePart erzeugen
				this.lineParts.addElement(new TextLinePart(
					cTA, urlStr, decode(urlBold, bold),
					decode(urlItalic, italic),
					decode(urlUnderline, underline),
					(urlFgColor == null ? fgColor : urlFgColor),
					(urlBgColor == null ? bgColor : urlBgColor),
					url
				));
				// i => Ende der Adresse
				i = hrefEnd - 1;
				// start => Ende der Adresse
				start = hrefEnd;
				// Schleife nach der Adresse erneut starten
				continue;
			}//if url
			// Smileys suchen
			int imax = Math.min(i + cTA.getMaxSmileyCodeLength(), len);
			String sbuf = text.substring(i, imax);
			Image image;
			// Lnge immer um 1 verkrzen
			for (int j = sbuf.length(); j > 0; j--) {
				// String abschneiden
				String smax = sbuf.substring(0, j);
				// (lngstes) Smiley gefunden
				if ((image = cTA.getSmiley(smax)) != null) {
					// evtl Text vorher noch ausgeben
					if (start < i) {
						this.lineParts.addElement(new TextLinePart(
							cTA, text.substring(start, i), bold, italic,
							underline, fgColor, bgColor
						));
					}
					// LinePart erzeugen
					this.lineParts.addElement(new ImageLinePart(
						cTA, smax, image, fgColor, bgColor
					));
					i += j - 1;
					start = i + 1;
					break;
				}//if
			}//for (smiley)
			// Kontrollcodes suchen
			if (text.charAt(i) == ColorTextArea.CHR0) {
				// evtl Text vorher noch ausgeben
				if (start < i) {
					this.lineParts.addElement(new TextLinePart(
						cTA, text.substring(start, i), bold, italic,
						underline, fgColor, bgColor
					));
				}
				if (text.charAt(i + 1) == ColorTextArea.BOLD) {
					i += 1;
					bold = !bold;
				} else if (text.charAt(i + 1) == ColorTextArea.ITALIC) {
					i += 1;
					italic = !italic;
				} else if (text.charAt(i + 1) == ColorTextArea.UNDERLINE) {
					i += 1;
					underline = !underline;
				} else if (text.charAt(i + 1) == ColorTextArea.RESET) {
					i += 1;
					bold = false;
					italic = false;
					underline = false;
				} else if (text.charAt(i + 1) == ColorTextArea.FG_COLOR) {
					int index = text.indexOf(ColorTextArea.CHR0, i + 2);
					if (index >= 0) {
						fgColor = ColorTextArea.string2Color(
							text.substring(i + 2, index)
						);
						i = index;
					} else {
						i += 1;
					}
				} else if (text.charAt(i + 1) == ColorTextArea.BG_COLOR) {
					int index = text.indexOf(ColorTextArea.CHR0, i + 2);
					if (index >= 0) {
						bgColor = ColorTextArea.string2Color(
							text.substring(i + 2, index)
						);
						i = index;
					} else {
						i += 1;
					}
				} else if (text.charAt(i + 1)
				 == ColorTextArea.FG_COLOR_DEFAULT) {
					i += 1;
					fgColor = cTA.getForeground();
				} else if (text.charAt(i + 1)
				 == ColorTextArea.BG_COLOR_DEFAULT) {
					i += 1;
					bgColor = cTA.getBackground();
				}
				start = i + 1;
			// Tabulatoren-Spezialbehandlung
			} else if (text.charAt(i) == '\t') {
				// evtl Text vorher noch ausgeben
				if (start < i) {
					this.lineParts.addElement(new TextLinePart(
						cTA, text.substring(start, i), bold, italic,
						underline, fgColor, bgColor
					));
				}
				start = i;
				while (text.charAt(i) == '\t') {
					i++;
				}
				this.lineParts.addElement(new TabLinePart(
					cTA,  text.substring(start, i), fgColor, bgColor
				));
				start = i;
				i--;
			}
		}//for(int i = 0; i < len; i++)
		// wenn noch Text da, dann ausgeben
		if (start < len) {
			this.lineParts.addElement(new TextLinePart(
				cTA, text.substring(start, len), bold, italic,
				underline, fgColor, bgColor
			));
		}//if
	}//Line

	/**
	 * berechnet die Abmessungen dieses Objekts anhand der Gre des
	 * Elternobjekts
	 *
	 * @return die Hhe in Pixeln dieses Objekts
	 */
	int sizeCalc() {
		// aktuelle Position in eigenen Koordinatensystem
		int x = this.cTA.getInsets().left;
		// Zeilenhhenvektor lschen
		this.lineHeights.removeAllElements();
		this.lineHeights.addElement(new Integer(0));
		// Anzahl LineParts
		int partCount = this.lineParts.size();
		// alle LineParts durchgehen
		for (int i = 0; i < partCount; i++) {
			LinePart lp = (LinePart)this.lineParts.elementAt(i);
			x = lp.sizeCalc(x, this.lineHeights);
		}
		// Gesamthhe bestimmen
		this.height = 0;
		int lineCount = this.lineHeights.size();
		for (int i = 0; i < lineCount; i++) {
			this.height += ((Integer)this.lineHeights.elementAt(i)).intValue();
		}
		int width = this.cTA.getSize().width;
		if (width < 1) {
			width = 1;
		}
		/*buffer = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);*/
		this.buffer = this.cTA.createImage(width, this.height);
		this.bufferValid = false;
		return this.height;
	}

	/**
	 * zeichnet dieses Line-Objekt
	 *
	 * @param g die Grafikoberflche, auf die gezeichnet werden soll
	 * @param y oberste Zeile, ab der gezeichnet werden soll
	 */
	void paint(Graphics g, int y) {
		// gepuffertes Bild updaten?
		if (!this.bufferValid) {
			// Puffer da?
			if (this.buffer == null) {
				sizeCalc();
			}
			Graphics bufferGraphics = this.buffer.getGraphics();
			VersionController.getInstance().setAntialiasing(bufferGraphics, this.cTA.isAntialiasing());
			bufferGraphics.setColor(this.cTA.getBackground());
			bufferGraphics.fillRect(0, 0, this.buffer.getWidth(null), this.buffer.getHeight(null));
			// Anzahl LineParts
			int partCount = this.lineParts.size();
			// alle LineParts durchgehen
			for (int i = 0; i < partCount; i++) {
				LinePart lp = (LinePart)this.lineParts.elementAt(i);
				lp.paint(bufferGraphics, this.lineHeights);
			}
			this.bufferValid = true;
		}
		// in einem Rutsch zeichnen
		g.drawImage(this.buffer, 0, y, null);
	}

	/**
	 * gibt die Hhe dieses Objekts zurck
	 *
	 * @return die Hhe dieses Objekts
	 */
	int getHeight() {
		return this.height;
	}

	/**
	 * bestimmt den LinePart an einem gegebenen Punkt
	 *
	 * @param p der Punkt
	 * @return der LinePart an diesem Punkt oder null, wenn keiner da ist
	 */
	LinePart getLinePart(Point p) {
		// physische Zeile berechnen
		int lines = this.lineHeights.size();
		int y = 0;
		int line;
		for (line = -1; (line < lines - 1) && (p.y > y); line++) {
			y += ((Integer)this.lineHeights.elementAt(line + 1)).intValue();
		}
		// untendrunter
		if (line == lines) {
			return null;
		}
		// LinePart finden
		int size = this.lineParts.size();
		for (int i = 0; i < size; i++) {
			// LinePart & Daten holen
			LinePart lp = (LinePart)this.lineParts.elementAt(i);
			int startLine = lp.getStartLine();
			int endLine = lp.getEndLine();
			int startX = lp.getStartX();
			int endX = lp.getEndX();
			// Anfangs- und Endzeile vergleichen
			if ((startLine <= line) && (line <= endLine)) {
				// 1. Anfangszeile == Endzeile => startX und endX vergleichen
				if (startLine == endLine) {
					if ((startX <= p.x) && (p.x < endX)) {
						return lp;
					} else if (p.x < startX) { // zu weit
						if (i > 0) {
							return (LinePart)this.lineParts.elementAt(i - 1);
						}
						return null;
					}
				// 2. Anfangszeile eines mehrzeiligen LinePart => startX
				// vergleichen
				} else if (startLine == line) {
					if (startX <= p.x) {
						return lp;
					}
					// zu weit
					if (i > 0) {
						return (LinePart)this.lineParts.elementAt(i - 1);
					}
					return null;
				// 3. Endzeile eines mehrzeiligen LinePart
				} else if (endLine == line) {
					if (p.x < endX) {
						return lp;
					}
				// 4. mittlere Zeile eines mehrzeiligen LinePart
				} else {
					return lp;
				}
			} else if (line < startLine) { // zu weit
				if (i > 0) {
					return (LinePart)this.lineParts.elementAt(i - 1);
				}
				return null;
			}
		}
		return null;
	}

	/**
	 * berechnet zu einem gegebenen Punkt den LinePart und dort die Position in
	 * dessen String
	 *
	 * @param p der Punkt
	 * @return die Position im String oder -1, wenn kein LinePart vorhanden
	 */
	int getStringPos(Point p) {
		// LinePart bestimmen
		LinePart lp = getLinePart(p);
		if (lp == null) {
			// vor 1. Zeile
			if (p.y <= 0) {
				return ColorTextArea.ABOVE;
			}
			// nach letzter Zeile
			if (p.y > this.height){
				return ColorTextArea.BELOW;
			}
			int lineIndent = this.cTA.getInsets().left;
			// nicht die 1. Zeile
			if (p.y > ((Integer)this.lineHeights.elementAt(0)).intValue()) {
				lineIndent += this.cTA.getSubsequentIndent();
			}
			if (p.x <= lineIndent) {
				return ColorTextArea.ABOVE;
			}
			return ColorTextArea.BELOW;
		}
		// physische Zeile berechnen
		int lines = this.lineHeights.size();
		int y = 0;
		int line;
		for (line = -1; (line < lines - 1) && (p.y > y); line++) {
			y += ((Integer)this.lineHeights.elementAt(line + 1)).intValue();
		}
		// Position ermitteln
		int pos = lp.getStringPos(new Point(p.x, line));
		// Fehlerwerte direkt
		if ((pos < 0) || (pos > this.text.length())) {
			return pos;
		}
		// und zurckrechnen
		LinePart _lp = (LinePart)this.lineParts.elementAt(0);
		int i = 0;
		while (_lp != lp) {
			pos += _lp.getText().length();
			_lp = (LinePart)this.lineParts.elementAt(++i);
		}
		return pos;
	}

	/**
	 * bestimmt die URL des LineParts an einem gegebenen Punkt
	 *
	 * @param p der Punkt
	 * @return die URL oder null, wenn kein LinePart oder keine URL vorhanden
	 */
	String getURL(Point p) {
		LinePart lp = getLinePart(p);
		if (lp == null) {
			return null;
		}
		return lp.getURL();
	}

	/**
	 * gibt den Text dieses Line-Objektes als String zurck
	 *
	 * @return der Text (ohne Formatierungscodes)
	 */
	String getText() {
		return this.text;
	}

	/**
	 * setzt Start und Ende der Markierung
	 *
	 * @param start die Startposition
	 * @param end die Endposition
	 * @return true, wenn sich die Markierung gendert hat
	 */
	boolean setSelection(int start, int end) {
		boolean changed = false;
		// Textlnge zwischenspeichern
		int len = this.lineParts.size();
		// Index auerhalb des gltigen Bereichs => Markierung entfernen
		if ((start == ColorTextArea.INVALID)
		 || (end == ColorTextArea.INVALID)) {
			for (int i = 0; i < len; i++) {
				LinePart lp = (LinePart)this.lineParts.elementAt(i);
				changed = changed | lp.setSelection(ColorTextArea.INVALID,
					ColorTextArea.INVALID
				);
			}
			this.bufferValid = this.bufferValid && !changed;
			return changed;
		}
		// Aktuelle Position
		int pos = 0;
		// alle LineParts durchgehen und Markierung setzen
		for (int i = 0; i < len; i++) {
			LinePart lp = (LinePart)this.lineParts.elementAt(i);
			int lpLen = lp.getText().length();
			// komplett unmarkiert
			if ((pos + lpLen <= start) || (pos >= end)) {
				changed = changed | lp.setSelection(ColorTextArea.INVALID,
					ColorTextArea.INVALID
				);
			// komplett markiert
			} else if ((pos >= start) && (pos + lpLen <= end)) {
				changed = changed | lp.setSelection(0, lpLen);
			// Markierung komplett innerhalb
			} else if ((pos <= start) && (pos + lpLen >= end)) {
				changed = changed | lp.setSelection(start - pos, end - pos);
			// nur am  Anfang markiert
			} else if (pos >= start) {
				changed = changed | lp.setSelection(0, end - pos);
			// nur am Ende markiert
			} else {
				changed = changed | lp.setSelection(start - pos, lpLen);
			}
			pos += lp.getText().length();
		}
		this.bufferValid = this.bufferValid && !changed;
		return changed;
	}

	/**
	 * suche das Ende einer URL in einem String
	 *
	 * @param s der zu durchsuchende String
	 * @param start Position in s, ab der gesucht wird
	 * @return das Ende der URL
	 */
	private int findEndOfHref(String s, int start) {
		//Lnge von s zwischenspeichern
		int len = s.length();
		// alle Zeichen durchgehen
		for (int i = start; i < len; i++) {
			char c = s.charAt(i);
			// Buchstaben, Zahlen: Weiter
			if (Character.isLetterOrDigit(c)) {
				continue;
			}
			// Sonderzeichen auswerten
			switch (c) {
				case '.' :
				case '?' :
				case '&' :
				case '=' :
				case '-' :
				case '_' :
				case '/' :
				case '#' :
				case ':' :
				case '~' :
				case '%' :
				case '+' :
					continue;
				default :
					return i;
			}
		}
		// URL geht bis zum Ende von s
		return len;
	}

	/**
	 * stellt fest, ob s eine URL ist
	 *
	 * @param s der zu testende String
	 * @return true, wenn s eine URL darstellt, false sonst
	 */
	private boolean isHref(String s) {
/*		// 8 Zeichen Minimum (www.x.ar)
		if (s.length() < 8) {
			return false;
		}*/
		String[] urlPrefix = this.cTA.getURLPrefixes();
		// zuerst nur das erste Zeichen testen
		char c = s.charAt(0);
		c = Character.toUpperCase(c);
		boolean firstPassFailed = true;
		for (int i = 0; i < urlPrefix.length; i++) {
			if (c == urlPrefix[i].charAt(0)) {
				firstPassFailed = false;
				break;
			}
		}
		// keine bereinstimmung?
		if (firstPassFailed) {
			return false;
		}
		// dann nach genauer bereinstimmung suchen
		String up = s.toUpperCase();
		for (int i = 0; i < urlPrefix.length; i++) {
			// nach dem URL-Prfix sollte noch was kommen
			if (up.startsWith(urlPrefix[i]) && Character.isLetterOrDigit(up.charAt(urlPrefix[i].length()))) {
				return true;
			}
		}
		// keine bereinstimmung
		return false;
	}

	/**
	 * entfernt alle Kontrollcodes aus einem String
	 *
	 * @param s der zu bereinigende String
	 * @return der bereinigte String
	 */
	private String strip(String s) {
		int len = s.length();
		StringBuffer temp = new StringBuffer(len);
		for(int i = 0; i < len; i++) {
			char ch = s.charAt(i);
			// Kontrollcodes suchen
			if (ch == ColorTextArea.CHR0) {
				switch(s.charAt(i + 1)) {
					case ColorTextArea.BOLD:
					case ColorTextArea.ITALIC:
					case ColorTextArea.UNDERLINE:
					case ColorTextArea.RESET:
					case ColorTextArea.FG_COLOR_DEFAULT:
					case ColorTextArea.BG_COLOR_DEFAULT: {
						i++;
						break;
					}
					case ColorTextArea.FG_COLOR:
					case ColorTextArea.BG_COLOR: {
						int index = s.indexOf(ColorTextArea.CHR0, i + 2);
						if (index >= 0) {
							i = index;
						} else {
							i++;
						}
						break;
					}
				}
			} else {
				temp.append(ch);
			}
		}//for(int i = 0; i < len; i++)
		return temp.toString();
	}

	/**
	 * ersetzt String.split, weil das vor java 1.4 noch nicht vorhanden ist
	 *
	 * @param source der zu zerteilende String
	 * @param at der Teilstring, an dem gesplittet wird
	 * @return die Teile des Strings
	 */
	public static String[] split (String source, String at) {
		String temp = source;
		boolean run = true;
		Vector list = new Vector ();
		//solange noch was zu tun ist
		while (run) {
			//suche nchstes Vorkommen
			int pos = temp.indexOf (at);
			//nicht gefunden => Abbruch, letztes Element noch speichern
			if (pos == -1) {
				run = false;
				list.addElement (temp);
			//am Anfang gefunden => Leerstring einfgen
			} else if (pos == 0) {
				list.addElement (""); //$NON-NLS-1$
				temp = temp.substring (at.length ());
			//gefunden => String auseinanderklamsern
			} else {
				list.addElement (temp.substring (0, pos));
				temp = temp.substring (pos + at.length ());
			}//if
		}//while
		//in Array umwandeln und zurckgeben
		String[] list2array = new String[list.size ()];
		list.copyInto (list2array);
		return list2array;
	}//split (...)

	/**
	 * verbindet den Inhalt eines Stringarrays mit glue
	 *
	 * @param source das zu verbindende Array
	 * @param glue der zwischen die Arrayelemente eungefgte String
	 * @return der zusammengefgte String
	 */
	public static String join (String[] source, String glue) {
		if ((source == null) || (source.length == 0)) {
			return ""; //$NON-NLS-1$
		}
		if (glue == null) {
			glue = ""; //$NON-NLS-1$
		}
		StringBuffer temp = new StringBuffer (source[0]);
		for (int i = 1; i < source.length; i++) {
			temp.append (glue).append (source[i]);
		}
		return temp.toString ();
	}
	
	/**
	 * Bestimmt den neuen Status eines Flags anhand des aktuellen Status und einer
	 * nderungsoption.
	 * 
	 * @param desired die nderungsoption.
	 * @param actual der aktuelle Wert.
	 * @return der neue Wert.
	 */
	private boolean decode(int desired, boolean actual) {
		if (desired == ColorTextArea.TRUE) {
			return true;
		} else if (desired == ColorTextArea.FALSE) {
			return false;
		} else if (desired == ColorTextArea.NO_CHANGE) {
			return actual;
		} else if (desired == ColorTextArea.INVERT) {
			return !actual;
		}
		throw new IllegalArgumentException();
	}
}