/*
 * Created on 14.06.2004
 */
package ec4.tools.ui;

import java.awt.Graphics;
import java.awt.Point;
import java.util.Vector;

/**
 * stellt einen Teil einer logischen Zeile dar, der eine einheitliche
 * Formatierung hat und sich ber mehrere physische Zeilen erstrecken kann
 * 
 * @author AM
 */
interface LinePart {
	
	/**
	 * gibt den Text dieses LineParts zurck
	 * 
	 * @return der Text
	 */
	public String getText();
	
	/**
	 * gibt die URL dieses LineParts zurck
	 * 
	 * @return die URL oder null, wenn keine vorhanden
	 */
	public String getURL();
	
	/**
	 * zeichnet diesen LinePart
	 * 
	 * @param g die Grafikoberflche zum Zeichnen
	 * @param lineHeights
	 */
	public void paint(Graphics g, Vector lineHeights);
	
	/**
	 * berechnet die Abmessungen dieses LineParts
	 * 
	 * @param x Anfangs-x-Koordinate
	 * @param lineHeights bisherige Zeilenhhen als Vektor von Integers
	 * @return die End-x-Koordinate
	 */
	public int sizeCalc(int x, Vector lineHeights);

	/**
	 * berechnet die Position des gegebenen Punkts im Text dieses Objekts
	 * 
	 * @param p der Punkt (im lokalen Koordinatensystem: x als Pixel, y
	 *          als physische Zeile)
	 * @return die Position im Text oder -1, wenn p auerhalb dieses Objekts ist
	 */
	public int getStringPos(Point p);
	
	/**
	 * gibt die x-Koordinate am Anfang dieses LineParts zurck
	 * 
	 * @return die x-Koordinate am Anfang
	 */
	public int getStartX();
	
	/**
	 * gibt die x-Koordinate am Ende dieses LineParts zurck
	 * 
	 * @return die x-Koordinate am Ende
	 */
	public int getEndX();

	/**
	 * gibt die Startzeile dieses Objekts in seiner Line zurck
	 * 
	 * @return die Startzeile
	 */
	public int getStartLine();

	/**
	 * gibt die Endzeile dieses Objekts in seiner Line zurck
	 * 
	 * @return die Endzeile
	 */
	public int getEndLine();

	/**
	 * setzt start und Ende der Markierung
	 * 
	 * @param start die Startposition
	 * @param end die Endposition
	 * @return true, wenn sich die Markierung gendert hat
	 */
	public boolean setSelection(int start, int end);
	
}
