/*
 * Created on 29.06.2004
 */
package ec4.tools.ui;

import java.awt.AWTError;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.LayoutManager;
import java.awt.Panel;
import java.awt.Scrollbar;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;

import ec4.tools.version.MouseWheelListener;
import ec4.tools.version.VersionController;

/**
 * ein einfaches vertikales Scrollfenster
 * 
 * @author AM
 */
public class SimpleScrollPane extends Panel implements AdjustmentListener,
	MouseWheelListener, ComponentListener {
	
	/**
	 * die Scrollbar, die zum scrollen benutzt wird
	 */
	private Scrollbar sb;
	
	/**
	 * die Komponente, die gescrollt wird
	 */
	private Component target;
	
	/**
	 * true, wenn der Superkonstruktor ausgefhrt wurde (wegen setLayout)
	 */
	private boolean initDone;
	
	/**
	 * erstellt eine neue ScrollPane
	 */
	public SimpleScrollPane() {
		this(null);
	}
	
	/**
	 * erstellt eine neue ScrollPane
	 * 
	 * @param target die zu scrollende Komponente (implementiert optional
	 *        Scrollable)
	 */
	public SimpleScrollPane(Component target) {
		super();
		this.initDone = true;
		this.target = target;
		super.setLayout(null);
		addComponentListener(this);
		VersionController.getInstance().buildMouseWheelListener(this, this);
		this.sb = new Scrollbar(Scrollbar.VERTICAL);
		this.sb.setMinimum(0);
		this.sb.setMaximum(1); // 0 mag sun-java 1.5 net
		this.sb.setVisibleAmount(1);
		this.sb.addAdjustmentListener(this);
		super.add(this.sb);
		if (target != null) {
			add(target);
		}
		doLayout();
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.awt.Container#add(java.awt.Component, int)
	 */
	public Component add(Component comp, int index) {
		return add(comp);
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.awt.Container#add(java.awt.Component, java.lang.Object, int)
	 */
	public void add(Component comp, Object constraints, int index) {
		add(comp);
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.awt.Container#add(java.awt.Component, java.lang.Object)
	 */
	public void add(Component comp, Object constraints) {
		add(comp);
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.awt.Container#add(java.awt.Component)
	 */
	public Component add(Component comp) {
		if (this.target != null) {
			remove(this.target);
		}
		this.target = comp;
		if (comp != null) {
			super.add(comp);
		}
		return comp;
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.awt.Container#add(java.lang.String, java.awt.Component)
	 */
	public Component add(String name, Component comp) {
		return add(comp);
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.awt.Container#remove(java.awt.Component)
	 */
	public void remove(Component comp) {
		if (comp == this.target) {
			super.remove(comp);
			this.target = null;
		}
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.awt.Container#remove(int)
	 */
	public void remove(int index) {
		remove(getComponent(index));
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.awt.Container#removeAll()
	 */
	public void removeAll() {
		super.removeAll();
		add(this.sb);
		this.target = null;
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.awt.Component#doLayout()
	 */
	public void doLayout() {
		//super.layout();
		Dimension sbSize = this.sb.getPreferredSize();
		Dimension mySize = getSize();
		this.sb.setBounds(mySize.width - sbSize.width, 0, sbSize.width, mySize.height);
		if (this.target != null) {
			int h = this.target.getPreferredSize().height;
			if (h < mySize.height) {
				h = mySize.height;
			}
			if (h == 0) {
				h = 1; // h = 0 mag sun-java 1.5 net
			}
			this.sb.setMaximum(h);
			this.sb.setVisibleAmount(mySize.height);
			if (this.target instanceof Scrollable) {
				this.target.setBounds(0, 0, mySize.width - sbSize.width, mySize.height);
				((Scrollable)this.target).scrollTo(this.sb.getValue());
			} else {
				this.target.setBounds(0, -this.sb.getValue(), mySize.width - sbSize.width, h);
			}
		} else {
			this.sb.setMaximum(1); // 0 mag sun-java 1.5 net
			this.sb.setVisibleAmount(1);
		}
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.awt.Container#setLayout(java.awt.LayoutManager)
	 */
	public void setLayout(LayoutManager mgr) {
		if (this.initDone) throw new AWTError("SimpleScrollPane controls layout"); //$NON-NLS-1$
	}
	
	/* 
	 * (non-Javadoc)
	 * @see java.awt.event.AdjustmentListener#adjustmentValueChanged(java.awt.event.AdjustmentEvent)
	 */
	public void adjustmentValueChanged(AdjustmentEvent e) {
		if (this.target != null) {
			if (this.target instanceof Scrollable) {
				((Scrollable)this.target).scrollTo(this.sb.getValue());
			} else {
				this.target.setLocation(0, -this.sb.getValue());
			}
		}
	}
	
	/* 
	 * (non-Javadoc)
	 * @see java.awt.event.ComponentListener#componentHidden(java.awt.event.ComponentEvent)
	 */
	public void componentHidden(ComponentEvent e) {
		// nix
	}
	
	/* 
	 * (non-Javadoc)
	 * @see java.awt.event.ComponentListener#componentMoved(java.awt.event.ComponentEvent)
	 */
	public void componentMoved(ComponentEvent e) {
		// nix
	}
	
	/* 
	 * (non-Javadoc)
	 * @see java.awt.event.ComponentListener#componentResized(java.awt.event.ComponentEvent)
	 */
	public void componentResized(ComponentEvent e) {
		// TODO Position anpassen: oben, mitte, unten
		doLayout();
	}
	
	/* 
	 * (non-Javadoc)
	 * @see java.awt.event.ComponentListener#componentShown(java.awt.event.ComponentEvent)
	 */
	public void componentShown(ComponentEvent e) {
		// nix
	}
	
	/**
	 * fragt die grte Scrollposition ab
	 * 
	 * @return die grte SCrollposition
	 */
	public int getMaximumScrollPosition() {
		return this.sb.getMaximum();
	}
	
	/**
	 * fragt die minimale Scrollposition ab
	 * 
	 * @return die minimale SCrollposition
	 */
	public int getMinimumScrollPosition() {
		return this.sb.getMinimum();
	}
	
	/**
	 * fragt die Scrollposition ab
	 * 
	 * @return die Scrollposition
	 */
	public int getScrollPosition() {
		return this.sb.getValue();
	}
	
	/**
	 * setzt die Scrollposition
	 * 
	 * @param v die Scrollposition
	 */
	public void setScrollPosition(int v) {
		this.sb.setValue(v);
		if (this.target != null) {
			if (this.target instanceof Scrollable) {
				((Scrollable)this.target).scrollTo(this.sb.getValue());
			} else {
				this.target.setLocation(0, -v);
			}
		}
	}
	
	/**
	 * fragt das BlockIncrement der Scrollbar ab
	 * 
	 * @return das BlockIncrement
	 */
	public int getBlockIncrement() {
		return this.sb.getBlockIncrement();
	}
	
	/**
	 * fragt das UnitIncrement der Scrollbar ab
	 * 
	 * @return das UnitIncrement
	 */
	public int getUnitIncrement() {
		return this.sb.getUnitIncrement();
	}
	
	/**
	 * setzt das BlockIncrement der Scrollbar
	 * 
	 * @param b das neue BlockIncrement
	 */
	public void setBlockIncrement(int b) {
		this.sb.setBlockIncrement(b);
	}
	
	/**
	 * setzt das UnitIncrement der Scrollbar
	 * 
	 * @param u das neue UnitIncrement
	 */
	public void setUnitIncrement(int u) {
		this.sb.setUnitIncrement(u);
	}
	
	/**
	 * fragt ab, wieviel von der Komponente sichtbar ist
	 * 
	 * @return die sichtbare Gre
	 */
	public int getVisibleAmount() {
		return this.sb.getVisibleAmount();
	}
	
	/* 
	 * (non-Javadoc)
	 * @see java.awt.event.MouseWheelListener#mouseWheelMoved(java.awt.event.MouseWheelEvent)
	 */
	public void mouseWheelMoved(int wheelRotation) {
		int pos = this.sb.getValue() + this.sb.getBlockIncrement() * wheelRotation;
		if (pos < 0) {
			pos = 0;
		}
		int temp = this.sb.getMaximum() - this.sb.getVisibleAmount();
		if (pos > temp) {
			pos = temp;
		}
		this.sb.setValue(pos);
		if (this.target != null) {
			if (this.target instanceof Scrollable) {
				((Scrollable)this.target).scrollTo(this.sb.getValue());
			} else {
				this.target.setLocation(0, -pos);
			}
		}
	}
}
