package ec4.tools.ui;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.LayoutManager;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

/**
 * ein Splitter teilt den verfgbaren PLatz zwischen 2 Komponenten auf
 * 
 * @author Andreas Mann
 */
public class Splitter extends Container implements MouseListener, MouseMotionListener {
	
	/**
	 * Konstante fr einen horizontalen Splitter
	 */
	public static final int HORIZONTAL_SPLIT = 0;
	
	/**
	 * Konstante fr einen vertikalen Splitter
	 */
	public static final int VERTICAL_SPLIT = 1;
	
	/**
	 * Konstante fr die erste (obere oder linke) Komponente dieses Splitters
	 */
	public static final String FIRST = "First"; //$NON-NLS-1$
	
	/**
	 * Konstante fr die zweite (untere oder rechte) Komponente dieses Splitters
	 */
	public static final String SECOND = "Second"; //$NON-NLS-1$
	
	/**
	 * Konstante fr die obere Komponente
	 */
	public static final String TOP = FIRST;
	
	/**
	 * Konstante fr die linke Kompoonente
	 */
	public static final String LEFT = FIRST;
	
	/**
	 * Konstante fr die untere Komponente
	 */
	public static final String BOTTOM = SECOND;
	
	/**
	 * Konstante fr die rechte Komponente
	 */
	public static final String RIGHT = SECOND;
	
	/**
	 * Konstante fr den Splitter
	 */
	private static final String SPLITTER = "Splitter"; //$NON-NLS-1$
	
	/**
	 * Konstante fr den bewegte Splitter
	 */
	private static final String MOVING_SPLITTER = "MovingSplitter"; //$NON-NLS-1$
	
	/**
	 * die Hlfte der Splitterbreite
	 */
	private static final int SPLITTER_SIZE = 3;
	
	/**
	 * die erste Komponente
	 */
	private Component first;
	
	/**
	 * die zweite Komponente
	 */
	private Component second;
	
	/**
	 * der Splitter
	 */
	private Component splitter;
	
	/**
	 * der bewegte Splitter
	 */
	private Component movingSplitter;
	
	/**
	 * die Orientierung des Splitters
	 */
	int orientation;
	
	/**
	 * die Position des Splitters
	 */
	private int splitterPos;
	
	/**
	 * gibt an, wie zustzlicher Platz beim Vergrern verteilt werden soll
	 */
	private double splitterResizeWeight;
	
	/**
	 * die Start-X-Position fr das Verschieben
	 */
	private int dragX;
	
	/**
	 * die Start-Y-Position fr das Verschieben
	 */
	private int dragY;
	
	/**
	 * die Regelung fr den Minimalplatz
	 * -1 -> Minimalgren der Komponenten
	 * 0  -> aus
	 * >0 -> diese Minimalgre
	 */
	private int constraint;
	
	/**
	 * erstellt einen neuen horizontalen Splitter
	 */
	public Splitter() {
		this(HORIZONTAL_SPLIT, null, null, -1);
	}
	
	/**
	 * erstellt einen Splitter
	 * 
	 * @param orientation die Orientierung des Splitters
	 */
	public Splitter(int orientation) {
		this(orientation, null, null, -1);
	}
	
	/**
	 * erstellt einen neuen horizontalen Splitter
	 * 
	 * @param first die erste Komponente
	 * @param second die zweite Komponente
	 */
	public Splitter(Component first, Component second) {
		this(HORIZONTAL_SPLIT, first, second, -1);
	}
	/**
	 * erstellt einen Splitter
	 * 
	 * @param orientation die Orientierung des Splitters
	 * @param first die erste Komponente
	 * @param second die zweite Komponente
	 */
	public Splitter(int orientation, Component first, Component second) {
		this(orientation, first, second, -1);
	}
	
	/**
	 * erstellt einen Splitter
	 * 
	 * @param orientation die Orientierung des Splitters
	 * @param first die erste Komponente
	 * @param second die zweite Komponente
	 * @param constraint die Regelung fr den Minimalplatz
	 */
	public Splitter(int orientation, Component first, Component second,
	 int constraint) {
		super();
		// initalisieren
		setLayout(null);
		setFirstComponent(first);
		setSecondComponent(second);
		// Parameter speichern
		this.orientation = orientation;
		this.constraint = constraint;
		this.splitterResizeWeight = 0;
		if (this.orientation == HORIZONTAL_SPLIT) {
			this.splitterPos = this.first != null ? this.first.getMinimumSize().width : 0;
		} else {
			this.splitterPos = this.first != null ? this.first.getMinimumSize().height : 0;
		}
//*
		// Splitter bauen
		this.splitter = new Component() {
			// Zeichnen
			public void paint(Graphics g) {
				// Gre ermitteln
				Dimension d = getSize();
				int w = d.width - 1;
				int h = d.height - 1;
				g.setColor(new Color(255, 255, 255));
				g.drawLine(0, 0, w - 1, 0);
				g.drawLine(0, 0, 0, h - 1);
				g.setColor(new Color(223, 223, 223));
				g.drawLine(1, 1, w - 2, 1);
				g.drawLine(1, 1, 1, h - 2);
				g.setColor(new Color(191, 191, 191));
				g.fillRect(2, 2, w - 3, h - 3);
				g.setColor(new Color(127, 127, 127));
				g.drawLine(w - 1, 1, w - 1, h - 1);
				g.drawLine(1, h - 1, w - 1, h - 1);
				g.setColor(new Color(0, 0, 0));
				g.drawLine(w, 0, w, h);
				g.drawLine(0, h, w, h);
			}
		};
/*/
		this.splitter = new Button();
//*/
		if (this.orientation == HORIZONTAL_SPLIT) {
			this.splitter.setCursor(new Cursor(Cursor.E_RESIZE_CURSOR));
		} else {
			this.splitter.setCursor(new Cursor(Cursor.N_RESIZE_CURSOR));
		}
		this.splitter.addMouseListener(this);
		this.splitter.addMouseMotionListener(this);
		add(this.splitter, SPLITTER);
		// bewegter Splitter
		this.movingSplitter = new Canvas() {
			public void paint(Graphics g) {
				if (Splitter.this.orientation == HORIZONTAL_SPLIT) {
					int w = getSize().height;
					for (int i = 0; i < w; i+=10) {
						g.setColor(
							(i / 10) % 2 == 0 ? Color.white : Color.black
						);
						g.fillRect(0, i, 10, 10);
					}
				} else {
					int h = getSize().width;
					for (int i = 0; i < h; i+=10) {
						g.setColor(
							(i / 10) % 2 == 0 ? Color.white : Color.black
						);
						g.fillRect(i, 0, 10, 10);
					}
				}
			}
		};
		if (this.orientation == HORIZONTAL_SPLIT) {
			this.movingSplitter.setCursor(new Cursor(Cursor.E_RESIZE_CURSOR));
		} else {
			this.movingSplitter.setCursor(new Cursor(Cursor.N_RESIZE_CURSOR));
		}
	}
	
	/**
	 * setzt die erste Komponente
	 * 
	 * @param first die neue erste Komponente
	 */
	public void setFirstComponent(Component first) {
		if (first == null) {
			if (this.first != null) {
				remove(this.first);
			}
		} else {
			add(first, FIRST);
		}
	}
	
	/**
	 * setzt die zweite Komponente
	 * 
	 * @param second die neue zweite Komponente
	 */
	public void setSecondComponent(Component second) {
		if (second == null) {
			if (this.second != null) {
				remove(this.second);
			}
		} else {
			add(second, SECOND);
		}
	}
	
	/**
	 * fragt die erste Komponente ab
	 * 
	 * @return die erste Komponente
	 */
	public Component getFirstComponent() {
		return this.first;
	}
	
	/**
	 * fragt die zweite Komponente ab
	 * 
	 * @return die zweite Komponente
	 */
	public Component getSecondComponent() {
		return this.second;
	}
	
/*
	/* 
	 * (non-Javadoc)
	 * @see java.awt.Container#add(java.awt.Component)
	 * /
	public Component add(Component comp) {
		this.first = comp;
		super.add(comp, FIRST);
		return comp;
	}
	
	/* 
	 * (non-Javadoc)
	 * @see java.awt.Container#add(java.lang.String, java.awt.Component)
	 * /
	public Component add(String name, Component comp) {
		if (name.equals(FIRST)) {
			this.first = comp;
		} else if (name.equals(SECOND)) {
			this.second = comp;
		} else {
			throw new IllegalArgumentException(name + " is an illegal constraint"); //$NON-NLS-1$
		}
		super.add(comp, name);
		return comp;
	}
	
	/* 
	 * (non-Javadoc)
	 * @see java.awt.Container#add(java.awt.Component, int)
	 * /
	public Component add(Component comp, int index) {
		if (index == 1) {
			add(comp, FIRST);
		} else if (index == 2) {
			add(comp, SECOND);
		} else {
			throw new IllegalArgumentException("index has to be 1 or 2"); //$NON-NLS-1$
		}
		return comp;
	}
	
	/* 
	 * (non-Javadoc)
	 * @see java.awt.Container#add(java.awt.Component, java.lang.Object)
	 * /
	public void add(Component comp, Object constraints) {
		if (constraints instanceof String) {
			add((String)constraints, comp);
		} else {
			throw new IllegalArgumentException("constraints has to be an instance of String"); //$NON-NLS-1$
		}
	}
	
	/* 
	 * (non-Javadoc)
	 * @see java.awt.Container#add(java.awt.Component, java.lang.Object, int)
	 * /
	public void add(Component comp, Object constraints, int index) {
		add(comp, constraints);
	}
/*/
	/* (non-Javadoc)
	 * @see java.awt.Container#addImpl(java.awt.Component, java.lang.Object, int)
	 */
	protected void addImpl(Component comp, Object constraints, int index) {
		if (comp == null) {
			throw new NullPointerException();
		}
		if (constraints == null) {
			if (index < 0) {
				this.first = comp;
				super.addImpl(comp, FIRST, -1);
			} else if (index == 1){
				this.first = comp;
				super.addImpl(comp, FIRST, -1);
			} else if (index == 2) {
				this.second = comp;
				super.addImpl(comp, SECOND, -1);
			} else {
				throw new IllegalArgumentException("index has to be 1 or 2"); //$NON-NLS-1$
			}
		} else if (!(constraints instanceof String)) {
			throw new IllegalArgumentException("constraints has to be an instance of String"); //$NON-NLS-1$
		} else if (constraints.equals(FIRST)) {
			this.first = comp;
			super.addImpl(comp, FIRST, -1);
		} else if (constraints.equals(SECOND)) {
			this.second = comp;
			super.addImpl(comp, SECOND, -1);
		} else if (constraints.equals(SPLITTER)) {
			super.addImpl(comp, SPLITTER, 0);
		} else if (constraints.equals(MOVING_SPLITTER)) {
			super.addImpl(comp, MOVING_SPLITTER, 0);
		} else {
			throw new IllegalArgumentException(((String)constraints) + " is an illegal constraint"); //$NON-NLS-1$
		}
	}
//*/
	
	/* 
	 * (non-Javadoc)
	 * @see java.awt.Container#remove(int)
	 */
	public void remove(int index) {
		if (getComponent(index) == this.first) {
			this.first = null;
		} else if (getComponent(index) == this.second) {
			this.second = null;
		}
		super.remove(index);
	}
	
	/* 
	 * (non-Javadoc)
	 * @see java.awt.Container#remove(java.awt.Component)
	 */
	public void remove(Component comp) {
		if (comp == this.first) {
			this.first = null;
		} else if (comp == this.second) {
			this.second = null;
		}
		super.remove(comp);
	}
	
	/* 
	 * (non-Javadoc)
	 * @see java.awt.Container#removeAll()
	 */
	public void removeAll() {
		boolean moving = false;
		if (isAncestorOf(this.movingSplitter)) {
			moving = true;
		}
		this.first = null;
		this.second = null;
		super.removeAll();
		add(this.splitter, SPLITTER);
		if (moving) {
			add(this.movingSplitter, MOVING_SPLITTER);
		}
	}
	
	/* 
	 * (non-Javadoc)
	 * @see java.awt.Container#setLayout(java.awt.LayoutManager)
	 */
	public void setLayout(LayoutManager mgr) {
		// ignorieren
	}
	
	/* 
	 * (non-Javadoc)
	 * @see java.awt.Component#doLayout()
	 */
	public void doLayout() {
		super.doLayout();
		Dimension mySize = getSize();
		int w = mySize.width;
		int h = mySize.height;
		if (this.orientation == VERTICAL_SPLIT) {
			if (this.first != null) {
				this.first.setBounds(0, 0, w,
					this.splitterPos - SPLITTER_SIZE
				);
			}
			this.splitter.setBounds(0, this.splitterPos - SPLITTER_SIZE, w,
				SPLITTER_SIZE * 2
			);
			if (this.second != null) {
				this.second.setBounds(0, this.splitterPos + SPLITTER_SIZE, w,
					h - this.splitterPos - SPLITTER_SIZE
				);
			}
		} else {
			if (this.first != null) {
				this.first.setBounds(0, 0, this.splitterPos - SPLITTER_SIZE, h);
			}
			this.splitter.setBounds(this.splitterPos - SPLITTER_SIZE, 0,
				SPLITTER_SIZE * 2, h
			);
			if (this.second != null) {
				this.second.setBounds(this.splitterPos + SPLITTER_SIZE, 0,
					w - this.splitterPos - SPLITTER_SIZE, h
				);
			}
		}
		if (this.first != null) {
			this.first.doLayout();
		}
		if (this.second != null) {
			this.second.doLayout();
		}
		
	}
	
	/**
	 * setzt die Position des Splitters
	 * 
	 * @param splitterPos die neue Splitterposition
	 */
	public void setSplitterPos(int splitterPos) {
		setSplitterPos(splitterPos, true);
	}
	
	/**
	 * setzt die Position des Splitters
	 * 
	 * @param splitterPos die neue Splitterposition
	 * @param constraint true, wenn die Regel fr den Minimalplatz beachtet
	 *        werden soll
	 */
	public void setSplitterPos(int splitterPos, boolean constraint) {
		this.splitterPos = splitterPos;
		if (constraint) {
			int min;
			int max;
			int size;
			Dimension mySize = getSize();
			int w = mySize.width;
			int h = mySize.height;
			if (this.constraint < 0) {
				if (this.orientation == HORIZONTAL_SPLIT) {
					min = (this.first != null ? this.first.getMinimumSize().width : 0) + SPLITTER_SIZE;
					max = (this.second != null ? this.second.getMinimumSize().width : 0) - SPLITTER_SIZE;
					size = w;
				} else {
					min = (this.first != null ? this.first.getMinimumSize().height : 0) + SPLITTER_SIZE;
					max = (this.second != null ? this.second.getMinimumSize().height : 0) - SPLITTER_SIZE;
					size = h;
				}
			} else {
				min = this.constraint + SPLITTER_SIZE;
				max = this.constraint - SPLITTER_SIZE;
				if (this.orientation == HORIZONTAL_SPLIT) {
					size = w;
				} else {
					size = h;
				}
			}
			if (min + max + 2 * SPLITTER_SIZE < size) {
				if (splitterPos < (min / 2)) {
					splitterPos = SPLITTER_SIZE;
				} else if (splitterPos < min) {
					splitterPos = min;
				} else if (splitterPos > (size - (max / 2))) {
					splitterPos = size - SPLITTER_SIZE;
				} else if (splitterPos > (size - max)) {
					splitterPos = size - max;
				}
			} else if (splitterPos < 0) {
				splitterPos = SPLITTER_SIZE;
			} else if (splitterPos > size) {
				if (size > 2 * SPLITTER_SIZE) {
					splitterPos = size - SPLITTER_SIZE;
				} else {
					splitterPos = SPLITTER_SIZE;
				}
			}
		}
		doLayout();
	}
	
	/**
	 * fragt die Splitterposition ab
	 * 
	 * @return die Splittersosition
	 */
	public int getSplitterPos() {
		
		return this.splitterPos;
		
	}
	
	/**
	 * legt die Verteilung zustzlichen Platzes beim Vergrern des Splitters
	 * fest
	 * 
	 * @param splitterResizeWeight die neue Verteilung
	 *        (0.0 - alles links/oben bis 1.0 - alles rechts/unten)
	 * @throws IllegalArgumentException wenn der Parameter auerhalb des
	 *         gltigen Bereiches ist
	 */
	public void setSplitterResizeWeight(double splitterResizeWeight) {
		if ((splitterResizeWeight < 0.0) || (splitterResizeWeight > 1.0)) {
			throw (new IllegalArgumentException (
			  "splitterResizeWeight has to be between 0.0 and 1.0" //$NON-NLS-1$
			));
		}
		this.splitterResizeWeight = splitterResizeWeight;
	}
	
	/**
	 * fragt die Verteilung zustzlichen PLatzes ab
	 * 
	 * @return die Verteilung
	 */
	public double getSplitterResizeWeight() {
		return this.splitterResizeWeight;
	}
	
	/* 
	 * (non-Javadoc)
	 * @see java.awt.Component#setBounds(int, int, int, int)
	 */
	public void setBounds(int x, int y, int width, int height) {
		// alte Gre merken
		int oldHeight = getSize().height;
		int oldWidth = getSize().width;
		super.setBounds(x, y, width, height);
		// Platz neu berechnen
		if ((oldHeight != 0) || (oldWidth != 0)) {
			int newSplitterPos = this.splitterPos;
			if ((width != oldWidth) && this.orientation == HORIZONTAL_SPLIT) {
				newSplitterPos = (int)(this.splitterResizeWeight * (width - oldWidth) + this.splitterPos);
			} else if ((height != oldHeight) && this.orientation == VERTICAL_SPLIT) {
				newSplitterPos = (int)(this.splitterResizeWeight * (height - oldHeight) + this.splitterPos);
			}
			setSplitterPos(newSplitterPos);
		}
	}
	
	/**
	 * fragt die die Regelung fr den Minimalplatz ab
	 * 
	 * @return die Regelung
	 */
	public int getConstraint() {
		return this.constraint;
	}
	
	/**
	 * legt die die Regelung fr den Minimalplatz fest
	 * 
	 * @param constraint die neue Regelung:
	 *        -1 -> Minimalgren der Komponenten
	 *        0  -> aus
	 *        >0 -> diese Minimalgre
	 */
	public void setConstraint(int constraint) {
		this.constraint = constraint;
	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
	 */
	public void mouseClicked(MouseEvent e) {
		// nix
	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
	 */
	public void mouseEntered(MouseEvent e) {
		// nix
	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
	 */
	public void mouseExited(MouseEvent e) {
		// nix
	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
	 */
	public void mousePressed(MouseEvent e) {
		if (e.getSource() != this.splitter) return;
		this.dragX = e.getX();
		this.dragY = e.getY();
		add(this.movingSplitter, MOVING_SPLITTER, 0);
		this.movingSplitter.setBounds(this.splitter.getBounds());
		
	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
	 */
	public void mouseReleased(MouseEvent e) {
		if (e.getSource() != this.splitter) return;
		int newSplitterPos;
		int max;
		if (this.orientation == HORIZONTAL_SPLIT) {
			newSplitterPos = this.splitterPos + e.getX() - this.dragX;
			max = this.getWidth();
		} else {
			newSplitterPos = this.splitterPos + e.getY() - this.dragY;
			max = this.getHeight();
		}
		if (newSplitterPos < SPLITTER_SIZE) {
			newSplitterPos = SPLITTER_SIZE;
		}
		if (newSplitterPos > max - SPLITTER_SIZE) {
			newSplitterPos = max - SPLITTER_SIZE;
		}
		setSplitterPos(newSplitterPos);
		super.remove(this.movingSplitter);
	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseMotionListener#mouseDragged(java.awt.event.MouseEvent)
	 */
	public void mouseDragged(MouseEvent e) {
		if (e.getSource() != this.splitter) return;
		/*
		if (orientation == HORIZONTAL_SPLIT) {
			setSplitterPos(splitterPos + e.getX() - dragX);
		} else {
			setSplitterPos(splitterPos + e.getY() - dragY);
		}
		*/
		Point location = this.splitter.getLocation();
		int min;
		int max;
		int size;
		if (this.orientation == HORIZONTAL_SPLIT) {
			location.x += e.getX() - this.dragX;
			if (this.constraint < 0) {
				min = (this.first != null ? this.first.getMinimumSize().width : 0) + SPLITTER_SIZE;
				max = (this.second != null ? this.second.getMinimumSize().width : 0) - SPLITTER_SIZE;
			} else {
				min = this.constraint + SPLITTER_SIZE;
				max = this.constraint - SPLITTER_SIZE;
			}
			size = getSize().width;
			if (min + max + 2 * SPLITTER_SIZE < size) {
				if (location.x < (min / 2) - SPLITTER_SIZE) {
					location.x = 0;
				} else if (location.x < min - SPLITTER_SIZE) {
					location.x = min - SPLITTER_SIZE;
				} else if (location.x > (size - (max / 2)) - SPLITTER_SIZE) {
					location.x = size - 2 * SPLITTER_SIZE;
				} else if (location.x > (size - max) - SPLITTER_SIZE) {
					location.x = size - max - SPLITTER_SIZE;
				}
			} else if (location.x < -SPLITTER_SIZE) {
				location.x = -SPLITTER_SIZE;
			} else if (location.x > size - SPLITTER_SIZE) {
				if (size > 2 * SPLITTER_SIZE) {
					location.x = size - 2 * SPLITTER_SIZE;
				} else {
					location.x = 0;
				}
			}
		} else {
			location.y += e.getY() - this.dragY;
			if (this.constraint < 0) {
				min = (this.first != null ? this.first.getMinimumSize().height : 0) + SPLITTER_SIZE;
				max = (this.second != null ? this.second.getMinimumSize().height : 0) - SPLITTER_SIZE;
			} else {
				min = this.constraint + SPLITTER_SIZE;
				max = this.constraint - SPLITTER_SIZE;
			}
			size = getSize().height;
			if (min + max + 2 * SPLITTER_SIZE < size) {
				if (location.y < (min / 2) - SPLITTER_SIZE) {
					location.y = 0;
				} else if (location.y < min - SPLITTER_SIZE) {
					location.y = min - SPLITTER_SIZE;
				} else if (location.y > (size - (max / 2)) - SPLITTER_SIZE) {
					location.y = size - 2 * SPLITTER_SIZE;
				} else if (location.y > (size - max) - SPLITTER_SIZE) {
					location.y = size - max - SPLITTER_SIZE;
				}
			} else if (location.y < -SPLITTER_SIZE) {
				location.y = -SPLITTER_SIZE;
			} else if (location.y > size - SPLITTER_SIZE) {
				if (size > 2 * SPLITTER_SIZE) {
					location.y = size - 2 * SPLITTER_SIZE;
				} else {
					location.y = 0;
				}
			}
		}
		this.movingSplitter.setLocation(location);
	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseMotionListener#mouseMoved(java.awt.event.MouseEvent)
	 */
	public void mouseMoved(MouseEvent e) {
		// nix
	}
}