/*
 * Erstellt: 06.07.2006
 * Gendert: $Date$
 * 
 * $Log$
 */
package ec4.tools.ui;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.util.Vector;


/**
 * ein LinePart, der Tabulatoren darstellt.
 * 
 * @author am
 * @version $Id$
 */
public class TabLinePart
implements LinePart {
	
	/**
	 * Die normale Breite eines Tabulators.
	 */
	private static final int TAB_WIDTH = 25;
	
	/**
	 * Die minimale Breite eines Tabulators.
	 */
	private static final int MIN_TAB_WIDTH = 5;
	
	/**
	 * die ColorTextArea, zu der dieser LinePart gehrt
	 */
	private ColorTextArea cTA;
	
	/**
	 * der Text, der diesem LinePart zugeordnet ist
	 */
	private String text;
	
	/**
	 * die Vordergrundfarbe dieses LineParts
	 */
	private Color fgColor;
	
	/**
	 * die Vordergrundfarbe dieses LineParts
	 */
	private Color bgColor;
	
	/**
	 * die Start-/Endzeile dieses LineParts
	 */
	private int line;
	
	/**
	 * die Startposition dieses LineParts
	 */
	private int startX;
	
	/**
	 * die Endposition dieses LineParts
	 */
	private int endX;
	
	/**
	 * Die Breite des ersten Tab-Zeichens.
	 */
	private int firstTabWidth;
	
	/**
	 * die Startposition der Markierung
	 */
	private int selStart;
	
	/**
	 * die Endposition der Markierung
	 */
	private int selEnd;
		
	/**
	 * erstellt einen neuen TabLinePart
	 * 
	 * @param cTA die ColorTextArea, zu der dieses Line-Objekt gehrt
	 * @param text der Text (ausschlielich Tablulatorzeichen '\t')
	 * @param fgColor die Vordergrundfarbe
	 * @param bgColor die Hintergrundfarbe
	 */
	TabLinePart(ColorTextArea cTA, String text,Color fgColor,  Color bgColor) {
		super();
		this.cTA = cTA;
		this.text = text;
		this.fgColor = fgColor;
		this.bgColor = bgColor;
	}
	
	/* (non-Javadoc)
	 * @see ec4.tools.ui.LinePart#getText()
	 */
	public String getText() {
		return this.text;
	}
	
	/* (non-Javadoc)
	 * @see ec4.tools.ui.LinePart#getURL()
	 */
	public String getURL() {
		return null;
	}
	
	/* (non-Javadoc)
	 * @see ec4.tools.ui.LinePart#paint(java.awt.Graphics, java.util.Vector)
	 */
	public void paint(Graphics g, Vector lineHeights) {
		// vertikale Position
		int y = 0;
		// Schriftmae holen
		FontHolder font = this.cTA.getFontHolder();
		// Positionsberechnung
		int height = font.getFontMetricLineHeight();
		// Hhen der vorhergehenden Zeilen und der eigenen addieren
		for (int i = 0; i <= this.line; i++) {
			y += ((Integer)lineHeights.elementAt(i)).intValue();
		}
		// keine Markierung
		if (this.selStart < 0 || this.selEnd < 0) {
			// Hintergrund zeichnen
			g.setColor(this.bgColor);
			g.fillRect(this.startX, y, this.firstTabWidth + (this.text.length() - 1) * TAB_WIDTH, height);
		} else {
			int selStartPos;
			int selWidth = (this.selEnd - this.selStart) * TAB_WIDTH;
			// Text vor Markierung
			if (this.selStart > 0) {
				// Hintergrund zeichnen
				g.setColor(this.bgColor);
				g.fillRect(this.startX, y, this.firstTabWidth + (this.selStart - 1) * TAB_WIDTH, height);
				selStartPos = this.startX + this.firstTabWidth + (this.selStart - 1) * TAB_WIDTH;
			} else {
				// erstes Tab separat
				g.setColor(this.fgColor);
				g.fillRect(this.startX, y, this.firstTabWidth, height);
				selStartPos = this.startX + this.firstTabWidth;
				selWidth -= this.firstTabWidth;
			}
			// Markierung zeichnen
			g.setColor(this.fgColor);
			g.fillRect(selStartPos, y, selWidth, height);
			// Text nach Markierung
			if (this.selEnd < this.text.length()) {
				// Hintergrund zeichnen
				g.setColor(this.bgColor);
				g.fillRect(selStartPos + selWidth, y, (this.text.length() + this.selEnd) * TAB_WIDTH, height);
			}
		}
	}
	
	/* (non-Javadoc)
	 * @see ec4.tools.ui.LinePart#sizeCalc(int, java.util.Vector)
	 */
	public int sizeCalc(int x, Vector lineHeights) {
		// diverse Daten holen:
		// linker Rand
		int leftBorder = this.cTA.getInsets().left;
		// Schriftdaten
		FontHolder font = this.cTA.getFontHolder();
		// Anfangsposition speichern
		this.line = lineHeights.size() - 1;
		this.startX = x;
		
		// Tab-Position berechnen
		this.firstTabWidth = TAB_WIDTH - (x - leftBorder) % TAB_WIDTH;
		if (this.firstTabWidth < MIN_TAB_WIDTH) {
			this.firstTabWidth += TAB_WIDTH;
		}
		// Zeilenhhe anpassen
		int lineHeight = ((Integer)lineHeights.elementAt(this.line)).intValue();
		lineHeight = Math.max(lineHeight, font.getFontMetricLineHeight());
		lineHeights.setElementAt(new Integer(lineHeight), this.line);
		// Endkoordinaten berechnen
		this.endX = this.startX + this.firstTabWidth + (this.text.length() - 1) * TAB_WIDTH;
		return this.endX;
	}
	
	/* (non-Javadoc)
	 * @see ec4.tools.ui.LinePart#getStringPos(java.awt.Point)
	 */
	public int getStringPos(Point p) {
		// Zeilennummer vergleichen
		if (p.y < this.line) {
			return ColorTextArea.ABOVE;
		} else if (p.y > this.line) {
			return ColorTextArea.BELOW;
		}
		// x-Koordinaten vergleichen
		if (p.x < this.startX) {
			return ColorTextArea.ABOVE;
		} else if (p.x > this.endX) {
			return ColorTextArea.BELOW;
		}
		if (p.x <= this.firstTabWidth / 2) {
			return 0;
		}
		int x = this.firstTabWidth + TAB_WIDTH / 2;
		int pos = 1;
		while (p.x > x && pos < this.text.length()) {
			pos++;
			x += TAB_WIDTH;
		}
		return pos;
	}
	
	/* (non-Javadoc)
	 * @see ec4.tools.ui.LinePart#getStartX()
	 */
	public int getStartX() {
		return this.startX;
	}
	
	/* (non-Javadoc)
	 * @see ec4.tools.ui.LinePart#getEndX()
	 */
	public int getEndX() {
		return this.endX;
	}
	
	/* (non-Javadoc)
	 * @see ec4.tools.ui.LinePart#getStartLine()
	 */
	public int getStartLine() {
		return this.line;
	}
	
	/* (non-Javadoc)
	 * @see ec4.tools.ui.LinePart#getEndLine()
	 */
	public int getEndLine() {
		return this.line;
	}
	
	/* (non-Javadoc)
	 * @see ec4.tools.ui.LinePart#setSelection(int, int)
	 */
	public boolean setSelection(int start, int end) {
		int oldSelStart = this.selStart;
		int oldSelEnd = this.selEnd;
		if ((start < 0) || (end < 0) || (start > end)
		 || (end > this.text.length())) {
			this.selStart = -1;
			this.selEnd = -1;
		} else {
			this.selStart = start;
			this.selEnd = end;
		}
		return (oldSelEnd != this.selEnd) || (oldSelStart != this.selStart);
	}
	
}

