/*
 * Created on 14.06.2004
 */
package ec4.tools.ui;

import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Point;
import java.util.Vector;

/**
 * ein LinePart, der formatierten Text darstellen kann
 * 
 * @author AM
 */
class TextLinePart implements LinePart {
	
	/**
	 * die ColorTextArea, zu der dieser LinePart gehrt
	 */
	private ColorTextArea cTA;
	
	/**
	 * der Text dieses LineParts
	 */
	private String text;
	
	/**
	 * true, wenn dieser LinePart fett dargestellt wird
	 */
	private boolean bold;
	
	
	/**
	 * true, wenn dieser LinePart kursiv dargestellt wird
	 */
	private boolean italic;
	
	/**
	 * true, wenn dieser LinePart unterstrichen dargestellt wird
	 */
	private boolean underline;
	
	/**
	 * die Vordergrundfarbe dieses LineParts
	 */
	private Color fgColor;
	
	/**
	 * die Vordergrundfarbe dieses LineParts
	 */
	private Color bgColor;
	
	/**
	 * die Startzeile dieses LineParts
	 */
	private int startLine;
	
	/**
	 * die Endzeile dieses LineParts
	 */
	private int endLine;
	
	/**
	 * die Startposition dieses LineParts
	 */
	private int startX;
	
	/**
	 * die Endposition dieses LinePArts
	 */
	private int endX;
	
	/**
	 * die URL, die mit diesem LinePart verknpft ist
	 */
	private String url;
	
	/**
	 * die Startposition der Markierung
	 */
	private int selStart;
	
	/**
	 * die Endposition der Markierung
	 */
	private int selEnd;
		
	/**
	 * Liste aller Zeilenumbrche und Start-X-Koordinaten innerhalb dieses
	 * LineParts (als int[])
	 */
	private Vector breaks;
	
	/**
	 * erstellt einen neuen TextLinePart
	 * 
	 * @param cTA die ColorTextArea, zu der dieses Line-Objekt gehrt
	 * @param text der angezeigte Text
	 * @param bold true, wenn der Text fett angezeigt werden soll
	 * @param italic true, wenn der Text kursiv angezeigt werden soll
	 * @param underline true, wenn der Text unterstrichen angezeigt werden soll
	 * @param fgColor die Vordergrundfarbe
	 * @param bgColor die Hintergrundfarbe
	 */
	TextLinePart(ColorTextArea cTA, String text, boolean bold, boolean italic,
	 boolean underline, Color fgColor, Color bgColor) {
		this(cTA, text, bold, italic, underline, fgColor, bgColor, null);
	}
	
	/**
	 * erstellt einen neuen TextLinePart
	 * 
	 * @param cTA die ColorTextArea, zu der dieses Line-Objekt gehrt
	 * @param text der angezeigte Text
	 * @param bold true, wenn der Text fett angezeigt werden soll
	 * @param italic true, wenn der Text kursiv angezeigt werden soll
	 * @param underline true, wenn der Text unterstrichen angezeigt werden soll
	 * @param fgColor die Vordergrundfarbe
	 * @param bgColor die Hintergrundfarbe
	 * @param url die mit diesem TextLinePart verknpfte URL
	 */
	TextLinePart(ColorTextArea cTA, String text, boolean bold, boolean italic,
	 boolean underline, Color fgColor, Color bgColor, String url) {
		super();
		this.cTA = cTA;
		this.text = text;
		this.bold = bold;
		this.italic = italic;
		this.underline = underline;
		this.fgColor = fgColor;
		this.bgColor = bgColor;
		this.url = url;
		this.selStart = -1;
		this.selEnd = -1;
	}
	
	/* (non-Javadoc)
	 * @see ec4.tools.ui.LinePart#getText()
	 */
	public String getText() {
		return this.text;
	}
	
	/* (non-Javadoc)
	 * @see ec4.tools.ui.LinePart#getURL()
	 */
	public String getURL() {
		return this.url;
	}
	
	/* (non-Javadoc)
	 * @see ec4.tools.ui.LinePart#paint(java.awt.Graphics)
	 */
	public void paint(Graphics g, Vector lineHeights) {
		// vertikale Position
		int y = 0;
		int x = this.startX;
		// Schriftmae holen
		FontHolder font = this.cTA.getFontHolder();
		int combinedBoldItalic = FontHolder.combine(this.bold, this.italic);
		FontMetrics fm = font.getFontMetrics(combinedBoldItalic);
		int descent = font.getFontMetricDescent(combinedBoldItalic);
		// Schrift setzen
		g.setFont(font.getUsedFont(combinedBoldItalic));
		// Hhen der vorhergehenden Zeilen und der eigenen addieren
		for (int i = 0; i <= this.startLine; i++) {
			y += ((Integer)lineHeights.elementAt(i)).intValue();
		}
		// Abstrich wieder abziehen
		y -= descent;
		// nur 1 Zeile
		if (this.breaks == null || this.breaks.size() == 0) {
			// zeichnen
			print(g, x, y, 0, this.text.length(), font);
			// x-Koordinate weiterschieben
			x += fm.stringWidth(this.text);
			return;
		}
		// 1. Zeile zeichnen
		print(g, x, y, 0, ((int[])this.breaks.elementAt(0))[0], font);
		// y erhhen
		y += ((Integer)lineHeights.elementAt(this.startLine + 1)).intValue();
		// Folgezeilen zeichnen
		for (int line = 1; line < this.endLine - this.startLine; line++) {
			// Zeilenanfang setzen
			int[] currentBreak = (int[])this.breaks.elementAt(line - 1);
			x = currentBreak[1];
			print(g, x, y, ((int[])this.breaks.elementAt(line - 1))[0],
			 ((int[])this.breaks.elementAt(line))[0], font
			);
			// y erhhen
			y += ((Integer)lineHeights.elementAt(this.startLine + line + 1)).intValue();
		}
		x = ((int[])this.breaks.lastElement())[1];
		// letzte Zeile zeichnen
		print(g, x, y, ((int[])this.breaks.lastElement())[0],
			this.text.length(), font
		);
	}
	
	/* (non-Javadoc)
	 * @see ec4.tools.ui.LinePart#sizeCalc()
	 */
	public int sizeCalc(int x, Vector lineHeights) {
		// Anfangsposition speichern
		this.startLine = lineHeights.size() - 1;
		this.startX = x;
		// Umbruchliste lschen
		if (this.breaks != null) {
			this.breaks.removeAllElements();
		}
		// diverse Daten holen:
		// linker Rand
		int leftBorder = this.cTA.getInsets().left;
		// Einrckung der Folgezeilen (incl. linkem Rand)
		int subIndent = leftBorder + this.cTA.getSubsequentIndent();
		// Abstand rechter Rand vom Ursprung
		int width = this.cTA.getSize().width - this.cTA.getInsets().right;
		// Schriftdaten
		FontHolder font = this.cTA.getFontHolder();
		FontMetrics fm = font.getFontMetrics(FontHolder.combine(this.bold, this.italic));
		// aktuelle Zeile
		int currentLine = this.startLine;
		// Sicherheitskopie des Strings
		String s = this.text;
		// Position im Orginalstring
		int pos = 0;
		// Ende der aktuellen Zeile bestimmen
		this.endX = x + fm.stringWidth(s);
		// solange die Zeile zu lang
		while ((this.endX > width) || (s.indexOf('\n') >= 0)) {
			int breakPos = 0;
			int i = 0;
			// suche geeignetes Leerzeichen zum Umbrechen
			while ((i = spaceIndex(s, i)) >= 0) {
				int newX = x + fm.stringWidth(s.substring(0, i));
				if (newX > width) {
					// Umbruch gefunden (=> breakPos)
					break;
				}
				// i auf nchstes Zeichen setzen
				i++;
				// folgende Leerzeichen berspringen
				while ((s.length() > i) && Character.isSpaceChar(s.charAt(i))/* && (s.charAt(i) != '\n')*/) {
					i++;
				}
				// i merken
				breakPos = i;
			}// for
			// nach Zeilenumbrchen suchen
			i = lineBreakIndex(s);
			if ((i != -1) && (i < breakPos)) {
				// i auf nchstes Zeichen setzen
				i++;
//				// folgende Leerzeichen berspringen
//				while ((s.length() > i) && Character.isSpaceChar(s.charAt(i)) && (s.charAt(i) != '\n')) {
//					i++;
//				}
				// i merken
				breakPos = i;
			}
			// Umbruch gleich am Anfang?
			if (breakPos == 0) {
				// x noch am Anfang der Zeile
				if (x <= subIndent) {
					// Zeilenumbruch am Leerzeichen unmglich => Umbruch
					// erzwingen
					for (breakPos = 1, i = 1; i < s.length() - 1; i++) {
						// solange, bis Zeilenende erreicht
						int newX = x + fm.stringWidth(s.substring(0, i + 1));
						if (newX > width) {
							breakPos = i;
							break;
						}//if
					}//for
				// Umbruch am ersten Zeichen des Strings?
				} else if (pos == 0) {
					// auf neuer Zeile beginnen
					this.startLine++;
					currentLine++;
					x = subIndent;
					this.startX = x;
					lineHeights.addElement(new Integer(0));
					// Ende der aktuellen Zeile bestimmen
					this.endX = x + fm.stringWidth(s);
					continue;
				}//if
			}//if
			// Umbruch im gltigen Bereich?
			if ((breakPos > 0) && ((breakPos < s.length()) || ((breakPos == s.length()) && (s.charAt(s.length() - 1) == '\n')))) {
				// in Zeilenhhenliste einfgen
				int lineHeight
				 = ((Integer)lineHeights.elementAt(currentLine)).intValue();
				lineHeight
				 = Math.max(lineHeight, font.getFontMetricLineHeight());
				lineHeights.setElementAt(new Integer(lineHeight), currentLine);
				// x an Zeilenanfang setzen (auf NewLine aufpassen)
				i = s.indexOf('\n');
				if ((i != -1) && (i < breakPos)) {
					x = leftBorder;
				} else {
					x = subIndent;
				}
				// in Umbruch-Liste einfgen
				if (this.breaks == null) {
					this.breaks = new Vector();
				}
				this.breaks.addElement(new int[] {breakPos + pos, x});
				// Zeilenzhler erhhen
				currentLine++;
				// s krzen
				s = s.substring(breakPos);
				pos += breakPos;
				// neue Zeile
				lineHeights.addElement(new Integer(0));
				// Ende der neuen Zeile bestimmen
				this.endX = x + fm.stringWidth(s);
			// Umbruch nach letztem Zeichen vermeiden (Zeilenumbruch siehe oben)
			} else if (breakPos == s.length()) {
				break;
			}//if
		}//while (xEnd > width)
		// Beachtung vom Rest
		// in Zeilenhhenliste einfgen
		int lineHeight
		 = ((Integer)lineHeights.elementAt(currentLine)).intValue();
		lineHeight
		 = Math.max(lineHeight, font.getFontMetricLineHeight());
		lineHeights.setElementAt(new Integer(lineHeight), currentLine);
		// Ende der neuen Zeile bestimmen
		this.endX = x + fm.stringWidth(s);
		this.endLine = currentLine;
		return this.endX;
	}
	
	/* (non-Javadoc)
	 * @see ec4.tools.ui.LinePart#getStringPos(java.awt.Point)
	 */
	public int getStringPos(Point p) {
		// Zeilennummer vergleichen
		if (p.y < this.startLine) {
			return ColorTextArea.ABOVE;
		} else if (p.y > this.endLine) {
			return ColorTextArea.BELOW;
		}
		// nur 1 Zeile
		if (this.startLine == this.endLine) {
			// x-Koordinaten vergleichen
			if (p.x < this.startX) {
				return ColorTextArea.ABOVE;
			} else if (p.x > this.endX) {
				return ColorTextArea.BELOW;
			}
			// Grenberechnung
			return getStringPos(this.text, p.x - this.startX);
		}
		// diverse Daten holen:
		// 1. Zeile
		if (p.y == this.startLine) {
			// Anfangs-x-Koordinate vergleichen
			if (p.x < this.startX) {
				return ColorTextArea.ABOVE;
			}
			// Grenberechnung
			return getStringPos(this.text.substring(0,
				((int[])this.breaks.firstElement())[0]
			), p.x - this.startX);
		} else if (p.y == this.endLine) {
			// End-x-Koordinate vergleichen
			if (p.x > this.endX) {
				return ColorTextArea.BELOW;
			}
			// Grenberechnung
			return getStringPos(
				this.text.substring(((int[])this.breaks.lastElement())[0]),
				p.x - ((int[])this.breaks.lastElement())[1]
			) + ((int[])this.breaks.lastElement())[0];
		} else {
			// Grenberechnung
			return getStringPos(this.text.substring(
				((int[])this.breaks.elementAt(p.y - this.startLine - 1))[0],
				((int[])this.breaks.elementAt(p.y - this.startLine))[0]
			), p.x - ((int[])this.breaks.elementAt(p.y - this.startLine - 1))[1])
			 + ((int[])this.breaks.elementAt(p.y - this.startLine - 1))[0];
		}
	}
	
	/* (non-Javadoc)
	 * @see ec4.tools.ui.LinePart#getStartX()
	 */
	public int getStartX() {
		return this.startX;
	}
	
	/* (non-Javadoc)
	 * @see ec4.tools.ui.LinePart#getEndX()
	 */
	public int getEndX() {
		return this.endX;
	}
	
	/* (non-Javadoc)
	 * @see ec4.tools.ui.LinePart#getStartLine()
	 */
	public int getStartLine() {
		return this.startLine;
	}
	
	/* (non-Javadoc)
	 * @see ec4.tools.ui.LinePart#getEndLine()
	 */
	public int getEndLine() {
		return this.endLine;
	}
	
	/**
	 * berechnet den Index des nchsten Leerzeichens im String
	 * 
	 * @param s der zu durchsuchende String
	 * @param i die Position, ab der gesucht wird
	 * @return die Position des nchsten Leerzeichens oder -1, wenn keins
	 *         vorhanden
	 */
	private int spaceIndex(String s, int i) {
		int len = s.length();
		for (int j = i; j < len; j++)
			if (Character.isSpaceChar(s.charAt(j)) || (s.charAt(j) == '\n'))
				return (j);
		return (-1);
	}
	
	/**
	 * berechnet den Index des ersten Zeilenumbruches im String
	 * 
	 * @param s der zu durchsuchende String
	 * @return die Position des nchsten Leerzeichens oder -1, wenn keins
	 *         vorhanden
	 */
	private int lineBreakIndex(String s) {
		int len = s.length();
		for (int j = 0; j < len; j++) {
			if (s.charAt(j) == '\n') {
				return (j);
			}
		}
		return (-1);
	}
	
	/**
	 * gibt einen zusammenhngenden String aus
	 * 
	 * @param g das Graphics-Objekt, auf das ausgegeben wird
	 * @param x die x-Koordinate, ab der ausgegeben wird
	 * @param y die y-Koordinate, ab der ausgegeben wird
	 * @param from die Position, ab der Ausgegeben wird
	 * @param to die Position, bis zu der ausgegeben wird
	 * @param f der FontHolder (um den nicht nochmal holen zu mssen)
	 */
	private void print(Graphics g, int x, int y, int from, int to, FontHolder f) {
		// Fontdaten beschaffen
		int cBI = FontHolder.combine(this.bold, this.italic);
		int ascent = f.getFontMetricAscent(cBI);
		int descent = f.getFontMetricDescent(cBI);
		int leading = f.getFontMetricLeading();
		int spaceWidth = f.getFontMetricSpaceWidth();
		// Markierungsfarben
		Color selectBg = this.cTA.getSelectionBgColor();
		Color selectFg = this.cTA.getSelectionFgColor();
		// nichts markiert
		if ((from >= this.selEnd) || (to < this.selStart)) {
			// Text vorbereiten
			String s = this.text.substring(from, to);
			if (s.endsWith("\n")) { //$NON-NLS-1$
				s = s.substring(0, s.length() - 1);
			}
			// Hintergrund zeichnen
			g.setColor(this.bgColor);
			int width = f.getFontMetrics(cBI).stringWidth(s);
			g.fillRect(x, y - ascent - leading, width + 1, ascent + descent + leading);
			// Vordergrund zeichnen
			g.setColor(this.fgColor);
			g.drawString(s, x, y);
			if (this.underline) {
				// Position des Unterstriches
				int below = descent / 4;
				if (below < 1) {
					below = 1;
				}
				// Zeichnen
				g.drawLine(x, y + below, x + width, y + below);
				if (this.bold) {
					g.drawLine(x, y + below + 1, x + width, y + below + 1);
				}
			}
		} else {
			String s;
			int width;
			// normaler Text am Anfang:
			if (from < this.selStart) {
				s = this.text.substring(from, this.selStart);
				if (s.endsWith("\n")) { //$NON-NLS-1$
					s = s.substring(0, s.length() - 1);
				}
				// Hintergrund zeichnen
				g.setColor(this.bgColor);
				width = f.getFontMetrics(cBI).stringWidth(s);
				g.fillRect(x, y - ascent - leading, width + 1, ascent + descent + leading);
				// Vordergrund zeichnen
				g.setColor(this.fgColor);
				g.drawString(s, x, y);
				if (this.underline) {
					// Position des Unterstriches
					int below = descent / 4;
					if (below < 1) {
						below = 1;
					}
					// Zeichnen
					g.drawLine(x, y + below, x + width, y + below);
					if (this.bold) {
						g.drawLine(x, y + below + 1, x + width, y + below + 1);
					}
				}
				x += width;
			}
			// Markierung:
			s = this.text.substring(Math.max(from, this.selStart), Math.min(to, this.selEnd));
			if (s.length() > 0) {
				// Hintergrund zeichnen
				g.setColor((selectBg == null ? this.fgColor : selectBg));
				width = f.getFontMetrics(cBI).stringWidth(s);
				if (s.endsWith("\n")) { //$NON-NLS-1$
					s = s.substring(0, s.length() - 1);
					width += spaceWidth;
				}
				g.fillRect(x, y - ascent - leading, width + 1, ascent + descent + leading);
				// Vordergrund zeichnen
				g.setColor((selectFg == null ? this.bgColor : selectFg));
				g.drawString(s, x, y);
				if (this.underline) {
					// Position des Unterstriches
					int below = descent / 4;
					if (below < 1) {
						below = 1;
					}
					// Zeichnen
					g.drawLine(x, y + below, x + width, y + below);
					if (this.bold) {
						g.drawLine(x, y + below + 1, x + width, y + below + 1);
					}
				}
				x += width;
			}
			// normaler Text am Ende
			if (to > this.selEnd) {
				s = this.text.substring(this.selEnd, to);
				if (s.endsWith("\n")) { //$NON-NLS-1$
					s = s.substring(0, s.length() - 1);
				}
				// Hintergrund zeichnen
				g.setColor(this.bgColor);
				width = f.getFontMetrics(cBI).stringWidth(s);
				g.fillRect(x, y - ascent - leading, width + 1, ascent + descent + leading);
				// Vordergrund zeichnen
				g.setColor(this.fgColor);
				g.drawString(s, x, y);
				if (this.underline) {
					// Position des Unterstriches
					int below = descent / 4;
					if (below < 1) {
						below = 1;
					}
					// Zeichnen
					g.drawLine(x, y + below, x + width, y + below);
					if (this.bold) {
						g.drawLine(x, y + below + 1, x + width, y + below + 1);
					}
				}
			}
		}// if
	}//print(...)
	
	/**
	 * berechnet die Zeichenposition in einem String anhand der Pixelposition
	 * 
	 * @param s der zu durchsuchende String
	 * @param pos die Pixelposition
	 * @return die Zeichenposition
	 */
	private int getStringPos(String s, int pos) {
		// Anfang?
		if (pos <= 0) {
			return 0;
		}
		// Schriftdaten
		FontHolder font = this.cTA.getFontHolder();
		FontMetrics fm = font.getFontMetrics(FontHolder.combine(this.bold, this.italic));
		// suchen
		int len = s.length();
		int width = 0;
		int oldWidth = 0; 
		for (int i = 0; i < len; i++) {
			width = fm.stringWidth(s.substring(0, i + 1));
			if (oldWidth + (width - oldWidth) / 2 > pos) {
				return i;
			}
			oldWidth = width;
		}
		return len;
	}

	/* (non-Javadoc)
	 * @see ec4.tools.ui.LinePart#setSelection(int, int)
	 */
	public boolean setSelection(int start, int end) {
		int oldSelStart = this.selStart;
		int oldSelEnd = this.selEnd;
		if ((start < 0) || (end < 0) || (start > end)
		 || (end > this.text.length())) {
			this.selStart = -1;
			this.selEnd = -1;
		} else {
			this.selStart = start;
			this.selEnd = end;
		}
		return (oldSelEnd != this.selEnd) || (oldSelStart != this.selStart);
	}

}
