/*
 * Created on 13.07.2004
 */
package ec4.tools.version;

import java.awt.Component;
import java.awt.Graphics;

/**
 * kapselt alle Versionsspezifischen Sachen
 * 
 * @author AM
 */
public abstract class VersionController {
	
	/**
	 * Klassenname der 1.1-kompatiblem Version
	 */
	private static final String CLASSNAME_11 = "ec4.tools.version.VersionController11"; //$NON-NLS-1$
	
	/**
	 * Klassenname der MS-kompatiblen Version
	 */
	private static final String CLASSNAME_MS = "ec4.tools.version.VersionControllerMS"; //$NON-NLS-1$
	
	/**
	 * Klassenname der 1.2-kompatiblen Version
	 */
	private static final String CLASSNAME_12 = "ec4.tools.version.VersionController12"; //$NON-NLS-1$
	
	/**
	 * Klassenname der 1.4-kompatiblen Version
	 */
	private static final String CLASSNAME_14 = "ec4.tools.version.VersionController14"; //$NON-NLS-1$
	
	/**
	 * Klassenname der Kaffe-kompatiblen Version
	 */
	private static final String CLASSNAME_KAFFE = "ec4.tools.version.VersionControllerKaffe"; //$NON-NLS-1$
	
	/**
	 * die aktuelle Instanz
	 */
	private static VersionController instance;
	
	/**
	 * Konstruktor
	 */
	protected VersionController() {
		super();
	}
	
	/**
	 * setzt das Antialiasing-Flag in g (sofern mglich
	 * 
	 * @param g das zu bearbeitende Grafikobjekt
	 * @param value true, wenn antialiasing benutzt werden soll
	 */
	public abstract void setAntialiasing(Graphics g, boolean value);
	
	/**
	 * fgt c einen MouseWheelListener hinzu (sofern mglich)
	 * 
	 * @param c die Komponente
	 * @param l der Listener
	 */
	public abstract void buildMouseWheelListener(Component c, MouseWheelListener l);
	
	/**
	 * sichert Netzwerkzugriffe zu (sofern ntig)
	 */
	public abstract void assertNetworkPerm();
	
	/**
	 * holt eine Instanz dieser Klasse, die der aktuellen VM angepasst ist
	 * 
	 * @return die Instanz
	 */
	public static VersionController getInstance()  {
		if (instance != null) {
			return instance;
		}
		Class clazz = null;
		
		String vendor = System.getProperty("java.vendor"); //$NON-NLS-1$
		String version = System.getProperty("java.version"); //$NON-NLS-1$
		int pointPos = version.indexOf('.');
		if (pointPos >= 0) {
			pointPos = version.indexOf('.', pointPos + 1);
		}
		if (pointPos >= 0) {
			version = version.substring(0, pointPos);
		}
		double ver = new Double(version).doubleValue();
		
		// Spezialflle: Microsoft, Kaffe
		if ((vendor.equals("Microsoft Corp.")) && (clazz == null)) { //$NON-NLS-1$
			try {
				clazz = Class.forName(CLASSNAME_MS);
			} catch (Throwable e) {
				//ignoriert
			}
		}
		if ((vendor.equals("Kaffe.org project")) && (clazz == null)) { //$NON-NLS-1$
			String rtVersion = System.getProperty("java.runtime.version"); //$NON-NLS-1$
			pointPos = rtVersion.indexOf('.');
			if (pointPos >= 0) {
				pointPos = rtVersion.indexOf('.', pointPos + 1);
			}
			if (pointPos >= 0) {
				rtVersion = rtVersion.substring(0, pointPos);
			}
			double rtVer = new Double(rtVersion).doubleValue();
			
			if (rtVer >= 1.0) {
				try {
					clazz = Class.forName(CLASSNAME_KAFFE);
				} catch (Throwable e) {
					//ignoriert
				}
			}
		}
		
		if ((ver >= 1.4) && (clazz == null)) {
			try {
				clazz = Class.forName(CLASSNAME_14);
			} catch (Throwable e) {
				//ignoriert
			}
		}
		if ((ver >= 1.2) && (clazz == null)) {
			try {
				clazz = Class.forName(CLASSNAME_12);
			} catch (Throwable e) {
				//ignoriert
			}
		}
		if (clazz == null) {
			try {
				clazz = Class.forName(CLASSNAME_11);
			} catch (Throwable e) {
				//ignoriert
			}
		}
		if (clazz == null) {
			throw new Error("kann keine Klasse finden"); //$NON-NLS-1$
		}
		try {
			instance = (VersionController)clazz.newInstance();
		} catch (InstantiationException e) {
			throw new Error("kann Klasse nicht laden"); //$NON-NLS-1$
		} catch (IllegalAccessException e) {
			throw new Error("kann Klasse nicht laden"); //$NON-NLS-1$
		}
		return instance;
	}
}
